/*---------------------------------------------------------------------------*
  Project:  iWnn
  File:     iwnn.h

  Copyright (C)2009-2012 Nintendo Co., Ltd.  All rights reserved.

  These coded instructions, statements, and computer programs contain
  proprietary information of Nintendo of America Inc. and/or Nintendo
  Company Ltd., and are protected by Federal copyright law.  They may
  not be disclosed to third parties or copied or duplicated in any form,
  in whole or in part, without the prior written consent of Nintendo.
  
 $Revision: $
 *---------------------------------------------------------------------------*/
#ifndef IWNN_H
#define IWNN_H

#include <nn/middleware.h>
#include <nn/types.h>
#include "./engine/nj_lib.h"
#include "./engine/nj_err.h"
#include "./engine/nj_dic.h"
#include "./engine/nj_dicif.h"
#include "./engine/nj_ext.h"    
#include "./engine/ex_giji.h"
#include "./engine/ex_predg.h"
#include "./engine/ex_filter.h"
#include "./engine/ex_nmfgiji.h"
#include "./engine/ex_nmcgiji.h"





typedef NJ_FILE           IWNN_FILE;         //File type
typedef NJ_DIC_HANDLE     IWNN_DIC_HANDLE;   //Dictionary handle
typedef NJ_HINDO IWNN_FREQUENCY;               //Frequency data type (data types limited to within the engine)
typedef NJ_DIC_FREQ IWNN_DIC_FREQUENCY;       //Dictionary frequency information definitions


/******************************************/
/*   Defined values (nj_ext.h)  Note: Rename support code */
/******************************************/
#define PSEUDO_GET_SCORE(len) GIJI_GETSCORE(len)

/******************************************/
/*   Defined values (ndapi.cpp)  Note: new additions */
/******************************************/
#define NJ_CREATE_DIC_TYPE_USER             0           //Create user dictionary
#define NJ_CREATE_DIC_TYPE_LEARN_AWNN       1           //Create learning dictionary (AWnn-compatible)
#define NJ_CREATE_DIC_TYPE_LEARN            2           //Create learning dictionary
#define NJ_CREATE_DIC_TYPE_USER_ADDITIONAL  3           //Create user dictionary (including additional information)
#define NJ_CREATE_DIC_TYPE_LEARN_ADDITIONAL 4           //Create learning dictionary (including additional information)

/******************************************/
/*   Defined values (ex_giji.h)  Note: Rename support */
/******************************************/
//Definition scope for pseudo-candidate types for application extensions (maximum values)
#define NJ_PSEUDO_TYPE_CUST_MAX    NJ_GIJI_TYPE_CUST_MAX

//Definition scope for pseudo-candidate types for application extensions (minimum values)
#define NJ_PSEUDO_TYPE_CUST_MIN    NJ_GIJI_TYPE_CUST_MIN 

/* Please see man pages for details 
 
 
 
 */
#define NJ_PSEUDO_SET_MAX    NJ_GIJISET_MAX

#define NJ_TYPE_HALF_DATE_DD    NJ_TYPE_HAN_DATE_DD               //Pseudo-candidate type: single-byte character date (D day)
#define NJ_TYPE_HALF_DATE_MD    NJ_TYPE_HAN_DATE_MD               //Pseudo-candidate type: single-byte character date (M-month, D-day)
#define NJ_TYPE_HALF_DATE_MD_SYMBOL    NJ_TYPE_HAN_DATE_MD_SYM    //Pseudo-candidate type: single-byte character date (M/D)
#define NJ_TYPE_HALF_DATE_MDD    NJ_TYPE_HAN_DATE_MDD             //Pseudo-candidate type: single-byte character date (M-month, DD-day)
#define NJ_TYPE_HALF_DATE_MDD_SYMBOL    NJ_TYPE_HAN_DATE_MDD_SYM  //Pseudo-candidate type: single-byte character date (M/DD)
#define NJ_TYPE_HALF_DATE_MM    NJ_TYPE_HAN_DATE_MM               //Pseudo-candidate type: single-byte character date (M month)
#define NJ_TYPE_HALF_DATE_MMD    NJ_TYPE_HAN_DATE_MMD             //Pseudo-candidate type: single-byte character date (MM-month, D-day) 
#define NJ_TYPE_HALF_DATE_MMD_SYMBOL    NJ_TYPE_HAN_DATE_MMD_SYM  //Pseudo-candidate type: single-byte character date (MM/D)
#define NJ_TYPE_HALF_DATE_MMDD    NJ_TYPE_HAN_DATE_MMDD           //Pseudo-candidate type: single-byte character date (MM-month, DD-day)
#define NJ_TYPE_HALF_DATE_MMDD_SYMBOL    NJ_TYPE_HAN_DATE_MMDD_SYM //Pseudo-candidate type: single-byte character date (MM/DD)
#define NJ_TYPE_HALF_DATE_YYYY    NJ_TYPE_HAN_DATE_YYYY            //Pseudo candidate type: single-byte character date (YYYY year)
#define NJ_TYPE_HALF_ALPHAMERIC_CAPITAL    NJ_TYPE_HAN_EIJI_CAP    //Pseudo-candidate type: single-byte alphabetic character (Initial Cap Only)
#define NJ_TYPE_HALF_ALPHAMERIC_LOWER    NJ_TYPE_HAN_EIJI_LOWER    //Pseudo-candidate type: single-byte alphabetic character (all lowercase)
#define NJ_TYPE_HALF_ALPHAMERIC_UPPER    NJ_TYPE_HAN_EIJI_UPPER    //Pseudo-candidate type: single-byte alphabetic character (ALL UPPERCASE) 
#define NJ_TYPE_HALF_NUMERIC_COMMA    NJ_TYPE_HAN_SUUJI_COMMA      //Pseudo-candidate type: single-byte numerals (delimited with commas)
#define NJ_TYPE_HALF_NUMERIC_READING    NJ_TYPE_HAN_SUUJI_YOMI     //Pseudo-candidate type: convert single-byte numeric character reading string to numerals
#define NJ_TYPE_HALF_TIME_HH    NJ_TYPE_HAN_TIME_HH                //Pseudo-candidate type: single-byte character time (H hour)
#define NJ_TYPE_HALF_TIME_HHM    NJ_TYPE_HAN_TIME_HHM              //Pseudo-candidate type: single-byte character time (HH-hour, M-minute)
#define NJ_TYPE_HALF_TIME_HHMM    NJ_TYPE_HAN_TIME_HHMM            //Pseudo-candidate type: single-byte character time (HH-hour, MM-minute)
#define NJ_TYPE_HALF_TIME_HHMM_SYMBOL    NJ_TYPE_HAN_TIME_HHMM_SYM //Pseudo-candidate type: single-byte character time (HH:MM)
#define NJ_TYPE_HALF_TIME_HM    NJ_TYPE_HAN_TIME_HM                //Pseudo-candidate type: single-byte character time (H:M)
#define NJ_TYPE_HALF_TIME_HMM    NJ_TYPE_HAN_TIME_HMM              //Pseudo-candidate type: single-byte character time (H-hour, MM-minute)
#define NJ_TYPE_HALF_TIME_HMM_SYMBOL    NJ_TYPE_HAN_TIME_HMM_SYM   //Pseudo-candidate type: single-byte character time (H:MM)
#define NJ_TYPE_HALF_TIME_MM    NJ_TYPE_HAN_TIME_MM                //Pseudo-candidate type: single-byte character time (M minute)
#define NJ_TYPE_FULL_ALPHAMERIC_CAPITAL    NJ_TYPE_ZEN_EIJI_CAP    //Pseudo-candidate type: double-byte alphabetic character (initial cap only)
#define NJ_TYPE_FULL_ALPHAMERIC_LOWER    NJ_TYPE_ZEN_EIJI_LOWER    //Pseudo-candidate type: double-byte alphabetic character (all lowercase)
#define NJ_TYPE_FULL_ALPHAMERIC_UPPER    NJ_TYPE_ZEN_EIJI_UPPER    //Pseudo-candidate type: double-byte alphabetic character (ALL UPPERCASE)
#define NJ_TYPE_FULL_KANJI_NUMERIC_GRADE_READING    NJ_TYPE_ZEN_KANSUUJI_KURAI_YOMI //Pseudo-candidate type: convert double-byte numeric character reading string to kanji numbers (rank)
#define NJ_TYPE_FULL_KANJI_NUMERIC_READING    NJ_TYPE_ZEN_KANSUUJI_YOMI //Pseudo-candidate type: convert double-byte numeric character reading string to kanji numbers
#define NJ_TYPE_FULL_NUMERIC_READING    NJ_TYPE_ZEN_SUUJI_YOMI           //Pseudo-candidate type: convert double-byte numeric character reading string to numerals

/******************************************/
/*   Defined values (nj_dicif.h)  Note: Rename support */
/******************************************/
#define NJG_OPERATION_ADD_WORD    NJG_OP_ADD_WORD             //Word registration
#define NJG_OPERATION_DELETE_WORD    NJG_OP_DEL_WORD          //Delete word
#define NJG_OPERATION_GET_ADDITIONAL    NJG_OP_GET_ADDITIONAL //Get additional information string
#define NJG_OPERATION_GET_STR    NJG_OP_GET_STRING            //Get notation string
#define NJG_OPERATION_GET_STROKE    NJG_OP_GET_STROKE         //Get string
#define NJG_OPERATION_GET_WORD_INFO    NJG_OP_GET_WORD_INFO   //Get word information
#define NJG_OPERATION_LEARN    NJG_OP_LEARN                   //Learn words
#define NJG_OPERATION_SEARCH    NJG_OP_SEARCH                 //First search
#define NJG_OPERATION_SEARCH_NEXT    NJG_OP_SEARCH_NEXT       //Subsequent candidate searches
#define NJG_OPERATION_UNDO_LEARN    NJG_OP_UNDO_LEARN         //Undo learn word

/******************************************/
/*   Defined values (nj_err.h)  Note: Rename support */
/******************************************/
#define NJ_ERR_AREA_SIZE_INVALID    NJ_ERR_AREASIZE_INVALID                            //An invalid dictionary size was specified
#define NJ_ERR_CANNOT_GET_QUEUE    NJ_ERR_CANNOT_GET_QUE                               //Could not get word information
#define NJ_ERR_DIC_FREQUENCY_INVALID    NJ_ERR_DIC_FREQ_INVALID                        //Invalid dictionary frequency setting 
#define NJ_ERR_PARTS_OF_SPEECH_GROUP_INVALID    NJ_ERR_HINSI_GROUP_INVALID             //An invalid part of speech group was specified
#define NJ_ERR_NO_PARTS_OF_SPEECH    NJ_ERR_NO_HINSI                                   //Could not get part of speech number for conversion
#define NJ_ERR_NO_RULE_DIC    NJ_ERR_NO_RULEDIC                                        //A rule dictionary was not specified 
#define NJ_ERR_PARAM_ENVIRONMENT_NULL    NJ_ERR_PARAM_ENV_NULL                         //NULL was specified for external variables (IWNN_CLASS) 
#define NJ_ERR_PARAM_DIVISION    NJ_ERR_PARAM_KUGIRI                                   //An invalid delimiter position was specified 
#define NJ_ERR_PARAM_NRM_NULL    NJ_ERR_PARAM_NORMAL_NULL                              //NULL was specified for the normalized search results storage buffer 
#define NJ_ERR_PARAM_READING_NULL    NJ_ERR_PARAM_YOMI_NULL                            //NULL was specified for the reading string  
#define NJ_ERR_PARAM_READING_SIZE    NJ_ERR_PARAM_YOMI_SIZE                            //An invalid reading string length was specified
#define NJ_ERR_PROTECTION_ERR    NJ_ERR_PROTECTION_ERROR                               //There was an attempt to access a protected area in the learning dictionary.
#define NJ_ERR_USER_CANDIDATE_INVALID    NJ_ERR_USER_KOUHO_INVALID                     //An invalid candidate string was specified during word registration
#define NJ_ERR_USER_READING_INVALID    NJ_ERR_USER_YOMI_INVALID                        //An invalid reading string was specified during word registration
#define NJ_ERR_READING_TOO_LONG    NJ_ERR_YOMI_TOO_LONG                                //A reading string with a length exceeding NJ_MAX_LEN was specified

#define NJ_FUNC_NJC_GET_PSEUDO_RESULT  NJ_FUNC_NJC_GET_GIJI_RESULT 
/******************************************/
/*   Macro (nj_err.h)  Note: Rename support */
/******************************************/
/* Please see man pages for details 
 
 
 
 
 
 */
#define NJ_SET_ERR_VALUE(x,y)    NJ_SET_ERR_VAL(x,y)

/******************************************/
/*   Defined values (nj_dic.h)  Note: Rename support */
/******************************************/
#define NJ_PARTS_OF_SPEECH_V2_FORE             NJ_HINSI_V2_F                      //Part of speech type  End of phrase
#define NJ_PARTS_OF_SPEECH_PSEUDO_FORE         NJ_HINSI_GIJI_F                    //Part of speech type  Pseudo (fore)
#define NJ_PARTS_OF_SPEECH_PSEUDO_BACK         NJ_HINSI_GIJI_B                    //Part of speech type  Pseudo (back)
#define NJ_PARTS_OF_SPEECH_NUMERIC_BACK        NJ_HINSI_SUUJI_B                   //Part of speech type  Number (back)
#define NJ_PARTS_OF_SPEECH_TOP_SENTENCE_BACK   NJ_HINSI_BUNTOU_B                  //Part of speech type  Start of sentence
#define NJ_PARTS_OF_SPEECH_SINGLE_KANJI_FORE   NJ_HINSI_TANKANJI_F                //Part of speech type  Single kanji (fore)
#define NJ_PARTS_OF_SPEECH_SINGLE_KANJI_BACK   NJ_HINSI_TANKANJI_B                //Part of speech type  Single kanji (back)
#define NJ_PARTS_OF_SPEECH_NOUN_FORE           NJ_HINSI_MEISI_F                   //Part of speech type  Noun (fore)
#define NJ_PARTS_OF_SPEECH_NOUN_BACK           NJ_HINSI_MEISI_B                   //Part of speech type  Noun (back) 
#define NJ_PARTS_OF_SPEECH_PERSON_NAME_FORE    NJ_HINSI_JINMEI_F                  //Part of speech type  Personal name (fore)
#define NJ_PARTS_OF_SPEECH_PERSON_NAME_BACK    NJ_HINSI_JINMEI_B                  //Part of speech type  Personal name (back)
#define NJ_PARTS_OF_SPEECH_NOUN_NO_CONJUGATION_FORE  NJ_HINSI_MEISI_NO_CONJ_F     //Part of speech type  Noun (no conjugation with "-suru")
#define NJ_PARTS_OF_SPEECH_NOUN_NO_CONJUGATION_BACK  NJ_HINSI_MEISI_NO_CONJ_B     //Part of speech type  Noun (no conjugation with "-suru")
#define NJ_PARTS_OF_SPEECH_PLACE_NAME_FORE     NJ_HINSI_CHIMEI_F                  //Part of speech type  Place name (fore)
#define NJ_PARTS_OF_SPEECH_PLACE_NAME_BACK     NJ_HINSI_CHIMEI_B                  //Part of speech type  Place name (back)
#define NJ_PARTS_OF_SPEECH_SYMBOL_FORE         NJ_HINSI_KIGOU_F                   //Part of speech type  Symbol (fore)
#define NJ_PARTS_OF_SPEECH_SYMBOL_BACK         NJ_HINSI_KIGOU_B                   //Part of speech type  Symbol (back)
#define NJ_PARTS_OF_SPEECH_V1_FORE             NJ_HINSI_V1_F                      //Part of speech type  End of phrase
#define NJ_PARTS_OF_SPEECH_V3_FORE             NJ_HINSI_V3_F                      //Part of speech type  End of phrase in middle of text

/******************************************/
/*   Defined values (nj_lib.h)  Note: Rename support */
/******************************************/
#define MM_PARTS_OF_SPEECH_GROUP_NOUN    MM_HGROUP_MEISI   //Part of speech group number: Noun
#define MM_PARTS_OF_SPEECH_GROUP_PSEUDO    MM_HGROUP_GIJI  //Part of speech group number: Pseudo
#define MM_PARTS_OF_SPEECH_GROUP_OTHER    MM_HGROUP_OTHER  //Part of speech group number: Other 
#define MM_MAX_MORPHOLIZE_LEN    MM_MAX_MORPHO_LEN         //[Morphological analysis] Maximum number of elements in a morphological analysis character array

//Default dictionary frequency :: Custom dictionary (for conversion) - Base -
#define NJ_DEFAULT_FREQUENCY_DIC_CUSTOM_CONVERSION_BASE    NJ_DEFAULT_FREQ_DIC_CUSTOM_H_BASE
// Default dictionary frequency :: Custom dictionary (for conversion) - High -  
#define NJ_DEFAULT_FREQUENCY_DIC_CUSTOM_CONVERSION_HIGH    NJ_DEFAULT_FREQ_DIC_CUSTOM_H_HIGH
//Default dictionary frequency :: Custom dictionary (for morphological analysis) - Base -
#define NJ_DEFAULT_FREQUENCY_DIC_CUSTOM_MORPHOLIZE_BASE    NJ_DEFAULT_FREQ_DIC_CUSTOM_M_BASE
//Default dictionary frequency :: Custom dictionary (for morphological analysis) - High - 
#define NJ_DEFAULT_FREQUENCY_DIC_CUSTOM_MORPHOLIZE_HIGH    NJ_DEFAULT_FREQ_DIC_CUSTOM_M_HIGH
//Default dictionary frequency :: Custom dictionary (for prediction) - Base - 
#define NJ_DEFAULT_FREQUENCY_DIC_CUSTOM_PREDICTION_BASE    NJ_DEFAULT_FREQ_DIC_CUSTOM_Y_BASE
//Default dictionary frequency :: Custom dictionary (for prediction) - High - 
#define NJ_DEFAULT_FREQUENCY_DIC_CUSTOM_PREDICTION_HIGH    NJ_DEFAULT_FREQ_DIC_CUSTOM_Y_HIGH

//Default dictionary frequency :: Prediction dictionary without extended reading (for conversion) - Base - 
#define NJ_DEFAULT_FREQUENCY_DIC_EXTENSION_READING_CONVERSION_BASE    NJ_DEFAULT_FREQ_DIC_EXT_YOMI_H_BASE
//Default dictionary frequency :: Prediction dictionary without extended reading (for conversion) - High - 
#define NJ_DEFAULT_FREQUENCY_DIC_EXTENSION_READING_CONVERSION_HIGH    NJ_DEFAULT_FREQ_DIC_EXT_YOMI_H_HIGH
//Default dictionary frequency :: Prediction dictionary without extended reading (for morphological analysis) - Base -
#define NJ_DEFAULT_FREQUENCY_DIC_EXTENSION_READING_MORPHOLIZE_BASE    NJ_DEFAULT_FREQ_DIC_EXT_YOMI_M_BASE
//Default dictionary frequency :: Prediction dictionary without extended reading (for morphological analysis) - High -  
#define NJ_DEFAULT_FREQUENCY_DIC_EXTENSION_READING_MORPHOLIZE_HIGH    NJ_DEFAULT_FREQ_DIC_EXT_YOMI_M_HIGH
//Default dictionary frequency :: Prediction dictionary without extended reading (for prediction) - Base -
#define NJ_DEFAULT_FREQUENCY_DIC_EXTENSION_READING_PREDICTION_BASE    NJ_DEFAULT_FREQ_DIC_EXT_YOMI_Y_BASE
//Default dictionary frequency :: Prediction dictionary without extended reading (for prediction) - High -
#define NJ_DEFAULT_FREQUENCY_DIC_EXTENSION_READING_PREDICTION_HIGH    NJ_DEFAULT_FREQ_DIC_EXT_YOMI_Y_HIGH

//Default dictionary frequency :: Ancillary word dictionary (for conversion) - Base -
#define NJ_DEFAULT_FREQUENCY_DIC_ANCILLARY_CONVERSION_BASE    NJ_DEFAULT_FREQ_DIC_FZK_H_BASE
//Default dictionary frequency :: Ancillary word dictionary (for conversion) - High -
#define NJ_DEFAULT_FREQUENCY_DIC_ANCILLARY_CONVERSION_HIGH    NJ_DEFAULT_FREQ_DIC_FZK_H_HIGH
//Default dictionary frequency :: Ancillary word dictionary (for morphological analysis) - Base -
#define NJ_DEFAULT_FREQUENCY_DIC_ANCILLARY_MORPHOLIZE_BASE    NJ_DEFAULT_FREQ_DIC_FZK_M_BASE
//Default dictionary frequency :: Ancillary word dictionary (for morphological analysis) - High -
#define NJ_DEFAULT_FREQUENCY_DIC_ANCILLARY_MORPHOLIZE_HIGH    NJ_DEFAULT_FREQ_DIC_FZK_M_HIGH
//Default dictionary frequency :: Ancillary word dictionary (for prediction) - Base - 
#define NJ_DEFAULT_FREQUENCY_DIC_ANCILLARY_PREDICTION_BASE    NJ_DEFAULT_FREQ_DIC_FZK_Y_BASE
//Default dictionary frequency :: Ancillary word dictionary (for prediction) - High - 
#define NJ_DEFAULT_FREQUENCY_DIC_ANCILLARY_PREDICTION_HIGH    NJ_DEFAULT_FREQ_DIC_FZK_Y_HIGH

//Default dictionary frequency :: Prediction dictionary without readings (for conversion) - Base -
#define NJ_DEFAULT_FREQUENCY_DIC_HEAD_READING_CONVERSION_BASE    NJ_DEFAULT_FREQ_DIC_HEAD_YOMI_H_BASE
//Default dictionary frequency :: Prediction dictionary without readings (for conversion) - High -
#define NJ_DEFAULT_FREQUENCY_DIC_HEAD_READING_CONVERSION_HIGH    NJ_DEFAULT_FREQ_DIC_HEAD_YOMI_H_HIGH
//Default dictionary frequency :: Prediction dictionary without readings (for morphological analysis) - Base -
#define NJ_DEFAULT_FREQUENCY_DIC_HEAD_READING_MORPHOLIZE_BASE    NJ_DEFAULT_FREQ_DIC_HEAD_YOMI_M_BASE
//Default dictionary frequency :: Prediction dictionary without readings (for morphological analysis) - High -
#define NJ_DEFAULT_FREQUENCY_DIC_HEAD_READING_MORPHOLIZE_HIGH    NJ_DEFAULT_FREQ_DIC_HEAD_YOMI_M_HIGH
//Default dictionary frequency :: Prediction dictionary without readings (for prediction) - Base - 
#define NJ_DEFAULT_FREQUENCY_DIC_HEAD_READING_PREDICTION_BASE    NJ_DEFAULT_FREQ_DIC_HEAD_YOMI_Y_BASE
//Default dictionary frequency :: Prediction dictionary without readings (for prediction) - High - 
#define NJ_DEFAULT_FREQUENCY_DIC_HEAD_READING_PREDICTION_HIGH    NJ_DEFAULT_FREQ_DIC_HEAD_YOMI_Y_HIGH

//Default dictionary frequency :: Learning dictionary (for conversion) - Base -
#define NJ_DEFAULT_FREQUENCY_DIC_LEARN_CONVERSION_BASE    NJ_DEFAULT_FREQ_DIC_LEARN_H_BASE
//Default dictionary frequency :: Learning dictionary (for conversion) - High -
#define NJ_DEFAULT_FREQUENCY_DIC_LEARN_CONVERSION_HIGH    NJ_DEFAULT_FREQ_DIC_LEARN_H_HIGH
//Default dictionary frequency :: Learning dictionary (for morphological analysis) - Base -
#define NJ_DEFAULT_FREQUENCY_DIC_LEARN_MORPHOLIZE_BASE    NJ_DEFAULT_FREQ_DIC_LEARN_M_BASE
//Default dictionary frequency :: Learning dictionary (for morphological analysis) - High - 
#define NJ_DEFAULT_FREQUENCY_DIC_LEARN_MORPHOLIZE_HIGH    NJ_DEFAULT_FREQ_DIC_LEARN_M_HIGH
//Default dictionary frequency :: Learning dictionary (for prediction) - Base -
#define NJ_DEFAULT_FREQUENCY_DIC_LEARN_PREDICTION_BASE    NJ_DEFAULT_FREQ_DIC_LEARN_Y_BASE
//Default dictionary frequency :: Learning dictionary (for prediction) - High -
#define NJ_DEFAULT_FREQUENCY_DIC_LEARN_PREDICTION_HIGH    NJ_DEFAULT_FREQ_DIC_LEARN_Y_HIGH

//Default dictionary frequency :: Pseudo-dictionary used for filtering and prediction (for conversion) - Base -
#define NJ_DEFAULT_FREQUENCY_DIC_LIMIT_PREDICTION_PRORAM_CONVERSION_BASE    NJ_DEFAULT_FREQ_DIC_LIMIT_PRED_PRORAM_H_BASE
//Default dictionary frequency :: Pseudo-dictionary used for filtering and prediction (for conversion) - High -
#define NJ_DEFAULT_FREQUENCY_DIC_LIMIT_PREDICTION_PRORAM_CONVERSION_HIGH    NJ_DEFAULT_FREQ_DIC_LIMIT_PRED_PRORAM_H_HIGH
//Default dictionary frequency :: Pseudo-dictionary for filtering and prediction (for morphological analysis) - Base -
#define NJ_DEFAULT_FREQUENCY_DIC_LIMIT_PREDICTION_PRORAM_MORPHOLIZE_BASE    NJ_DEFAULT_FREQ_DIC_LIMIT_PRED_PRORAM_M_BASE
//Default dictionary frequency :: Pseudo-dictionary used for filtering and prediction (for morphological analysis) - High -
#define NJ_DEFAULT_FREQUENCY_DIC_LIMIT_PREDICTION_PRORAM_MORPHOLIZE_HIGH    NJ_DEFAULT_FREQ_DIC_LIMIT_PRED_PRORAM_M_HIGH
//Default dictionary frequency :: Pseudo-dictionary used for filtering and prediction (for prediction) - Base - 
#define NJ_DEFAULT_FREQUENCY_DIC_LIMIT_PREDICTION_PRORAM_PREDICTION_BASE    NJ_DEFAULT_FREQ_DIC_LIMIT_PRED_PRORAM_Y_BASE
//Default dictionary frequency :: Pseudo-dictionary used for filtering and prediction (for prediction) - High - 
#define NJ_DEFAULT_FREQUENCY_DIC_LIMIT_PREDICTION_PRORAM_PREDICTION_HIGH    NJ_DEFAULT_FREQ_DIC_LIMIT_PRED_PRORAM_Y_HIGH

//Default dictionary frequency :: Pseudo-dictionary (for conversion) - Base -
#define NJ_DEFAULT_FREQUENCY_DIC_PRORAM_CONVERSION_BASE    NJ_DEFAULT_FREQ_DIC_PRORAM_H_BASE
//Default dictionary frequency :: Pseudo-dictionary (for conversion) - High -
#define NJ_DEFAULT_FREQUENCY_DIC_PRORAM_CONVERSION_HIGH    NJ_DEFAULT_FREQ_DIC_PRORAM_H_HIGH
//Default dictionary frequency :: Pseudo-dictionary (for morphological analysis) - Base -
#define NJ_DEFAULT_FREQUENCY_DIC_PRORAM_MORPHOLIZE_BASE    NJ_DEFAULT_FREQ_DIC_PRORAM_M_BASE
//Default dictionary frequency :: Pseudo-dictionary (for morphological analysis) - High -
#define NJ_DEFAULT_FREQUENCY_DIC_PRORAM_MORPHOLIZE_HIGH    NJ_DEFAULT_FREQ_DIC_PRORAM_M_HIGH
//Default dictionary frequency :: Pseudo-dictionary (for prediction) - Base -
#define NJ_DEFAULT_FREQUENCY_DIC_PRORAM_PREDICTION_BASE    NJ_DEFAULT_FREQ_DIC_PRORAM_Y_BASE
//Default dictionary frequency :: Pseudo-dictionary (for prediction) - High -
#define NJ_DEFAULT_FREQUENCY_DIC_PRORAM_PREDICTION_HIGH    NJ_DEFAULT_FREQ_DIC_PRORAM_Y_HIGH

//Default dictionary frequency :: Single kanji dictionary (for conversion) - Base -
#define NJ_DEFAULT_FREQUENCY_DIC_SINGLE_CONVERSION_BASE    NJ_DEFAULT_FREQ_DIC_TAN_H_BASE
//Default dictionary frequency :: Single kanji dictionary (for conversion) - High -
#define NJ_DEFAULT_FREQUENCY_DIC_SINGLE_CONVERSION_HIGH    NJ_DEFAULT_FREQ_DIC_TAN_H_HIGH
//Default dictionary frequency :: Single kanji dictionary (for morphological analysis) - Base -
#define NJ_DEFAULT_FREQUENCY_DIC_SINGLE_MORPHOLIZE_BASE    NJ_DEFAULT_FREQ_DIC_TAN_M_BASE
//Default dictionary frequency :: Single kanji dictionary (for morphological analysis) - High -
#define NJ_DEFAULT_FREQUENCY_DIC_SINGLE_MORPHOLIZE_HIGH    NJ_DEFAULT_FREQ_DIC_TAN_M_HIGH
//Default dictionary frequency :: Single kanji dictionary (for prediction) - Base -
#define NJ_DEFAULT_FREQUENCY_DIC_SINGLE_PREDICTION_BASE    NJ_DEFAULT_FREQ_DIC_TAN_Y_BASE
//Default dictionary frequency :: Single kanji dictionary (for prediction) - High -
#define NJ_DEFAULT_FREQUENCY_DIC_SINGLE_PREDICTION_HIGH    NJ_DEFAULT_FREQ_DIC_TAN_Y_HIGH

//Default dictionary frequency :: Integrated dictionary (for conversion) - Base - 
#define NJ_DEFAULT_FREQUENCY_DIC_UNION_CONVERSION_BASE    NJ_DEFAULT_FREQ_DIC_UBASE_H_BASE
//Default dictionary frequency :: Integrated dictionary (for conversion) - High - 
#define NJ_DEFAULT_FREQUENCY_DIC_UNION_CONVERSION_HIGH    NJ_DEFAULT_FREQ_DIC_UBASE_H_HIGH
//Default dictionary frequency :: Integrated dictionary (for morphological analysis) - Base - 
#define NJ_DEFAULT_FREQUENCY_DIC_UNION_MORPHOLIZE_BASE    NJ_DEFAULT_FREQ_DIC_UBASE_M_BASE
//Default dictionary frequency :: Integrated dictionary (for morphological analysis) - High - 
#define NJ_DEFAULT_FREQUENCY_DIC_UNION_MORPHOLIZE_HIGH    NJ_DEFAULT_FREQ_DIC_UBASE_M_HIGH
//Default dictionary frequency :: Integrated dictionary (for prediction) - Base - 
#define NJ_DEFAULT_FREQUENCY_DIC_UNION_PREDICTION_BASE    NJ_DEFAULT_FREQ_DIC_UBASE_Y_BASE
//Default dictionary frequency :: Integrated dictionary (for prediction) - High - 
#define NJ_DEFAULT_FREQUENCY_DIC_UNION_PREDICTION_HIGH    NJ_DEFAULT_FREQ_DIC_UBASE_Y_HIGH

//Default dictionary frequency :: User dictionary (for conversion) - Base - 
#define NJ_DEFAULT_FREQUENCY_DIC_USER_CONVERSION_BASE    NJ_DEFAULT_FREQ_DIC_USER_H_BASE
//Default dictionary frequency :: User dictionary (for conversion) - High - 
#define NJ_DEFAULT_FREQUENCY_DIC_USER_CONVERSION_HIGH    NJ_DEFAULT_FREQ_DIC_USER_H_HIGH
//Default dictionary frequency :: User dictionary (for morphological analysis) - Base - 
#define NJ_DEFAULT_FREQUENCY_DIC_USER_MORPHOLIZE_BASE    NJ_DEFAULT_FREQ_DIC_USER_M_BASE
//Default dictionary frequency :: User dictionary (for morphological analysis) - High - 
#define NJ_DEFAULT_FREQUENCY_DIC_USER_MORPHOLIZE_HIGH    NJ_DEFAULT_FREQ_DIC_USER_M_HIGH
//Default dictionary frequency :: User dictionary (for prediction) - Base - 
#define NJ_DEFAULT_FREQUENCY_DIC_USER_PREDICTION_BASE    NJ_DEFAULT_FREQ_DIC_USER_Y_BASE
//Default dictionary frequency :: User dictionary (for prediction) - High - 
#define NJ_DEFAULT_FREQUENCY_DIC_USER_PREDICTION_HIGH    NJ_DEFAULT_FREQ_DIC_USER_Y_HIGH

//Default dictionary frequency :: Prediction dictionary without readings (for conversion) - Base -
#define NJ_DEFAULT_FREQUENCY_DIC_READING_CONVERSION_BASE    NJ_DEFAULT_FREQ_DIC_YOMI_H_BASE
//Default dictionary frequency :: Prediction dictionary without readings (for conversion) - High -
#define NJ_DEFAULT_FREQUENCY_DIC_READING_CONVERSION_HIGH    NJ_DEFAULT_FREQ_DIC_YOMI_H_HIGH
//Default dictionary frequency :: Prediction dictionary without readings (for morphological analysis) - Base -
#define NJ_DEFAULT_FREQUENCY_DIC_READING_MORPHOLIZE_BASE    NJ_DEFAULT_FREQ_DIC_YOMI_M_BASE
//Default dictionary frequency :: Prediction dictionary without readings (for morphological analysis) - High -
#define NJ_DEFAULT_FREQUENCY_DIC_READING_MORPHOLIZE_HIGH    NJ_DEFAULT_FREQ_DIC_YOMI_M_HIGH
//Default dictionary frequency :: Prediction dictionary without readings (for prediction) - Base -
#define NJ_DEFAULT_FREQUENCY_DIC_READING_PREDICTION_BASE    NJ_DEFAULT_FREQ_DIC_YOMI_Y_BASE
//Default dictionary frequency :: Prediction dictionary without readings (for prediction) - High -
#define NJ_DEFAULT_FREQUENCY_DIC_READING_PREDICTION_HIGH    NJ_DEFAULT_FREQ_DIC_YOMI_Y_HIGH


#define NJ_MAX_ANCILLARY_CANDIDATE    NJ_MAX_FZK_CANDIDATE          //[All candidates] Ancillary word parsing number of patterns
#define NJ_MAX_ANCILLARY_LEN    NJ_MAX_FZK_LEN                      //[All candidates] Ancillary word parsing: Maximum number of elements in character array
#define NJ_MAX_PHRASE_CONNECT    NJ_MAX_PHR_CONNECT                 //[Conversion feature] Maximum number of connected phrases
#define NJ_MAX_USER_CANDIDATE_LEN    NJ_MAX_USER_KOUHO_LEN          //[Word registration] Maximum number of elements in a candidate character array

#define NJ_MLD_OPERATION_COMMIT    NJ_MLD_OP_COMMIT                 //Learning dictionary operations: Record location of operation
#define NJ_MLD_OPERATION_COMMIT_TO_TOP    NJ_MLD_OP_COMMIT_TO_TOP   //Learning dictionary operation: Move learning information
#define NJ_MLD_OPERATION_COMMIT_CANCEL    NJ_MLD_OP_COMMIT_CANCEL   //Learning dictionary operation: Cancel operations
#define NJ_MLD_OPERATION_GET_SPACE    NJ_MLD_OP_GET_SPACE           //Learning dictionary operation: Get number of possible registrations

#define NJ_NO_MULTI_CONVERSION    NJ_NO_RENBUN                      //Parsing limitation: No multiple phrase conversion results
#define NJ_NO_SINGLE_CONVERSION    NJ_NO_TANBUN                     //Parsing limitation: No single phrase conversion results
#define NJ_NO_PREDICTION    NJ_NO_YOSOKU                            //Parsing limitation: No prediction results inside prediction dictionary
#define NJ_NO_ALL_CANDIDATE    NJ_NO_ZEN                            //Parsing limitation: No All Candidates results
#define NJ_NO_READING_ON    NJ_YOMINASI_ON                          //Parsing limitation: There is a no reading prediction filtered search 

#define NJ_STATE_MAX_FREQUENCY    NJ_STATE_MAX_FREQ                 //State setting  Maximum dictionary frequency value
#define NJ_STATE_MIN_FREQUENCY    NJ_STATE_MIN_FREQ                 //State setting  Minimum dictionary frequency value

#define NJ_TYPE_HALF_NUMERIC    NJ_TYPE_HAN_SUUJI                   //Pseudo-candidate type: single-byte numeric character
#define NJ_TYPE_HALF_KATAKANA    NJ_TYPE_HANKATA                    //Pseudo-candidate type: single-byte katakana
#define NJ_TYPE_FULL_NUMERIC    NJ_TYPE_ZEN_SUUJI                   //Pseudo-candidate type: double-byte numeral
#define NJ_USER_QUEUE_SIZE    NJ_USER_QUE_SIZE                      //[Word registration] User dictionary size
#define NJ_USER2_QUEUE_SIZE    NJ_USER2_QUE_SIZE                    // [Word registration] User dictionary (with additional information) size


/******************************************/
/*   Structures (nj_lib.h)  Note: Rename support  */
/******************************************/
/* Please see man pages for details 
 
 */
typedef struct {
    u32     dicType;                              //[Internal Use] Dictionary type
    u32     dicSize;                              //[Internal Use] Dictionary size
    u32     mode;                                 //[Internal Use] Operation mode
    IWNN_FILE *     fileStream;                   //[Internal Use] File pointer
    void *     cacheArea[NJ_FLASH_DIC_CACHE_MAX]; //[Internal Use] Cache region
    void *     extensionData;                    //Extended info
} IWNN_FLASH_DIC_INFO;



/* Please see man pages for details 
 
 */
typedef struct {
    u8            type;           //Dictionary handle type
#define NJ_DIC_HANDLE_TYPE_NRM                 NJ_DIC_H_TYPE_NORMAL      //Normal dictionary (0x00)
#define NJ_DIC_HANDLE_TYPE_PROGRAM             NJ_DIC_H_TYPE_PROGRAM     //Pseudo-dictionary (0x01)
#define NJ_DIC_HANDLE_TYPE_PROGRAM_ANCILLARY   NJ_DIC_H_TYPE_PROGRAM_FZK //Pseudo-dictionary - ancillary words (0x02)
#define NJ_DIC_HANDLE_TYPE_ON_FLASH            NJ_DIC_H_TYPE_ON_FLASH    //FLASH dictionary (0x03)

    u8            limit;          //Search limits
    NN_PADDING2;

    IWNN_DIC_HANDLE  handle;              //Dictionary handle and pseudo-dictionary function pointer
    void *           extensionArea;       //Pseudo-dictionary work memory
    void *           addInfo[NJ_MAX_ADDITIONAL_INFO]; //Additional information region

    IWNN_DIC_FREQUENCY  dicFrequency[NJ_MODE_TYPE_MAX];    //Dictionary frequency information
#define NJ_MODE_TYPE_CONVERSION NJ_MODE_TYPE_HENKAN  //Normal conversion dictionary frequency (0)
#define NJ_MODE_TYPE_PREDICTION NJ_MODE_TYPE_YOSOKU  //Predictive conversion dictionary frequency (1)
#define NJ_MODE_TYPE_MORPHOLIZE NJ_MODE_TYPE_MORPHO  //Morphological analysis dictionary frequency (2) 

    NJ_SEARCH_CACHE *   searchCache;       //Cache management region
} IWNN_DIC_INFO;



/* Please see man pages for details 
 
 */
typedef struct {
    IWNN_DIC_INFO    dic[NJ_MAX_DIC];              //Dictionary information
    IWNN_DIC_HANDLE  ruleHandle[NJ_MODE_TYPE_MAX]; //Rule dictionary mount region
    u16              mode;                         //[Internal Use] Operation mode
    NN_PADDING2;
    wchar_t          keyword[NJ_MAX_KEYWORD];      //[Internal Use] Fuzzy search keyword
} IWNN_DIC_SET;

/* Please see man pages for details 
 
 */
typedef struct {
    u16  charSetCnt;                    //charset storage count
    NN_PADDING2;
    wchar_t    *from[NJ_MAX_CHARSET];   //Character before replacement
    wchar_t    *to[NJ_MAX_CHARSET];     //Character after replacement
} IWNN_CHARSET;

/* Please see man pages for details 
 
 */
typedef struct {

    u8 operation;          //Search method
#define NJ_CURSOR_OPERATION_COMPLETE NJ_CUR_OP_COMP    //Forward lookup complete match search
#define NJ_CURSOR_OPERATION_FORE    NJ_CUR_OP_FORE         //Forward lookup prefix match search
#define NJ_CURSOR_OPERATION_LINK    NJ_CUR_OP_LINK         //Derived search
#define NJ_CURSOR_OPERATION_REVERSE NJ_CUR_OP_REV          //Reverse lookup complete match search
#define NJ_CURSOR_OPERATION_REVERSE_FORE  NJ_CUR_OP_REV_FORE   //Reverse lookup prefix match search

    u8 mode;               //Order in which search candidates were obtained
#define NJ_CURSOR_MODE_FREQUENCY NJ_CUR_MODE_FREQ     //Frequency order
#define NJ_CURSOR_MODE_READING   NJ_CUR_MODE_YOMI     //Reading order
#define NJ_CURSOR_MODE_REGIST    NJ_CUR_MODE_REGIST   //Order of registration


      NN_PADDING2;
      IWNN_DIC_SET *dicSet;              //Dictionary set to be searched

    /* Please see man pages for details 
     
     */
    struct {
        u8 *fore;          //Previous part of speech conditions
        u16 foreSize;      //Number of previous parts of speech registered
        u16 foreFlag;      //0: Normal comparison, 1: Inverse comparison
        u8 *rear;          //Back part of speech conditions
        u16 rearSize;      //Number of back parts of speech registered
        u16 rearFlag;      //0: Normal comparison, 1: Inverse comparison
        u8 *noReadingFore; //Previous part of speech condition (no reading)
        u16 previousBackPartsOfSpeech;     // Back part of speech number for pre-confirmed word (no extended readings)
        NN_PADDING2;
    } partsOfSpeech;

    wchar_t  *reading;     //Reading string of the word to be searched for  
                           //@attention (u8) 0 is used for termination
    u16 readingLen;        //Reading string length (number of elements in the character array)
    u16 yclen;             //Reading string length (number of characters)  Used only by the evaluation block
    wchar_t  *kanji;       //Prediction source notation string when operation = NJ_CUR_OP_LINK is specified
                           //@attention (u8) 0 is used for termination
    IWNN_CHARSET *charSet;   //Fuzzy character set

    u8   ancillaryConnect;  //Ancillary word connection yes/no
    u8   ctrl_opt;          //[Internal Use]
    NN_PADDING2;
} IWNN_SEARCH_CONDITION;

/* Please see man pages for details 
 
 */
typedef struct {
    IWNN_DIC_HANDLE  handle;        //Dictionary handle
    void *      extensionArea;      //Pseudo-dictionary work memory
    void *      addInfo[NJ_MAX_ADDITIONAL_INFO]; //Additional information region
    u32      current;       //Current position
    u32      top;           //Head position
    u32      bottom;        //Tail position
    u32      relation[NJ_MAX_PHR_CONNECT];   //Head position
    u32      attr;          //Attribute data
    u16      currentCache;  //Current search cache position
    u8       currentInfo;   //Number of joined phrases
    u8       status;        //Search status
#define NJ_STATUS_SEARCH_NO_INIT          1       //Not initialized (=NJ_ST_SEARCH_NO_INIT)
#define NJ_STATUS_SEARCH_READY            2       //Initialized (=NJ_ST_SEARCH_READY)
#define NJ_STATUS_SEARCH_END              3       //No more candidates (=NJ_ST_SEARCH_END)
#define NJ_STATUS_SEARCH_END_EXTENSION    4       //No more candidates (no prefix match candidates either) (=NJ_ST_SEARCH_END_EXT) */
    u8       type;          //Dictionary type
    NN_PADDING3;
} IWNN_SEARCH_LOCATION;

/* Please see man pages for details 
 
 */
typedef struct {
    IWNN_FREQUENCY     cacheFrequency;   //Evaluated value
    NN_PADDING2;
    IWNN_DIC_FREQUENCY dicFrequency;     //Dictionary frequency
    s16           dicFrequencyMax;       //Maximum dictionary frequency value
    s16           dicFrequencyMin;       //Minimum dictionary frequency value
    IWNN_SEARCH_LOCATION location;       //Search position
} IWNN_SEARCH_LOCATION_SET;


/* Please see man pages for details 
 
 */
typedef struct {
    IWNN_SEARCH_CONDITION condition;                 //Dictionary search conditions
    IWNN_SEARCH_LOCATION_SET locationSet[NJ_MAX_DIC];  //Searched word information for each dictionary
} IWNN_CURSOR;



/* Please see man pages for details 
 
 */
typedef struct {
    u8 partsOfSpeechGroup;          //Part of speech group
#define NJ_PARTS_OF_SPEECH_NOUN                NJ_HINSI_MEISI             //Noun (conjugates with "-suru")
#define NJ_PARTS_OF_SPEECH_PERSON_NAME         NJ_HINSI_JINMEI            //Personal name
#define NJ_PARTS_OF_SPEECH_NOUN_NO_CONJUGATION NJ_HINSI_MEISI_NO_CONJ     //Noun (no conjugation with "-suru")
#define NJ_PARTS_OF_SPEECH_PLACE_NAME          NJ_HINSI_CHIMEI            //Place name/station name
#define NJ_PARTS_OF_SPEECH_SYMBOL              NJ_HINSI_KIGOU             //Symbols
#define NJ_PARTS_OF_SPEECH_DETAIL              NJ_HINSI_DETAIL            //Get details
    NN_PADDING3;

    wchar_t  reading[NJ_MAX_LEN + NJ_TERM_LEN];         //Registered reading string (including NUL)
    wchar_t  candidate[NJ_MAX_RESULT_LEN + NJ_TERM_LEN]; //Registered candidate string (including NUL)
    wchar_t  additional[NJ_MAX_ADDITIONAL_LEN + NJ_TERM_LEN]; //Additional information string (including NUL)

    //Independent word information
    struct {
        u16  readingLen;     //Reading character array length
        u16  candidateLen;   //Candidate character array length
        u32  partsOfSpeech;  //Part of speech
        u32  attr;           //Attribute data
        s16  frequency;      //Frequency
        NN_PADDING2;
    } stem;

    //Ancillary word information
    struct {
        u16  readingLen;     //Reading character array length
        u16  candidateLen;   //Candidate character array length
        u32  partsOfSpeech;  //Part of speech
        s16  frequency;      //Frequency
        NN_PADDING2;
    } ancillary;
    s16   connect;           //Connection flag
    NN_PADDING2;
} IWNN_WORD_INFO;


/* Please see man pages for details 
 
 */
typedef struct {
    u16  info1;                   //Upper 9 bits: previous part of speech number, Lower 7 bits: Number of elements in the reading character array
    u16  info2;                   //Upper 9 bits: following part of speech number, Lower 7 bits: Number of elements in the candidate character array
    IWNN_FREQUENCY   frequency;   //Frequency values
    NN_PADDING2;
} IWNN_ANCILLARY ;
 
typedef struct {
    u16  info1;       //Upper 9 bits: previous part of speech number, Lower 7 bits: Number of elements in the reading character array
    u16  info2;       //Upper 9 bits: following part of speech number, Lower 7 bits: Number of elements in the candidate character array
    IWNN_FREQUENCY   frequency;    //Frequency values
    NN_PADDING2;
    IWNN_SEARCH_LOCATION location; //Dictionary location
    u8   type;                     //Type of pseudo-dictionary
    NN_PADDING3;
} IWNN_STEM;

typedef struct {
    wchar_t  *reading;            //Reading string
    IWNN_STEM stem;               //Independent word information
    IWNN_ANCILLARY ancillary;     //Ancillary word information
    
} IWNN_WORD;
/******************************************/
/*   Macro (nj_lib.h)  Note: Rename support */
/******************************************/
/* Please see man pages for details 
 
 
 
 
 
 */
#define NJ_GET_FORE_PARTS_OF_SPEECH_FROM_STEM(s) NJ_GET_FPOS_FROM_STEM(((NJ_WORD*)s))
/* Please see man pages for details 
 
 
 
 
 
 */
#define NJ_GET_READING_LEN_FROM_STEM(s) NJ_GET_YLEN_FROM_STEM(((NJ_WORD*)s))
/* Please see man pages for details 
 
 
 
 
 
 */
#define NJ_GET_BACK_PARTS_OF_SPEECH_FROM_STEM(s) NJ_GET_BPOS_FROM_STEM(((NJ_WORD*)s))
/* Please see man pages for details 
 
 
 
 
 
 */
#define NJ_GET_CANDIDATE_LEN_FROM_STEM(s) NJ_GET_KLEN_FROM_STEM(((NJ_WORD*)s))
/* Please see man pages for details 
 
 
 
 
 
 */
#define NJ_GET_FREQUENCY_FROM_STEM(s) NJ_GET_FREQ_FROM_STEM(((NJ_WORD*)s))

/* Please see man pages for details 
 
 
 
 
 
 */
#define NJ_GET_WORD_TYPE_FROM_STEM(s) NJ_GET_TYPE_FROM_STEM(((NJ_WORD*)s))

/* Please see man pages for details 
 
 
 
 
 
 
 */
#define NJ_SET_FORE_PARTS_OF_SPEECH_TO_STEM(s,v) NJ_SET_FPOS_TO_STEM(((NJ_WORD*)s),v)
/* Please see man pages for details 
 
 
 
 
 
 
 */
#define NJ_SET_READING_LEN_TO_STEM(s,v) NJ_SET_YLEN_TO_STEM(((NJ_WORD*)s),v)
/* Please see man pages for details 
 
 
 
 
 
 
 */
#define NJ_SET_BACK_PARTS_OF_SPEECH_TO_STEM(s,v) NJ_SET_BPOS_TO_STEM(((NJ_WORD*)s),v)
/* Please see man pages for details 
 
 
 
 
 
 
 */
#define NJ_SET_CANDIDATE_LEN_TO_STEM(s,v) NJ_SET_KLEN_TO_STEM(((NJ_WORD*)s),v)
/* Please see man pages for details 
 
 
 
 
 
 
 */
#define NJ_SET_FREQUENCY_TO_STEM(s,v) NJ_SET_FREQ_TO_STEM(((NJ_WORD*)s),v)
/* Please see man pages for details 
 
 
 
 
 
 
 */
#define NJ_SET_WORD_TYPE_TO_STEM(s,v) NJ_SET_TYPE_TO_STEM(((NJ_WORD*)s),v)

/* Please see man pages for details 
 
 
 
 
 
 */
#define NJ_GET_FORE_PARTS_OF_SPEECH_FROM_ANCILLARY(f) NJ_GET_FPOS_FROM_FZK(((NJ_WORD*)f))
/* Please see man pages for details 
 
 
 
 
 
 */
#define NJ_GET_READING_LEN_FROM_ANCILLARY(f) NJ_GET_YLEN_FROM_FZK(((NJ_WORD*)f))
/* Please see man pages for details 
 
 
 
 
 
 */
#define NJ_GET_BACK_PARTS_OF_SPEECH_FROM_ANCILLARY(f) NJ_GET_BPOS_FROM_FZK(((NJ_WORD*)f))
/* Please see man pages for details 
 
 
 
 
 
 */
#define NJ_GET_CANDIDATE_LEN_FROM_ANCILLARY(f) NJ_GET_KLEN_FROM_FZK(((NJ_WORD*)f))
/* Please see man pages for details 
 
 
 
 
 
 */
#define NJ_GET_FREQUENCY_FROM_ANCILLARY(f) NJ_GET_FREQ_FROM_FZK(((NJ_WORD*)f))
/* Please see man pages for details 
 
 
 
 
 
 
 */
#define NJ_SET_FORE_PARTS_OF_SPEECH_TO_ANCILLARY(s,v) NJ_SET_FPOS_TO_FZK(((NJ_WORD*)s),v)
/* Please see man pages for details 
 
 
 
 
 
 
 */
#define NJ_SET_READING_LEN_TO_ANCILLARY(s,v) NJ_SET_YLEN_TO_FZK(((NJ_WORD*)s),v)
/* Please see man pages for details 
 
 
 
 
 
 
 */
#define NJ_SET_BACK_PARTS_OF_SPEECH_TO_ANCILLARY(s,v) NJ_SET_BPOS_TO_FZK(((NJ_WORD*)s),v)
/* Please see man pages for details 
 
 
 
 
 
 
 */
#define NJ_SET_CANDIDATE_LEN_TO_ANCILLARY(s,v) NJ_SET_KLEN_TO_FZK(((NJ_WORD*)s),v)
/* Please see man pages for details 
 
 
 
 
 
 
 */
#define NJ_SET_FREQUENCY_TO_ANCILLARY(s,v) NJ_SET_FREQ_TO_FZK(((NJ_WORD*)s),v)

/******************************************/
/*   Defined values (njx_lib.h)  Note: Rename support  */
/******************************************/
#define NJ_ANALYZE_FORWARD_SEARCH_WITH_NO_READING    NJ_ANALYZE_FORWARD_SEARCH_WITH_YOMINASI // Prefix match analysis (including no reading prediction) 
#define NJ_ANALYZE_NEXT_PREDICTION    NJ_ANALYZE_NEXT_YOSOKU        // No reading prediction (derived predictions)  

/******************************************/
/*   Macro (njx_lib.h)  Note: Rename support */
/******************************************/
/* Please see man pages for details 
 
 
 
 
 
 
 */
#define NJ_SET_ENVIRONMENT(x,y) ((x).dicSet = (y))


/* Please see man pages for details 
 
 */
typedef struct {
    u16 operationId;    //Operation number

#define NJ_OPERATION_MASK  NJ_OP_MASK            //Operation mask (0x000f)
    /* Please see man pages for details 
     
     
     
     
     
     
     */
#define NJ_GET_RESULT_OPERATION(id)  ((id) & NJ_OPERATION_MASK)
#define NJ_OPERATION_SEARCH       NJ_OP_SEARCH        //Dictionary search (0x0000)
#define NJ_OPERATION_CONVERT      NJ_OP_CONVERT       //Normal conversion (0x0001)
#define NJ_OPERATION_ANALYZE      NJ_OP_ANALYZE       //Predictive conversion (0x0002)
#define NJ_OPERATION_MORPHOLIZE   NJ_OP_MORPHOLIZE    //Morphological analysis (0x0003)
#define NJ_OPERATION_ENVIRONMENT  NJ_OP_ENV           //All candidates (0x0004)


    /* Please see man pages for details 
     
     
     
     
     
     
           
           
           
           
           
           
           
           
           
     */
#define NJ_FUNC_ALL_CANDIDATES NJ_FUNC_ZENKOUHO
#define NJ_FUNC_SEARCH_REVERSE NJ_FUNC_SEARCH_R
#define NJ_FUNC_GET_PSEUDO     NJ_FUNC_GET_GIJI



#define NJ_TYPE_PSEUDO_MASK           0x0800  //Pseudo type mask
    /* Please see man pages for details 
     
     
     
     
     
     
           
           
           
     */
#define NJ_GET_PSEUDO_BIT(id)       ((id) & NJ_TYPE_PSEUDO_MASK)
#define NJ_TYPE_PSEUDO_BIT   NJ_TYPE_GIJI_BIT


    /* Please see man pages for details 
     
     
     
     
     
     
       
       
       
       
       
       
     */
#define NJ_DIC_PSEUDO               NJ_DIC_GIJI
    NN_PADDING2;
    IWNN_WORD word;    //[Internal Use] Word information
} IWNN_RESULT;


/* Please see man pages for details 
 
 */
typedef struct {
    u16  autoConversionCnt;                                       //Multiple conversion auto start candidate count
#define NJ_OPTION_AUTO_CONVERSION_CNT     NJ_OPT_AUTOCONV_CNT     //Default value for the number of multiple phrase conversion auto start candidates (NJ_MAX_CANDIDATE)

    u16  extensionMode;                                              //API extended operation mode (NJ_ADD_WORD_OPTIMIZE_OFF/NJ_OPTION_FORECAST_TOP_FREQUENCY)
#define NJ_OPTION_FORECAST_TOP_FREQUENCY  NJ_OPT_FORECAST_TOP_FREQ   //Fuzzy search specifications OFF (0x0002)


    void *  phase2Filter;       //Pointer to the candidate filtering function
    void *  phase2Option;       //Option of the candidate filtering function
    void *  phase1Filter;       //Pointer to the dictionary lookup filtering function
    void *  phase1Option;       //Options of the dictionary lookup filtering function
} IWNN_OPTION;



/* Please see man pages for details 
 
 */
typedef struct {
    u16  mode;                          //Parsing limits (Default = NJ_DEFAULT_MODE)
    u16  forecastLearnLimit;            //Maximum number of prediction candidates in learning dictionary (default = NJ_DEFAULT_FORECAST_LEARN_LIMIT)
    u16  forecastLimit;                 //Maximum number of prediction candidates (default = NJ_DEFAULT_FORECAST_LEARN_LIMIT)
    NN_PADDING2;
    u8   charMin;                       //Minimum number of reading characters (default = NJ_DEFAULT_CHAR_MIN)
    u8   charMax;                       //Maximum number of reading characters (default = NJ_DEFAULT_CHAR_MAX)
    NN_PADDING2;
} IWNN_ANALYZE_OPTION;


/* Please see man pages for details 
 
 */
typedef struct {
    s16 sysMaxBias[NJ_MAX_STATE];                 //State setting: Maximum bias value setting
    s16 sysMinBias[NJ_MAX_STATE];                 //State setting: Minimum bias value setting
    s16 sysAddBias[NJ_MAX_STATE];                 //State setting: Value derived from always adding the bias value
    s16 sysBaseBias[NJ_MAX_STATE];                //State setting: Restored bias value
    s16 sysChangeBias[NJ_MAX_STATE][NJ_MAX_STATE];//State setting: Amount of bias to add during learning
    s16 dicInfoMax[NJ_MAX_DIC];                   //State setting: Maximum dictionary frequency value
    s16 dicInfoMin[NJ_MAX_DIC];                   //State setting: Minimum dictionary frequency value
} IWNN_STATE_CALC_PARAMETER;

/* Please see man pages for details 
 
 */
typedef struct {
    s16 system[NJ_MAX_STATE];        //State settings used by applications and the engine
#ifndef NJ_OPT_STATE_TYPE2
#define NJ_CATEGORY_FIELD_PERSON          NJ_CAT_FIELD_PERSON             //Input field: Personal name
#define NJ_CATEGORY_FIELD_NOUN            NJ_CAT_FIELD_NOUN               //Input field: Succession of noun
#define NJ_CATEGORY_FIELD_HEAD            NJ_CAT_FIELD_HEAD               //Input field: Start of sentence
#define NJ_CATEGORY_EXPRESSION_COLLOQUIAL NJ_CAT_EXPRESSION_COLLOQUIAL    //Expression type: Soft expression (spoken language)
#define NJ_CATEGORY_EXPRESSION_WRITTEN    NJ_CAT_EXPRESSION_WRITTEN       //Expression type: Hard expression (written language)
#define NJ_CATEGORY_TIME_MORNING          NJ_CAT_TIME_MORNING             //Time expression: Morning
#define NJ_CATEGORY_TIME_NOON             NJ_CAT_TIME_NOON                //Time expression: Noon
#define NJ_CATEGORY_TIME_NIGHT            NJ_CAT_TIME_NIGHT               //Time expression: Night
#define NJ_CATEGORY_TIME_SPRING           NJ_CAT_TIME_SPRING              //Time expression: Spring
#define NJ_CATEGORY_TIME_SUMMER           NJ_CAT_TIME_SUMMER              //Time expression: Summer
#define NJ_CATEGORY_TIME_AUTUMN           NJ_CAT_TIME_AUTUMN              //Time expression: Autumn
#define NJ_CATEGORY_TIME_WINTER           NJ_CAT_TIME_WINTER              //Time expression: Winter
#define NJ_CATEGORY_TIME_PAST             NJ_CAT_TIME_PAST                //Time expression: Past
#define NJ_CATEGORY_TIME_FUTURE           NJ_CAT_TIME_FUTURE              //Time expression: Future
#define NJ_CATEGORY_FEEL_PLUS             NJ_CAT_FEEL_PLUS                //Expression of feeling: Positive connotation  
#define NJ_CATEGORY_FEEL_MINUS            NJ_CAT_FEEL_MINUS               //Expression of feeling: Negative connotation
#else /* !NJ_OPT_STATE_TYPE2*/
#define NJ_CATEGORY_FIELD_PERSON          NJ_CAT_FIELD_PERSON             //Input field: Personal name
#define NJ_CATEGORY_FIELD_NOUN            NJ_CAT_FIELD_NOUN               //Input field: Succession of noun
#define NJ_CATEGORY_FIELD_HEAD            NJ_CAT_FIELD_HEAD               //Input field: Start of sentence
#define NJ_CATEGORY_EXPRESSION_COLLOQUIAL NJ_CAT_EXPRESSION_COLLOQUIAL    //Expression type: Soft expression (spoken language)
#define NJ_CATEGORY_EXPRESSION_WRITTEN    NJ_CAT_EXPRESSION_WRITTEN       //Expression type: Hard expression (written language)
#define NJ_CATEGORY_TIME_MORNING          NJ_CAT_TIME_MORNING             //Time expression: Morning
#define NJ_CATEGORY_TIME_NOON             NJ_CAT_TIME_NOON                //Time expression: Noon
#define NJ_CATEGORY_TIME_NIGHT            NJ_CAT_TIME_NIGHT               //Time expression: Night
#define NJ_CATEGORY_TIME_PAST             NJ_CAT_TIME_PAST                //Time expression: Past
#define NJ_CATEGORY_TIME_FUTURE           NJ_CAT_TIME_FUTURE              //Time expression: Future
#define NJ_CATEGORY_FEEL_PLUS             NJ_CAT_FEEL_PLUS                //Expression of feeling: Positive connotation  
#define NJ_CATEGORY_FEEL_MINUS            NJ_CAT_FEEL_MINUS               //Expression of feeling: Negative connotation
#define NJ_CATEGORY_MONTH_JANUARY         NJ_CAT_MONTH_JAN                //Month expression: January
#define NJ_CATEGORY_MONTH_FEBRUARY        NJ_CAT_MONTH_FEB                //Month expression: February
#define NJ_CATEGORY_MONTH_MARCH           NJ_CAT_MONTH_MAR                //Month expression: March
#define NJ_CATEGORY_MONTH_APRIL           NJ_CAT_MONTH_APR                //Month expression: April
#define NJ_CATEGORY_MONTH_MAY             NJ_CAT_MONTH_MAY                //Month expression: May
#define NJ_CATEGORY_MONTH_JUNE            NJ_CAT_MONTH_JUN                //Month expression: June
#define NJ_CATEGORY_MONTH_JULY            NJ_CAT_MONTH_JUL                //Month expression: July
#define NJ_CATEGORY_MONTH_AUGUST          NJ_CAT_MONTH_AUG                //Month expression: August
#define NJ_CATEGORY_MONTH_SEPTEMBER       NJ_CAT_MONTH_SEP                //Month expression: September
#define NJ_CATEGORY_MONTH_OCTOBER         NJ_CAT_MONTH_OCT                //Month expression: October
#define NJ_CATEGORY_MONTH_NOVEMBER        NJ_CAT_MONTH_NOV                //Month expression: November
#define NJ_CATEGORY_MONTH_DECEMBER        NJ_CAT_MONTH_DEC                //Month expression: December
#endif /* !NJ_OPT_STATE_TYPE2 */

#define NJ_MAX_EXTENSION_STATE            NJ_MAX_EXT_STATE                  //Number of extended state settings used by the application
    s16 extension[NJ_MAX_EXTENSION_STATE];                                  //Extended state settings used by the application
    IWNN_STATE_CALC_PARAMETER *calcParam;
} IWNN_STATE;


/******************************************/
/*   Structures (njx_lib.h)  Note: Rename support  */
/******************************************/
/* Please see man pages for details 
 
 */
typedef struct {
    u8 iwnn_now;               //[Internal] Work counter
    u8 mm_yomi_search_index;   //[Internal] Flag when getting reading
    u8 mm_yomi_fzk_len;        //[Internal] Ancillary word character array length
    u8 njcMode;                //[Internal] Conversion mode  0: Initial status, 1: Currently converting, 2: Getting all candidates, 3: Performing morphological analysis
    NJC_SENTENCE        m_ConvResult;    //[Internal] Multiple phrase conversion result storage region

    //[Internal] Conversion process work region
    union {
        NJC_CONV        Conv;           //[Internal] Multiple phrase conversion process work
        NJC_CANDIDATE   Cand;           //[Internal] All candidates process result storage region
    } m_Buf;
    NJC_CANDIDATE_WORK m_cwork;         //[Internal] Independent word parsing work region
    IWNN_CURSOR m_ccur[NJ_MAX_LEN];       //[Internal] Dictionary search cursor
    NJ_WQUE que_tmp;                    //[Internal] Temporary queue
    wchar_t learn_string_tmp[NJ_MAX_RESULT_LEN + NJ_TERM_LEN];    //[Internal] Temporary learning string buffer
    wchar_t muhenkan_tmp[NJ_MAX_RESULT_LEN + NJ_TERM_LEN];        //[Internal] Temporary no-conversion string buffer
    NJ_PREVIOUS_SELECTION_INFO previous_selection;                //[Internal] Pre-confirmation information
    u32 prev_search_range[NJ_MAX_DIC][2];                         //[Internal] Derived index search range

    NJ_RESULT relation_results[NJ_MAX_RELATION_RESULTS];    //[Internal] Derived prediction result storage information
    u32 relation_results_count;                             //[Internal] Derived prediction result storage counter


    //[Internal] Buffer used to delete duplicate candidates
    union {
        NJ_WORK_BUF workbuf;  //[Internal] For checking notation
        NJ_YOMI_BUF yomibuf;  //[Internal] For checking the reading
    } h_buf;

    NJ_ANALYZE_ENV environment;    //[Internal] Parsing environment settings


    u8 relation_cand_flg;                //[Internal] Filtered search target candidate confirmation flag for each dictionary (0: Candidate to undergo non-filtered search, 1: Candidate to undergo filtered search)
    NN_PADDING3;
    u8 wk_relation_cand_flg[NJ_MAX_DIC]; //[Internal] Filtered search target candidate confirmation flag for each dictionary (0: Candidate to undergo non-filtered search, 1: Candidate to undergo filtered search)
    IWNN_OPTION option_data;    //[Internal] Option Settings


    IWNN_DIC_SET dicSet;         //Dictionary set structure
    IWNN_DIC_SET tmp_dic_set;    //[Internal] Temporary dictionary set
    IWNN_STATE   state;          //State setting for state-adaptive prediction

    struct {
        u8   commit_status; //[Internal] Learning dictionary current operation status flag (0: No current operation, 1: Operation underway)
        NN_PADDING1;
        u16  save_top;      //[Internal] Learning dictionary current operation status - Start of registration range -
        u16  save_bottom;   //[Internal] Learning dictionary current operation status - End of registration range -
        u16  save_count;    //[Internal] Learning dictionary current operation status - Number of registrations -
    } learndic_status;
    wchar_t tmp[NJ_MAX_RESULT_LEN + NJ_TERM_LEN];    //[Internal] General-purpose work region
} IWNN_CLASS;
/******************************************/
/*   Structures (nj_lib.h)  Note: Rename support */
/******************************************/
/* Please see man pages for details 
 
 */
typedef struct {
    IWNN_RESULT *result;  //Process result information
    IWNN_CLASS  *iwnn;    //Parsing information class
} IWNN_MERGE_RESULT;


/******************************************/
/*   Structures (nj_dicif.h)  Note: Rename support */
/******************************************/
/* Please see man pages for details 
 
 */
typedef struct {
    u16                   forePartsOfSpeech;        //Previous part of speech
    u16                   backPartsOfSpeech;        //Following part of speech
    u16                   connect;                  //Connection flag
    u16                   undo;                     //Undo flag
} IWNN_LEARN_WORD;

/* Please see man pages for details 
 
 */
typedef struct {
    IWNN_SEARCH_CONDITION *       condition;     //Search condition
    IWNN_SEARCH_LOCATION_SET *    location;      //Search cursor
    IWNN_DIC_SET *                dicSet;        //Dictionary set to be searched
    IWNN_WORD *                   word;          //Word information
    IWNN_LEARN_WORD *            learnWord;     //Word registration information
    wchar_t *                   stroke;        //Location to get reading string
    wchar_t *                   string;        //Location to get notation string
    wchar_t *                   additional;    //Location to get additional information string
    u16                         strokeSize;    //The size of "stroke" (use of byte units or character units changes based on the message)
    u16                         strSize;       //The size of "string" (use of byte units or character units changes based on the message)
    u32                         additionalSize;//The size of "additional" (use of byte units or character units changes based on the message)
    u16                         dicIdx;        //Dictionary mount location
    NN_PADDING2;
} IWNN_PROGRAM_DIC_MESSAGE;

/******************************************/
/*   Function types (nj_dicif.h)  Note: Rename support */
/******************************************/
/* Please see man pages for details 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 */
typedef s16(*IWNN_PROGRAM_DIC_IF)(IWNN_CLASS *iwnn, u16 request, IWNN_PROGRAM_DIC_MESSAGE *message);


/******************************************/
/*   Structures (njfilter.h)  Note: Rename support */
/******************************************/
/* Please see man pages for details 
 
 */
typedef struct {
    IWNN_SEARCH_CONDITION * condition; //Search condition
    wchar_t *   stroke;        //Reading string buffer
    wchar_t *   string;        //Notation string buffer
    wchar_t *   additional;    //Additional information buffer
    s16    strokeLen;          //Reading character array length
    s16    strLen;             //Notation character array length
    s32    additionalLen;      //Additional information character array length
    IWNN_RESULT * result;      //Word information
    void *   option;           //Filter options
} IWNN_PHASE1_FILTER_MESSAGE;

/* Please see man pages for details 
 
 */
typedef struct {
    IWNN_RESULT * result;         //Word information
    void *   option;              //Filter options
} IWNN_PHASE2_FILTER_MESSAGE;

/******************************************/
/*   Function types (njfilter.h)  Note: Rename support */
/******************************************/
/* Please see man pages for details 
 
 
 
 
 
 
 
 
 
 
 
 
 */
typedef s16(*IWNN_PHASE1_FILTER_IF)(IWNN_CLASS * iwnn, IWNN_PHASE1_FILTER_MESSAGE *message);


/* Please see man pages for details 
 
 
 
 
 
 
 
 */
typedef s16(*IWNN_PHASE2_FILTER_IF)(IWNN_CLASS * iwnn, IWNN_PHASE2_FILTER_MESSAGE *message);


/******************************************/
/*   Structures (ex_giji.h)  Note: Rename support  */
/******************************************/
/* Please see man pages for details 
 
 */
typedef NJ_GIJISET IWNN_PSEUDO_SET;


/******************************************/
/*            IWNN class proper              */
/******************************************/
namespace mw { 
namespace iwnn {
    
    
    /* Please see man pages for details 
    
        
    */
    class IWNN
    {
    public:        
        IWNN();        
        virtual ~IWNN();
       
        /*
        
               
        
        
        
                              
        
        
        
        

        
        
        
        */        
        static s16 MmxSplitWord(IWNN_CLASS* iwnn,  IWNN_RESULT* results, u8* processLen, const wchar_t* input);

        /*
        
               
        
        
        
        
        
        
        */        
        static s16 MmxGetPartsOfSpeech(IWNN_CLASS* iwnn, const IWNN_RESULT* result);

        /*
        
               
        
        
        
        
        
        
        
        
        
        
        */        
        static s16 MmxGetReading(IWNN_CLASS* iwnn, 
                          wchar_t* reading, 
                          u8* stemLen, 
                          const IWNN_RESULT* result, 
                          u16 readingSize);

        /*
        
               
        
        
        
        
        
        
        
        
        
        */        
        static s16 MmxSelect(IWNN_CLASS* iwnn, 
                      const IWNN_RESULT* result, 
                      const wchar_t* reading, 
                      u8 independentFlag, 
                      u8 connect);

        /*
        
               
               
               
        
        
        
        
        
                              
        
        
        
        
        
        */        
        static s16 NjxAnalyze(IWNN_CLASS* iwnn, 
                       IWNN_RESULT* result, 
                       const IWNN_CHARSET* charSet, 
                       const wchar_t* reading, 
                       const IWNN_ANALYZE_OPTION* option);

        /*
        
               

        
        
                
        
        
        
        
        
        
        
        
        */        
        static s16 NjxConversion(IWNN_CLASS* iwnn, 
                          IWNN_RESULT* results, 
                          const wchar_t* reading, 
                          u8 analyzeLevel, 
                          u8 devidePos);

        /*
        
        
        
        
        
        
        
        
        
        */         
        static s16 NjxAllCandidates(IWNN_CLASS* iwnn, 
                             IWNN_RESULT* result, 
                             const IWNN_RESULT* target, 
                             u16 candidateNum);

        /*
        
               
        
        
        
        
        
        
        
        */         
        static s16 NjxGetStrokeWord(IWNN_CLASS* iwnn, IWNN_RESULT* result, const wchar_t* reading);

        /*
        
               

        
        
        
        
        
        
        
        */ 
        static s16 NjxGetStroke(wchar_t* buf, 
                         const IWNN_CLASS* iwnn, 
                         const IWNN_RESULT* result, 
                         u16 bufSize);

        /*
        
               

        
        
        
        
        
        
        
        */ 
        static s16 NjxGetCandidate(wchar_t* buf,
                            const IWNN_CLASS* iwnn,
                            const IWNN_RESULT* result, 
                            u16 bufSize);

        /*
        
               

        
        
        
        
        
        
        */ 
        static s16 NjxSearchWord(IWNN_CURSOR* cursor,const IWNN_CLASS* iwnn);

        /*
        
               

        
                                  
        
        
        
        
        
        
        */ 
        static s16 NjxGetWord(IWNN_CURSOR* cursor, IWNN_RESULT* result,const IWNN_CLASS* iwnn);

        /*
        
               
        
        
        
        
        
                                   
                                   
        
        
        
        
        
        */ 
        static s16 NjxCheckDic(IWNN_DIC_HANDLE handle,
                        const IWNN_CLASS* iwnn, 
                        u8 dicType, 
                        u8 restore, 
                        u32 size);

        /*
        
               
        
        
        
        
        
        
        */ 
        static IWNN_DIC_HANDLE NjxGetDicHandle(const IWNN_CLASS* iwnn, const IWNN_RESULT* result);

        /*
        
               
        
        
        
        
                                  
                                  
                                  
        
                                  
                                  
        
        
        
        */ 
        static s16 NjxAddWord(IWNN_CLASS* iwnn, const IWNN_WORD_INFO* word, u8 type,u8 connect);

        /*
        
               
               
        
        
        
        
        
        */ 
        static s16 NjxDeleteWord(IWNN_CLASS* iwnn, const IWNN_RESULT* result);

        /*
        
        
        
        
        
                                    
                                    
                                    
                                    
                                    
        
        
        
        
        */ 
        static s16 NjxCreateDic(IWNN_DIC_HANDLE handle, const IWNN_CLASS* iwnn,s8 type, u32 size);

        /*
        
               
        
        
        
        
        
        
        
        
        
        */ 
        static s32 NjxGetAdditionalInfo(wchar_t* buf, 
                                 const IWNN_CLASS* iwnn, 
                                 const IWNN_RESULT* result, 
                                 s8 index, 
                                 u32 bufSize);

        /*
        
               
                      
        
        
        
        
        
        */ 
        static s16 NjxSetState(IWNN_CLASS* iwnn, const IWNN_STATE* state);

        /*
        
               
                      
        
        
        
        
        
        */         
        static s16 NjxGetState(IWNN_CLASS* iwnn, IWNN_STATE* state);

        /*
        
               
                    
        
        
        
        
        
        */ 
        static s16 NjxSetOption(IWNN_CLASS* iwnn, const IWNN_OPTION* option);

        /*
        
               
               
        
        
        
        
                                   
                                   
        
        
        
        */ 
        static s16 NjxSelect(IWNN_CLASS* iwnn, 
                      const IWNN_RESULT* learningResult, 
                      const IWNN_RESULT* preConfirmedResult, 
                      u8 connect);
        /*
        
               
                 
        
          
       
        
        
        
        
        
        */ 
        static s16 NjxUndo(IWNN_CLASS* iwnn, u16 undoCount);

        /*
        
               
        
        
        
        
        
        
        
        
        
        */ 
        static s16 NjxGetCharType(u8* stemType, 
                           u8* ancillaryWordType, 
                           const IWNN_CLASS* iwnn,
                           const IWNN_RESULT* result);

        /*
        
               
                
        
        
        
        
        
        
        */ 
        static s16 NjxInit(IWNN_CLASS* iwnn, const IWNN_OPTION* option);

        /*
        
               

        
        
        
                                   
                                   
        
        
        
        */ 
        static s16 NjxChangeDicType(IWNN_DIC_HANDLE handle, const IWNN_CLASS* iwnn,u8 direct);
        /*
        
               
        
        

        
        
        
        
        
        
        
        
        
        */ 
       static s16 NjxCheckAdditionalInfo(IWNN_CLASS* iwnn, 
                                   u8 dicType, 
                                   const IWNN_DIC_HANDLE handle, 
                                   const void* additionalInfo, 
                                   u32 size);
        /*
        
               
        
        
        
        
        
        
        
        
        
        
        */ 
        static s16 NjxMergeWordList(IWNN_MERGE_RESULT* wordList, 
                             s32 listMax, 
                             u32 mode, 
                             const IWNN_CLASS* iwnn, 
                             const IWNN_RESULT* result,  
                             s32 num);
        /*
        
               

        
        
        

        
        
        */ 
        static s16 NjxGetWordInfo(IWNN_WORD_INFO* info, 
                           const IWNN_CLASS* iwnn, 
                           const IWNN_RESULT* result);

        /*
        
               
        
        
        
        
        
        
        
        */ 
        static s16 NjxManageLearnDic(const IWNN_CLASS* iwnn, u32 operation);

        /*
        
        
        
        

        
        
        */ 
        static s32 NjxGetFlashDicCacheSize(const IWNN_CLASS* iwnn, const IWNN_FILE* fileStream);

        /*
        
        
        
        
        
        
        
        
        
        
        */ 
        static s32 NjxSetFlashDicInfo(IWNN_FLASH_DIC_INFO* flashDicInfo, 
                               const IWNN_CLASS* iwnn, 
                               const IWNN_FILE* fileStream, 
                               const u8* cacheArea, 
                               u32 cacheSize);

        /*
        
        
        
        
        
        
        
        */ 
        static s16 NjdRuleGetPartsOfSpeech (IWNN_DIC_HANDLE ruleHandle, u8 type);

        /*
        
        
        
        

        
        
         */ 
        static s16 NjexPhase1Filter(IWNN_CLASS * iwnn, IWNN_PHASE1_FILTER_MESSAGE *message);

        /*
        
        
        
        

        
        
         */ 
        static s16 NjexPhase2Filter(IWNN_CLASS * iwnn, IWNN_PHASE2_FILTER_MESSAGE *message);

        /*
        
               
        
        
        
        
        
                       
                       
                       
                       
                       

        
        
        
         */ 
        static s16 NjexGetPseudo(IWNN_RESULT *result,IWNN_CLASS *iwnn, wchar_t *reading, u8 type);

        /*
        

        
        
                                
                                
                                
                                
                                
                                
        

        
        
         */ 
        static s16 NjexPseudoDic(IWNN_CLASS* iwnn, u16 request, IWNN_PROGRAM_DIC_MESSAGE* message);

        /*
        
        
        
        
                         
                          
                          
                          
                          
        

        
                    
                    
                    
                    
                    
        
         */ 
        static s16 NjexNumericCharPseudoDic(IWNN_CLASS* iwnn, u16 request, IWNN_PROGRAM_DIC_MESSAGE* message);

        /*
        

        
        
                           
                           
                           
                           
                           
        

        
                    
                    
                    
                    
                    
        
        */ 
        static s16 NjexNumericForecastPseudoDic(IWNN_CLASS* iwnn, u16 request, IWNN_PROGRAM_DIC_MESSAGE* message);

        /*
        

        
        
                           
                           
                           
                           
                           
                           
        

        
        
         */ 
        static s16 NjexPredictionPseudoDic(IWNN_CLASS* iwnn, u16 request, IWNN_PROGRAM_DIC_MESSAGE* message);

        /*
        

        
        
        

        
        
         */ 
        static s16 NjdConnectTest(IWNN_SEARCH_CONDITION *condition, u16 forePartsOfSpeech, u16 backPartsOfSpeech);

    };


}
}
#endif
