/*-----------------------------------------------------------------------*
 Project: iWnn API Analyze Conversion Demo: Pseudo Dictionary Sample
 File:    ex_pseudo_ascii.cpp

 Copyright (C) 2009-2012 Nintendo Co., Ltd. All rights reserved.

 These coded instructions, statements, and computer programs contain
 proprietary information of Nintendo of America Inc. and/or Nintendo
 Company Ltd., and are protected by Federal copyright law. They may
 not be disclosed to third parties or copied or duplicated in any form,
 in whole or in part, without the prior written consent of Nintendo.

 $Revision: $
 *-----------------------------------------------------------------------*/
#include <mw/iwnn/iwnnCTR.h>
#include "ex_pseudo_ascii.h"



//Standard settings for an ASCII pseudo candidate set
static IWNN_PSEUDOSET_ASCII AsciiPseudoSet = {
    1,
    {
        NJ_TYPE_ASCII_HALF_TO_FULL, //Convert ASCII to corresponding double-byte characters
        0
    }
};
//ASCII character string length
#define NJQG_HALF_CHAR_LEN  1
//Double-byte string length
#define NJQG_FULL_CHAR_LEN  1


/*
 

 
   
 
 
 
 
*/
#define NJQG_CONV_TO_WCHAR(x)                                            \
    ((u16)(((*((u8*)(x)) << 8) & 0xff00) | (*(((u8*)(x))+1) & 0xff)))

/*
 
 
 
 
 
 
*/
#define NJG_COPY_INT16_TO_CHAR(to, from)                                \
    { ((u8*)(to))[0] = (u8)(((from) >> 8) & 0x00ff);        \
        ((u8*)(to))[1] = (u8)((from) & 0x00ff); }

/*
 
 
 
 
 
 
 
*/
#define NJQG_IS_HALF_ALPHABET_2BYTE(x) ( (((*x) >= NJQG_HALF_LOWER_A) &&    \
                                         ((*x) <= NJQG_HALF_LOWER_Z)) ? 1 : \
                                        ((((*x) >= NJQG_HALF_UPPER_A) &&    \
                                         ((*x) <= NJQG_HALF_UPPER_Z)) ? 2 : 0) )


/*
 
 
  
 
  
  
*/
#define NJQG_IS_HALF_NUMERIC_2BYTE(x) (  \
                                         (((*x) >= NJQG_HALF_NUMERIC_0) &&   \
                                           ((*x) <= NJQG_HALF_NUMERIC_9))    \
                                         ? 1 : 0)

/*
 
 
 
 
 
 
*/
#define NJQG_IS_HALF_SYMBOL_2BYTE(x)    ( (\
                                           ((*x) >= NJQG_HALF_SYMBOL_SPACE &&       \
                                           (*x) <= NJQG_HALF_SYMBOL_SLASH)     ||   \
                                           ((*x) >= NJQG_HALF_SYMBOL_COLON &&       \
                                            (*x) <= NJQG_HALF_SYMBOL_AT_MARK)  ||   \
                                           ((*x) >= NJQG_HALF_SYMBOL_LEFT_SQUARE_BRACKET &&    \
                                            (*x) <= NJQG_HALF_SYMBOL_GRAVE_ACCENT)          || \
                                           ((*x) >= NJQG_HALF_SYMBOL_LEFT_CURLY_BRACKET &&     \
                                            (*x) <= NJQG_HALF_SYMBOL_OVERLINE)                 \
                                        ) ? 1 : 0)

/*
 
 
 
 
 
 
 
 
*/
#define NJQG_CHAR_DIFF(a, b)    ((s16)(*(a) - *(b)))



//Internal function prototype declarations
static s16 GetPseudoAsciiResult(NJ_DIC_HANDLE rule,
                                       wchar_t *reading, u16 len,
                                       u8 type, IWNN_RESULT *pseudo);
static s16 GetPseudoAsciiAllResult(u16 pseudoForePos, u16 pseudoBackPos,
                                           wchar_t *reading, u16 len,
                                           IWNN_RESULT *pseudo, u8 pseudoType);
static s16 GetPseudoAscii(u16 pseudoForePos, u16 pseudoBackPos,
                                wchar_t *reading, u16 len, IWNN_RESULT *p, u8 pseudoType);
static s16 GetPseudoAsciiCandidate(IWNN_WORD *word, wchar_t *candidate, u16 size, u8 type);
static s16 GetAsciiString(IWNN_WORD *word, wchar_t *candidate, u16 size, u8 type);
static s16 NjqgSearchHalfToFullTable(wchar_t *reading, u16 *conv);


/*
 
 
 
 
                          
                          
                          
                          
                          
                          
                          
                          
                          
                          
 

 
 
*/
NJ_EXTERN s16 NjexPseudoAsciiDic(IWNN_CLASS *iwnn, u16 request, IWNN_PROGRAM_DIC_MESSAGE *message)
{

    s16 ret;                        //Function return value
    u8 type;                        //ASCII pseudo candidate type
    s16 i;                          //Loop counter
    IWNN_RESULT result;             //Processing result
    u16 len;                        //Reading string length
    IWNN_PSEUDOSET_ASCII *pseudoSetAscii;    //ASCII pseudo set

    //Select the ASCII pseudo set to use
    if ((message->dicIdx < NJ_MAX_DIC) &&
        (iwnn->dicSet.dic[message->dicIdx].extensionArea == NULL)) {
        //Use the default ASCII pseudo set if the frequency learning region is NULL
        pseudoSetAscii = &AsciiPseudoSet;
    } else {
        //Handle information stored in the frequency learning region as IWNN_PSEUDOSET_ASCII
        pseudoSetAscii = (IWNN_PSEUDOSET_ASCII *)(iwnn->dicSet.dic[message->dicIdx].extensionArea);
    }

    switch (request) {
    case NJG_OPERATION_SEARCH:    //First search
        if ((message->condition->operation != NJ_CURSOR_OPERATION_COMPLETE)
            || (message->condition->mode != NJ_CURSOR_MODE_FREQUENCY)) {
            //No support for anything other than forward lookup complete match frequency order
            message->location->location.status = NJ_STATUS_SEARCH_END_EXTENSION;
            return 0;
        }

        if (((iwnn->njcMode == 2) || 
             ((iwnn->njcMode == 0) && (iwnn->environment.type == NJ_ANALYZE_COMPLETE)))) {
            //ASCII pseudo candidates are used only when getting all candidates when using the psuedo candidate search state FULL_MODE_PSEUDO_PROC
            //

            //Check the number of ASCII pseudo candidate set settings
            for (i = 0; i < pseudoSetAscii->count; i++) {
                type = pseudoSetAscii->type[i];

                //Try creating an ASCII pseudo candidate
                ret = GetPseudoAsciiResult(message->dicSet->ruleHandle[0],
                                             message->condition->reading, message->condition->readingLen,
                                             type, &result);
                if (ret > 0) {
                    //The ASCII pseudo candidate was successfully created
                    message->location->location.current = i;    //Set the ASCII pseudo set number to current
                    message->location->location.currentInfo = 0x10;
                    message->location->cacheFrequency = NJ_GET_FREQUENCY_FROM_STEM(&(result.word));
                    message->location->location.status = NJ_STATUS_SEARCH_READY;
                    return 1;
                } else if (ret < 0) {
                    //Creating the pseudo candidate resulted in an error
                    message->location->location.status = NJ_STATUS_SEARCH_END_EXTENSION;
                    return ret;
                }
            }
        }
        //Could not create pseudo candidate
        message->location->location.status = NJ_STATUS_SEARCH_END_EXTENSION;
        return 0;

    case NJG_OPERATION_SEARCH_NEXT:    //Subsequent candidate searches
        //Find the pseudo candidate that can be created next
        //Start checking staring from the next ASCII pseudo set number after the previous one created
        for (i = (u8)(message->location->location.current + 1); i < pseudoSetAscii->count; i++) {
            type = pseudoSetAscii->type[i];

            //Try creating an ASCII pseudo candidate
            ret = GetPseudoAsciiResult(message->dicSet->ruleHandle[0],
                                         message->condition->reading, message->condition->readingLen,
                                         type, &result);
            if (ret > 0) {
                //The ASCII pseudo candidate was successfully created
                message->location->location.current = i;    //Set the ASCII pseudo set number to current
                message->location->location.currentInfo = 0x10;
                message->location->cacheFrequency = NJ_GET_FREQUENCY_FROM_STEM(&(result.word));
                message->location->location.status = NJ_STATUS_SEARCH_READY;
                return 1;
            } else if (ret < 0) {
                //Creating the pseudo candidate resulted in an error
                message->location->location.status = NJ_STATUS_SEARCH_END_EXTENSION;
                return ret;
            }
        } 
        //Could not create pseudo candidate
        message->location->location.status = NJ_STATUS_SEARCH_END_EXTENSION;
        return 0;

    case NJG_OPERATION_GET_WORD_INFO:    //Get word information
        //Get the ASCII pseudo candidate type and create an ASCII pseudo candidate

        //Check the range of ASCII pseudo candidates
        if ((u8)message->location->location.current >= pseudoSetAscii->count) {
            return -1;
        }
        type = pseudoSetAscii->type[(u8)message->location->location.current];
        ret = GetPseudoAsciiResult(message->dicSet->ruleHandle[0],
                                     message->word->reading, NJ_GET_READING_LEN_FROM_STEM(message->word),
                                     type, &result);
        if (ret > 0) {
            //The pseudo candidate can be created because the check is made during the candidate search
            *(message->word) = result.word;
            message->word->stem.location = message->location->location;
            return 0;
        } else {
            return -1;
        }

    case NJG_OPERATION_GET_STROKE:    //Get reading
        //Create a reading string
        len = NJ_GET_READING_LEN_FROM_STEM(message->word);
        if (message->strokeSize < ((len + NJ_TERM_LEN) * sizeof(wchar_t))) {
            //An error results if there is not enough buffer memory for storage
            return -1;
        }
        nj_strncpy(message->stroke, message->word->reading, len);
        message->stroke[len] = NJ_CHAR_NUL;
        return (s16)len;

    case NJG_OPERATION_GET_STR:    //Get notation
        //Create a notation string

        //Check the range of pseudo candidates
        if ((u8)message->word->stem.location.current >= pseudoSetAscii->count) {
            return -1;
        }
        type = pseudoSetAscii->type[(u8)message->word->stem.location.current];
        ret = GetPseudoAsciiCandidate(message->word, message->string, message->strSize, type);
        return ret;

    case NJG_OPERATION_GET_ADDITIONAL:    //Get additional information
    case NJG_OPERATION_LEARN:             //Learning
    case NJG_OPERATION_UNDO_LEARN:        //Undo learning
    case NJG_OPERATION_ADD_WORD:          //Word registration
    case NJG_OPERATION_DELETE_WORD:       //Delete word
        return 0;
    default:
        break;
    }
    return -1; //Error
}
/*
 
 
 
 
 
 
 
 
 
 
 
*/
static s16 GetPseudoAsciiResult(NJ_DIC_HANDLE rule,
                                       wchar_t *reading, u16 len,
                                       u8 type, IWNN_RESULT *pseudo)
{

    u16 forePos, backPos;    //Get the pre-position/post-position number
    s16 num;                 //Function return value


    //Check arguments
    if (rule == NULL) {
        return NJ_SET_ERR_VALUE(NJ_FUNC_NJC_GET_PSEUDO_RESULT, NJ_ERR_NO_RULE_DIC);
    }
    if ((reading == NULL) || (len == 0)) {
        return NJ_SET_ERR_VALUE(NJ_FUNC_NJC_GET_PSEUDO_RESULT, NJ_ERR_PARAM_READING_NULL);
    }
    if (pseudo == NULL) {
        return NJ_SET_ERR_VALUE(NJ_FUNC_NJC_GET_PSEUDO_RESULT, NJ_ERR_PARAM_RESULT_NULL);
    }

    //Check the range of pseudo candidate types of the argument
    if ((type < NJ_TYPE_ASCII_MIN_VALUE) || (type > NJ_TYPE_ASCII_MAX_VALUE)) {

        /* Please see man pages for details 
         
         
         
         
         
         */
        return 0;
    }

    //Get the pseudo candidate part of speech number from the rule dictionary
    forePos = mw::iwnn::IWNN::NjdRuleGetPartsOfSpeech(rule, NJ_PARTS_OF_SPEECH_PSEUDO_FORE);  //Get the part of speech number of the last pseudo
    backPos = mw::iwnn::IWNN::NjdRuleGetPartsOfSpeech(rule, NJ_PARTS_OF_SPEECH_PSEUDO_BACK);  //Get the part of speech number of the next pseudo

    if ((forePos == 0) || (backPos == 0)) {
        //Failedl to get the pseudo part of speech number
        /* Please see man pages for details 
         
         
         
         
         
         
         
         
         
         
         
         
         */
        return 0;
    }

    //num = 1: Obtained candidate successfully, 0: Failed
    num = GetPseudoAsciiAllResult(forePos, backPos, reading, len, pseudo, type);

    return num;
}

/*
 
 
 
 
 
 
 
 
 
 
 
*/
static s16 GetPseudoAsciiAllResult(u16 pseudoForePos, u16 pseudoBackPos,
                                           wchar_t *reading, u16 len,
                                           IWNN_RESULT *pseudo, u8 pseudoType)
{

    s16  ret;     //Function return value
    s16  i;       //Loop counter


    //Initialization
    ret = 0;
    pseudo->operationId = 0;
    pseudo->word.reading               = NULL;
    pseudo->word.stem.frequency        = 0;
    pseudo->word.stem.type             = NJ_TYPE_UNDEFINE;
    pseudo->word.ancillary.frequency         = 0;

    pseudo->word.stem.location.handle       = NULL;
    pseudo->word.stem.location.type         = NJ_DIC_H_TYPE_NORMAL;
    pseudo->word.stem.location.extensionArea     = NULL;
    for (i = 0; i < NJ_MAX_ADDITIONAL_INFO; i++) {
        pseudo->word.stem.location.addInfo[i] = NULL;
    }
    pseudo->word.stem.location.current      = 0;
    pseudo->word.stem.location.top          = 0;
    pseudo->word.stem.location.bottom       = 0;
    pseudo->word.stem.location.currentCache= 0;
    pseudo->word.stem.location.currentInfo = 0x10;  //num=1, offset = 0
    pseudo->word.stem.location.status       = NJ_STATUS_SEARCH_NO_INIT;
    pseudo->word.stem.location.attr         = 0x00000000;
    for (i = 0; i < NJ_MAX_PHRASE_CONNECT; i++) {
        pseudo->word.stem.location.relation[i] = 0;
    }

    //Check that the ASCII pseudo candidate can be created
    ret = GetPseudoAscii(pseudoForePos, pseudoBackPos, reading, len, pseudo, pseudoType);

    return ret;
}

/*
 
 
 
 
 
 
 
 
 
 
 
*/
static s16 GetPseudoAscii(u16 pseudoForePos, u16 pseudoBackPos,
                                wchar_t *reading, u16 len, IWNN_RESULT *p, u8 pseudoType)
{

    wchar_t   *ptrReading;       //Address of the reading information
    u16 len0;                    //String length of the ASCII pseudo candidate
    u16 cnt = 0;                 //Loop counter used for checking
    s16  ret;                    //Indicates whether a reading is given in the reading character table
                                 //0: Yes, one exists; Negative: No
    u16 candidate;               //Candidate string after conversion
    u16 tmpChar;                 //The reading character buffer


    //Get the character length (len0) after conversion
    ptrReading = reading;
    len0 = 0;
    candidate = 0;
    tmpChar = 0;

    while (cnt < len) {

        tmpChar = NJQG_CONV_TO_WCHAR(ptrReading);
        switch (pseudoType) {
        case NJ_TYPE_ASCII_HALF_TO_FULL:    //Convert ASCII to double-byte

                if(  NJQG_IS_HALF_ALPHABET_2BYTE(&tmpChar) == 0 &&
                     NJQG_IS_HALF_NUMERIC_2BYTE(&tmpChar) == 0 &&
                     NJQG_IS_HALF_SYMBOL_2BYTE(&tmpChar) == 0 ){  

                    //Check that a corresponding character exists in the supplemental the table
                    ret = NjqgSearchHalfToFullTable((wchar_t*)&tmpChar, &candidate);
                    if (ret == 0) {
                        //Return that there is no candidate because there is no corresponding character in the table
                        return 0;
                    }
                }
            len0 += NJQG_HALF_CHAR_LEN;
            ptrReading += NJQG_HALF_CHAR_LEN;
            cnt += NJQG_HALF_CHAR_LEN;
            break;
        default:
            //Unconditionally return that there is no candidate if the setting for the ASCII pseudo candidate set is incorrect
            return 0;

        }
    }
    //Do not create the ASCII psuedo candidate if it exceeds the maximum notation string length upon completion of analysis
    if (len0 > NJ_MAX_RESULT_LEN) {
        return 0; 
    }
    p->operationId = (NJ_OPERATION_CONVERT | NJ_FUNC_SEARCH | NJ_DIC_PSEUDO);
    p->word.reading = reading;
    NJ_SET_READING_LEN_TO_STEM(&p->word, len);
    NJ_SET_CANDIDATE_LEN_TO_STEM(&p->word, len0);
    NJ_SET_FORE_PARTS_OF_SPEECH_TO_STEM(&p->word, pseudoForePos);
    NJ_SET_BACK_PARTS_OF_SPEECH_TO_STEM(&p->word, pseudoBackPos);
    NJ_SET_WORD_TYPE_TO_STEM(&p->word, NJ_TYPE_UNDEFINE);
    NJ_SET_FREQUENCY_TO_STEM(&p->word, PSEUDO_GET_SCORE(len));

    return 1;
}

/*
 
 
 
 
 
 
 
 
*/
static s16 GetPseudoAsciiCandidate(IWNN_WORD *word, wchar_t *candidate, u16 size, u8 type)
{

    s16 len = 0;    //Function return value


    //Generate a notation string
    len = GetAsciiString(word, candidate, size, type);

    if (len <= 0) {
        return NJ_SET_ERR_VALUE(NJ_FUNC_NJC_GET_CANDIDATE, NJ_ERR_BUFFER_NOT_ENOUGH);
    }
    return len;
}

/*
 
 
 
 
 
 
 
 
 
*/
static s16 GetAsciiString(IWNN_WORD *word, wchar_t *candidate, u16 size, u8 type)
{

    wchar_t   *reading;     //Address of the reading information
    wchar_t   *dst;         //Address of the candidate result
    u16 len;                //Byte length of the reading string
    u16 cnt;                //Loop counter
    u16 tmpChar;            //The reading character buffer
    u16 tmpConversion;      //Conversion candidate string storage buffer
    s16  ret;               //Function return value


    reading = word->reading;
    dst = candidate;
    len = NJ_GET_READING_LEN_FROM_STEM(word);
    if ((u16)((NJ_GET_CANDIDATE_LEN_FROM_STEM(word) + NJ_TERM_LEN) * sizeof(wchar_t)) > size) {
        //An error results if data does not fit in the storage buffer
        return -1; 
    }

    //Initial settings
    cnt = len;
    tmpConversion = 0;

    while (cnt > 0) {

        tmpChar = NJQG_CONV_TO_WCHAR(reading);
        switch (type) {
        case NJ_TYPE_ASCII_HALF_TO_FULL:    //Convert ASCII to double-byte
                if(  NJQG_IS_HALF_ALPHABET_2BYTE(&tmpChar) > 0 ||
                     NJQG_IS_HALF_NUMERIC_2BYTE(&tmpChar) > 0 ||
                     NJQG_IS_HALF_SYMBOL_2BYTE(&tmpChar) > 0 ){  


                    //Check that a corresponding character exists in the supplemental the table
                    ret = NjqgSearchHalfToFullTable((wchar_t*)&tmpChar, &tmpConversion);
                    if (ret == 0) {
                         //Cannot be converted because there is no entry in the table
                        return -1;
                    }
                    NJG_COPY_INT16_TO_CHAR(dst, tmpConversion);
                }
            dst += NJQG_HALF_CHAR_LEN;
            reading += NJQG_HALF_CHAR_LEN;
            cnt -= NJQG_HALF_CHAR_LEN;
            break;
        default:
            //Code executed if the setting of the pseudo candidate set is incorrect
            return -1;

        }
    }
    *dst = NJ_CHAR_NUL;

    return (s16)NJ_GET_CANDIDATE_LEN_FROM_STEM(word);
}

/*
 
 
 
 
 
 
 
 
*/
static s16 NjqgSearchHalfToFullTable(wchar_t *reading, u16 *conv)
{

    u16 left;             // Binary search range: Beginning
    u16 right;            // Binary search range: End
    u16 mid = 0;          // Binary search range: Search position
    s16  cmp;              // Comparison result
    s16  found = 0;        // Match confirmation flag for search

    //2-part search for character that matches the search character
    left = 0;
    right = NJQG_EXP_HALF_TO_FULL_NUM - 1;
    while (left <= right) {
        mid = left + ((right - left) / 2);
        cmp = NJQG_CHAR_DIFF(reading, &NjqgAsciiHalfToFullTable[mid].src);
        if (cmp < 0) {
            //Located ahead (to the left) of the search position
            right = mid - 1;
        } else if (cmp > 0) {
            //Located behind (to the right) of the search position
            left = mid + 1;
        } else {
            //Search target match
            //Set the character resulting after conversion
            found = 1;
            *conv = NjqgAsciiHalfToFullTable[mid].dst;
            break;
        }
        if (((mid <= 0) && (cmp <= 0)) ||
            ((mid >= (NJQG_EXP_HALF_TO_FULL_NUM - 1)) && (cmp > 0))) {
            //Search before the beginning of the additional character table (ASCII) and after the end
            break;
        }
    }
    if ( found == 0 ) {
        //Search target mismatch
        return 0;
    }
    //Search target match
    return 1;
}

