/*-----------------------------------------------------------------------*
 Project: iWnn API Analyze Conversion Demo: Main Function
 File:    main.cpp

 Copyright (C)2009-2012 Nintendo Co., Ltd.  All rights reserved.

 These coded instructions, statements, and computer programs contain
 proprietary information of Nintendo of America Inc. and/or Nintendo
 Company Ltd., and are protected by Federal copyright law.  They may
 not be disclosed to third parties or copied or duplicated in any form,
 in whole or in part, without the prior written consent of Nintendo.

 $Revision: $
 *-----------------------------------------------------------------------*/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <nn/os.h>
#include <nn/dbg.h>
#include <nn/fs.h>
#include <nn/fnd.h>
#include <nn/applet.h>
#include <mw/iwnn/iwnnCTR.h>
#include "fprintf.h"
#include "ex_pseudo_ascii.h"

#define     MAX_LEARN_COUNT 1000     //Number of learning dictionary indices
#define     MAX_DEMO_CANDIDATE 10    //Number of prediction/multiple phrase candidates obtained
#define     MIN_DEMO_CANDIDATE_LEN 3 //Minimum candidate length for candidate filtering

#define     LEARN_DIC_SIZE (80 + 42*MAX_LEARN_COUNT) //Learning dictionary size


s32    readDic( u8 **dicData,char *dicName);          //Function for loading dictionary data
s32    loadReading(wchar_t *ptrReading,char *fileName,s32 stringSize);     //Function for loading reading strings
s16    demoAnalyze(IWNN_CLASS *environment, wchar_t *reading, u16 select);       //Predictive conversion demo function
s16    demoConversion(IWNN_CLASS *environment, wchar_t *reading, u16 select);    //Multiple phrase conversion demo function
void   releaseDic(void);    //Function for unmounting dictionary data
u16    wcharStringLen(wchar_t *c);  //Function for getting the length of the wchar string


s16 MyCharTypeFilter(IWNN_CLASS * iwnn, IWNN_PHASE1_FILTER_MESSAGE *message);//Dictionary lookup filter function
s16 MyCandidateLenFilter(IWNN_CLASS * iwnn, IWNN_PHASE2_FILTER_MESSAGE *message);//Candidate filter function

wchar_t     analyzePseudoReading2[NJ_MAX_LEN+NJ_TERM_SIZE]; //Predictive conversion reading storage region 3
wchar_t     analyzeReading[NJ_MAX_LEN+NJ_TERM_SIZE];     //Predictive conversion reading storage region 1
wchar_t     analyzePseudoReading[NJ_MAX_LEN+NJ_TERM_SIZE]; //Predictive conversion reading storage region 2
wchar_t     conversionReading[NJ_MAX_LEN+NJ_TERM_SIZE];  //Multiple phrase conversion reading storage region 1
wchar_t     conversionReading2[NJ_MAX_LEN+NJ_TERM_SIZE];        //Multiple phrase conversion reading storage region 2
wchar_t     conversionReading3[NJ_MAX_LEN+NJ_TERM_SIZE];        //Multiple phrase conversion reading storage region 3
IWNN_CLASS    iwnnClass;                                //iWnn class structure
IWNN_OPTION   funcOption;                               //iWnn setting options
IWNN_DIC_SET  dicSet;                                   //Dictionary set structure
s16    dicCnt;                                   //Mounted dictionary count
IWNN_FLASH_DIC_INFO    flashDicInfo[NJ_MAX_DIC];       //FLASH type dictionary information storage region

nn::fnd::ExpHeap            dicHeap;       //Instance for dynamic memory allocation
nn::fs::FileOutputStream*   logFile=NULL;  //Log file output instance
nn::fs::FileStream*    inFileUbase1=NULL;  //Instance for working with Integrated Dictionary File 1.
nn::fs::FileStream*    inFileUbase2=NULL;  //Instance for working with Integrated Dictionary File 2.
u8* ptrUbase1=NULL;                        //Integrated Dictionary 1 handle pointer
u8* ptrUbase2=NULL;                        //Integrated Dictionary 2 handle pointer





/*
 
               
               
               
               
               
               
               

 

 
 */
extern "C" int nnMain()
{
    s32    fileSize;
    s16    ret = 0;
    u8     bom[2]={0xff,0xfe};


    //Enabling the applet library
    nn::applet::Enable();

    //fs initialization
    nn::fs::Initialize();

    //SD Card initialization
    nn::Result result = nn::fs::MountSdmc();
    if( result.IsFailure() ){
        return -1;
    }    

    //Result output file (Result1.txt) initialization
    logFile = new (nn::fs::FileOutputStream);
    if(!logFile){
        nn::fs::Unmount("sdmc:");
        return -1;
    }

    logFile->Initialize(FPRINTF_FILE_PATH,true);
    //Write BOM symbol
    logFile->Write(bom,sizeof(bom),true);

    Fprintf(logFile,L"demo start!\n");

    //Initialize the heap region
    dicHeap.Initialize(nn::os::GetDeviceMemoryAddress(), nn::os::GetDeviceMemorySize(), nn::os::ALLOCATE_OPTION_LINEAR);
    
    
    dicCnt = 0;
    memset(&iwnnClass, 0x00, sizeof(IWNN_CLASS));
    memset(&dicSet, 0x00, sizeof(IWNN_DIC_SET));
    memset(&flashDicInfo, 0x00, (NJ_MAX_DIC) * sizeof(IWNN_FLASH_DIC_INFO)); 
    memset(analyzePseudoReading2, 0x00, (NJ_MAX_LEN+NJ_TERM_SIZE) * sizeof(wchar_t));
    memset(analyzeReading, 0x00, (NJ_MAX_LEN+NJ_TERM_SIZE) * sizeof(wchar_t));
    memset(analyzePseudoReading, 0x00, (NJ_MAX_LEN+NJ_TERM_SIZE) * sizeof(u16));
    memset(conversionReading, 0x00, (NJ_MAX_LEN+NJ_TERM_SIZE) * sizeof(wchar_t));
    memset(conversionReading2, 0x00, (NJ_MAX_LEN+NJ_TERM_SIZE) * sizeof(wchar_t));
    memset(conversionReading3, 0x00, (NJ_MAX_LEN+NJ_TERM_SIZE) * sizeof(wchar_t));
    
    /********************************/
    /* 1. Dictionary set structure settings       */
    /********************************/
    NN_LOG("  iWnn Demo: Mounting dictionaries.\n");

    dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION] = NULL;
    dicSet.ruleHandle[NJ_MODE_TYPE_PREDICTION] = NULL;
    dicSet.ruleHandle[NJ_MODE_TYPE_MORPHOLIZE] = NULL;
    
    //Learning dictionary
    dicSet.dic[dicCnt].handle = (IWNN_DIC_HANDLE)dicHeap.Allocate(LEARN_DIC_SIZE);
        
    if (dicSet.dic[dicCnt].handle == NULL) {
        Fprintf(logFile, L"error:learn-dic malloc\n");
        logFile->Finalize();
        nn::fs::Unmount("sdmc:");
        delete logFile;
        dicHeap.Finalize();
        return -1;
    }
    ret = mw::iwnn::IWNN::NjxCreateDic(dicSet.dic[dicCnt].handle, &iwnnClass, NJ_CREATE_DIC_TYPE_LEARN, LEARN_DIC_SIZE);

    if (ret < 0) {
        releaseDic();
        Fprintf(logFile, L"error:learn-dic create\n");
        logFile->Finalize();
        nn::fs::Unmount("sdmc:");
        delete logFile;
        dicHeap.Finalize();
        return -1;
    }
    
    dicSet.dic[dicCnt].type                               = NJ_DIC_HANDLE_TYPE_NRM;
    dicSet.dic[dicCnt].limit                              = 0;
    dicSet.dic[dicCnt].extensionArea                           = NULL;
    dicSet.dic[dicCnt].searchCache                           = NULL;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_CONVERSION].base = 501;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_CONVERSION].high = 1000;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_PREDICTION].base = 501;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_PREDICTION].high = 1000;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_MORPHOLIZE].base = 501;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_MORPHOLIZE].high = 1000;
    dicCnt++;

    //Integrated dictionary 1 (njubase1.a)
    inFileUbase1 = new(nn::fs::FileStream);
    inFileUbase1->Initialize(L"sdmc:/iWnn/dic/JA/32/njubase1.a",nn::fs::OPEN_MODE_READ);
    
    fileSize = mw::iwnn::IWNN::NjxGetFlashDicCacheSize(&iwnnClass, (IWNN_FILE*)inFileUbase1);    
    
    if(fileSize <= 0){   
        releaseDic();
        Fprintf(logFile, L"error:[%d]dic open\n", dicCnt);
        logFile->Finalize();
        nn::fs::Unmount("sdmc:");
        delete logFile;
        dicHeap.Finalize();
        return -1;
    }    
    
    ptrUbase1 = NULL;
    ptrUbase1  = (u8*)(dicHeap.Allocate(fileSize));
    if (mw::iwnn::IWNN::NjxSetFlashDicInfo(&(flashDicInfo[dicCnt]), &iwnnClass, (IWNN_FILE*)inFileUbase1, ptrUbase1, fileSize) < 0) {       
        releaseDic();
        Fprintf(logFile, L"error:[%d]dic open\n", dicCnt);
        logFile->Finalize();
        nn::fs::Unmount("sdmc:");
        delete logFile;
        dicHeap.Finalize();
        return -1;
        
    } else {
        dicSet.dic[dicCnt].handle = (IWNN_DIC_HANDLE)&(flashDicInfo[dicCnt]);
        dicSet.dic[dicCnt].addInfo[0] = NULL;
        dicSet.dic[dicCnt].addInfo[1] = NULL;
    }

    /* In this iWnn library, all of the dictionary data is expanded to memory.
       The iWnn functions that issue file access are:
        - NjxGetFlashDicCacheSize
        - NjxSetFlashDicInfo
        - NjxCheckDic
       and those functions only.
       With this program, inFileUbase1 after this will become unnecessary so, the file is closed here
    */
    inFileUbase1->Finalize();
    delete inFileUbase1;
    inFileUbase1=NULL;
        
    dicSet.dic[dicCnt].type                               = NJ_DIC_HANDLE_TYPE_ON_FLASH;
    dicSet.dic[dicCnt].limit                              = 0;
    dicSet.dic[dicCnt].extensionArea                           = NULL;
    dicSet.dic[dicCnt].searchCache                           = NULL;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_CONVERSION].base = 400;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_CONVERSION].high = 550;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_PREDICTION].base = 100;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_PREDICTION].high = 560;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_MORPHOLIZE].base = 0;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_MORPHOLIZE].high = 10;
    dicCnt++;
    
    //Integrated dictionary 2 (njubase2.a)
    inFileUbase2 = new(nn::fs::FileStream);
    inFileUbase2->Initialize(L"sdmc:/iWnn/dic/JA/32/njubase2.a",nn::fs::OPEN_MODE_READ);
        
    fileSize = mw::iwnn::IWNN::NjxGetFlashDicCacheSize(&iwnnClass, (IWNN_FILE*)inFileUbase2);     
   
    if(fileSize <= 0){ 
        releaseDic();
        Fprintf(logFile, L"error:[%d]dic open\n", dicCnt);
        logFile->Finalize();
        nn::fs::Unmount("sdmc:");
        delete logFile;
        dicHeap.Finalize();
        return -1;
    }
    
    ptrUbase2 = NULL;
    ptrUbase2 = (u8 *)dicHeap.Allocate(fileSize);
    if (mw::iwnn::IWNN::NjxSetFlashDicInfo(&(flashDicInfo[dicCnt]), &iwnnClass, (IWNN_FILE*)inFileUbase2, ptrUbase2, fileSize) < 0) {  
        releaseDic();
        Fprintf(logFile, L"error:[%d]dic open\n", dicCnt);
        logFile->Finalize();
        nn::fs::Unmount("sdmc:");
        delete logFile;
        dicHeap.Finalize();
        return -1;        
    } else {
        dicSet.dic[dicCnt].type   = NJ_DIC_HANDLE_TYPE_ON_FLASH;
        dicSet.dic[dicCnt].handle = (IWNN_DIC_HANDLE)&(flashDicInfo[dicCnt]);
        dicSet.dic[dicCnt].addInfo[0] = NULL;
        dicSet.dic[dicCnt].addInfo[1] = NULL;
    }

    /* In this iWnn library, all of the dictionary data is expanded to memory.
       The iWnn functions that issue file access are:
        - NjxGetFlashDicCacheSize
        - NjxSetFlashDicInfo
        - NjxCheckDic
       and those functions only.
       With this program, inFileUbase2 after this will become unnecessary so, the file is closed here
    */
    inFileUbase2->Finalize();
    delete inFileUbase2;
    inFileUbase2=NULL;

    dicSet.dic[dicCnt].type                               = NJ_DIC_HANDLE_TYPE_ON_FLASH;
    dicSet.dic[dicCnt].limit                              = 2;
    dicSet.dic[dicCnt].extensionArea                           = NULL;
    dicSet.dic[dicCnt].searchCache                           = NULL;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_CONVERSION].base = 400;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_CONVERSION].high = 550;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_PREDICTION].base = 100;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_PREDICTION].high = 560;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_MORPHOLIZE].base = 0;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_MORPHOLIZE].high = 10;

    dicCnt++;

    //Auxiliary dictionary (njfzk.a)
    if( readDic( &dicSet.dic[dicCnt].handle, "sdmc:/iWnn/dic/JA/32/njfzk.a") <= 0 ){
        releaseDic();
        Fprintf(logFile, L"error:[%d]dic open\n", dicCnt);
        logFile->Finalize();
        nn::fs::Unmount("sdmc:");
        delete logFile;
        dicHeap.Finalize();
        return -1;
    }
    dicSet.dic[dicCnt].type                               = NJ_DIC_HANDLE_TYPE_NRM;
    dicSet.dic[dicCnt].limit                              = 0;
    dicSet.dic[dicCnt].extensionArea                           = NULL;
    dicSet.dic[dicCnt].searchCache                           = NULL;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_CONVERSION].base = 400;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_CONVERSION].high = 500;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_PREDICTION].base = 10;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_PREDICTION].high = 0;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_MORPHOLIZE].base = 400;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_MORPHOLIZE].high = 500;
    dicCnt++;    

    //Single kanji dictionary (njtan.a)
    if( readDic( &dicSet.dic[dicCnt].handle,"sdmc:/iWnn/dic/JA/32/njtan.a") <= 0 ){
        releaseDic();
        Fprintf(logFile, L"error:[%d]dic open\n", dicCnt);
        logFile->Finalize();
        nn::fs::Unmount("sdmc:");
        delete logFile;
        dicHeap.Finalize();
        return -1;
    }

    dicSet.dic[dicCnt].type                               = NJ_DIC_HANDLE_TYPE_NRM;
    dicSet.dic[dicCnt].limit                              = 0;
    dicSet.dic[dicCnt].extensionArea                           = NULL;
    dicSet.dic[dicCnt].searchCache                           = NULL;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_CONVERSION].base = 0;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_CONVERSION].high = 10;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_PREDICTION].base = 10;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_PREDICTION].high = 0;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_MORPHOLIZE].base = 0;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_MORPHOLIZE].high = 10;
    dicCnt++;

    //Extended no reading dictionary (njexyomi.a)
    if( readDic( &dicSet.dic[dicCnt].handle,"sdmc:/iWnn/dic/JA/32/njexyomi.a") <= 0 ){
        releaseDic();
        Fprintf(logFile, L"error:[%d]dic open\n", dicCnt);
        logFile->Finalize();
        nn::fs::Unmount("sdmc:");
        delete logFile;
        dicHeap.Finalize();
        return -1;
    }

    dicSet.dic[dicCnt].type                               = NJ_DIC_HANDLE_TYPE_NRM;
    dicSet.dic[dicCnt].limit                              = 0;
    dicSet.dic[dicCnt].extensionArea                           = NULL;
    dicSet.dic[dicCnt].searchCache                           = NULL;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_CONVERSION].base = 10;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_CONVERSION].high = 0;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_PREDICTION].base = 100;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_PREDICTION].high = 244;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_MORPHOLIZE].base = 10;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_MORPHOLIZE].high = 0;
    dicCnt++;

    // Implementation of pseudo-dictionary sample 1 
    dicSet.dic[dicCnt].type = NJ_DIC_HANDLE_TYPE_PROGRAM;
    dicSet.dic[dicCnt].extensionArea = NULL;
    dicSet.dic[dicCnt].handle = (u8*)(mw::iwnn::IWNN::NjexPseudoDic);
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_CONVERSION].base = NJ_DEFAULT_FREQUENCY_DIC_PRORAM_CONVERSION_BASE;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_CONVERSION].high = NJ_DEFAULT_FREQUENCY_DIC_PRORAM_CONVERSION_HIGH;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_PREDICTION].base = NJ_DEFAULT_FREQUENCY_DIC_PRORAM_PREDICTION_BASE;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_PREDICTION].high = NJ_DEFAULT_FREQUENCY_DIC_PRORAM_PREDICTION_HIGH;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_MORPHOLIZE].base = NJ_DEFAULT_FREQUENCY_DIC_PRORAM_MORPHOLIZE_BASE;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_MORPHOLIZE].high = NJ_DEFAULT_FREQUENCY_DIC_PRORAM_MORPHOLIZE_HIGH;
    dicSet.dic[dicCnt].searchCache = NULL;
    dicSet.dic[dicCnt].limit = 0;
    dicCnt++;

    // Implementation of pseudo-dictionary sample 2 
    dicSet.dic[dicCnt].type = NJ_DIC_HANDLE_TYPE_PROGRAM;
    dicSet.dic[dicCnt].extensionArea = NULL;
    dicSet.dic[dicCnt].handle = (u8*)(mw::iwnn::IWNN::NjexPredictionPseudoDic);
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_CONVERSION].base =
                                                   NJ_DEFAULT_FREQUENCY_DIC_LIMIT_PREDICTION_PRORAM_CONVERSION_BASE;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_CONVERSION].high = 
                                                   NJ_DEFAULT_FREQUENCY_DIC_LIMIT_PREDICTION_PRORAM_CONVERSION_HIGH;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_PREDICTION].base = 
                                                   NJ_DEFAULT_FREQUENCY_DIC_LIMIT_PREDICTION_PRORAM_PREDICTION_BASE;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_PREDICTION].high = 
                                                   NJ_DEFAULT_FREQUENCY_DIC_LIMIT_PREDICTION_PRORAM_PREDICTION_HIGH;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_MORPHOLIZE].base = 
                                                   NJ_DEFAULT_FREQUENCY_DIC_LIMIT_PREDICTION_PRORAM_MORPHOLIZE_BASE;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_MORPHOLIZE].high = 
                                                   NJ_DEFAULT_FREQUENCY_DIC_LIMIT_PREDICTION_PRORAM_MORPHOLIZE_HIGH;
    dicSet.dic[dicCnt].searchCache = NULL;
    dicSet.dic[dicCnt].limit = 0;
    dicCnt++;

    // Implementation of pseudo-dictionary sample 3 
    dicSet.dic[dicCnt].type = NJ_DIC_HANDLE_TYPE_PROGRAM;
    dicSet.dic[dicCnt].extensionArea = NULL;
    dicSet.dic[dicCnt].handle = (u8*)(NjexPseudoAsciiDic);
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_CONVERSION].base = NJ_DEFAULT_FREQUENCY_DIC_PRORAM_CONVERSION_BASE;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_CONVERSION].high = NJ_DEFAULT_FREQUENCY_DIC_PRORAM_CONVERSION_HIGH;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_PREDICTION].base = NJ_DEFAULT_FREQUENCY_DIC_PRORAM_PREDICTION_BASE;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_PREDICTION].high = NJ_DEFAULT_FREQUENCY_DIC_PRORAM_PREDICTION_HIGH;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_MORPHOLIZE].base = NJ_DEFAULT_FREQUENCY_DIC_PRORAM_MORPHOLIZE_BASE;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_MORPHOLIZE].high = NJ_DEFAULT_FREQUENCY_DIC_PRORAM_MORPHOLIZE_HIGH;
    dicSet.dic[dicCnt].searchCache = NULL;
    dicSet.dic[dicCnt].limit = 0;
    dicCnt++;

    //Rule dictionary (njcon.a)
    if( readDic( &dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION],"sdmc:/iWnn/dic/JA/njcon.a") <= 0 ){
        releaseDic();
        Fprintf(logFile, L"error:rule dic open\n");
        logFile->Finalize();
        nn::fs::Unmount("sdmc:");
        delete logFile;
        dicHeap.Finalize();
        return -1;
    }
    dicSet.ruleHandle[NJ_MODE_TYPE_PREDICTION] = dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION];
    dicSet.ruleHandle[NJ_MODE_TYPE_MORPHOLIZE] = dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION];
    dicSet.mode  =  NJ_CACHE_MODE_NONE;

    /********************************/
    /*2. Loading a reading string for predictive conversion*/
    /********************************/
    NN_LOG("  iWnn Demo: Loading reading string for prediction.\n");
    if( (loadReading( analyzeReading,"sdmc:/iWnn/anlz.txt",sizeof(wchar_t)*(NJ_MAX_LEN+NJ_TERM_SIZE))) <= 0 ){
        releaseDic();       
        Fprintf(logFile, L"error:analyze demo reading open\n");
        logFile->Finalize();
        nn::fs::Unmount("sdmc:");
        delete logFile;
        dicHeap.Finalize();
        return -1;
    }

    if( (loadReading( analyzePseudoReading,"sdmc:/iWnn/anlz_giji.txt",sizeof(wchar_t)*(NJ_MAX_LEN+NJ_TERM_SIZE))) <= 0 ){
        releaseDic();       
        Fprintf(logFile, L"error:analyze pseudo demo reading open\n");
        logFile->Finalize();
        nn::fs::Unmount("sdmc:");
        delete logFile;
        dicHeap.Finalize();
        return -1;
    }
    
    
    if( (loadReading( analyzePseudoReading2,"sdmc:/iWnn/anlz_giji2.txt",sizeof(wchar_t)*(NJ_MAX_LEN+NJ_TERM_SIZE))) <= 0 ){
        releaseDic();
        Fprintf(logFile, L"error:analyze pseudo2 demo reading open\n");  
        logFile->Finalize();
        nn::fs::Unmount("sdmc:");
        delete logFile;
        dicHeap.Finalize();
        return -1;
    }

    /***********************************/
    /*3. Loading a reading string for multiple phrase conversion */
    /***********************************/
    NN_LOG("  iWnn Demo: Loading reading string for conversion.\n");
    if( (loadReading( conversionReading,"sdmc:/iWnn/conv.txt",sizeof(wchar_t)*(NJ_MAX_LEN+NJ_TERM_SIZE))) <= 0 ){
        releaseDic();
        Fprintf(logFile, L"error:convertion1 demo reading open\n");
        logFile->Finalize();
        nn::fs::Unmount("sdmc:");
        delete logFile;
        dicHeap.Finalize();
        return -1;
    }
    
    if( (loadReading( conversionReading2,"sdmc:/iWnn/conv2.txt",sizeof(wchar_t)*(NJ_MAX_LEN+NJ_TERM_SIZE))) <= 0 ){
        releaseDic();
        Fprintf(logFile, L"error:conversion2 demo reading open\n");
        logFile->Finalize();
        nn::fs::Unmount("sdmc:");
        delete logFile;
        dicHeap.Finalize();
        return -1;
    }

    
    if( (loadReading( conversionReading3,"sdmc:/iWnn/conv3.txt",sizeof(wchar_t)*(NJ_MAX_LEN+NJ_TERM_SIZE))) <= 0 ){
        releaseDic();
        Fprintf(logFile, L"error:conversion3 demo reading open\n");
        logFile->Finalize();
        nn::fs::Unmount("sdmc:");
        delete logFile;
        dicHeap.Finalize();
        return -1;
    }

    /********************************/
    /* 4. iWnn engine initialization       */
    /********************************/
    NN_LOG("  iWnn Demo: Initializing iWnn engine.\n");
    funcOption.autoConversionCnt = NJ_OPTION_AUTO_CONVERSION_CNT;
    funcOption.phase2Filter  = (void *)MyCandidateLenFilter; 
    funcOption.phase1Filter  = (void *)MyCharTypeFilter;

    mw::iwnn::IWNN::NjxInit(&iwnnClass, &funcOption);   
    iwnnClass.dicSet = dicSet;
    

  
    /********************************/
    /* 5. Predictive conversion demo                */
    /********************************/
    NN_LOG("  iWnn Demo: Prediction demo1.\n");
    ret = demoAnalyze( &iwnnClass, analyzeReading, 3);
    if( ret < 0 ){
        releaseDic();
        Fprintf(logFile, L"error:analyze demo\n");
        logFile->Finalize();
        nn::fs::Unmount("sdmc:");
        delete logFile;
        dicHeap.Finalize();
        return -1;
    }

    NN_LOG("  iWnn Demo: Prediction demo2.\n");
    ret = demoAnalyze( &iwnnClass, analyzePseudoReading, 3);
    if( ret < 0 ){
        releaseDic();
        Fprintf(logFile, L"error:analyze pseudo demo\n");
        logFile->Finalize();
        nn::fs::Unmount("sdmc:");
        delete logFile;
        dicHeap.Finalize();
        return -1;
    }

    NN_LOG("  iWnn Demo: Prediction demo3.\n");
    ret = demoAnalyze( &iwnnClass, analyzePseudoReading2, 1);
    if( ret < 0 ){
        releaseDic();
        Fprintf(logFile, L"error:analyze pseudo2 demo\n");
        logFile->Finalize();
        nn::fs::Unmount("sdmc:");
        delete logFile;
        dicHeap.Finalize();
        return -1;
    }


    /********************************/
    /* 6. Multiple phrase conversion demo             */
    /********************************/
    NN_LOG("  iWnn Demo: Conversion demo1.\n");
    ret = demoConversion( &iwnnClass, conversionReading, 3);
    if( ret < 0 ){
        releaseDic();
        Fprintf(logFile, L"error:conversion1 demo\n");
        logFile->Finalize();
        nn::fs::Unmount("sdmc:");
        delete logFile;
        dicHeap.Finalize();
        return -1;
    }

    NN_LOG("  iWnn Demo: Conversion demo2.\n");
    ret = demoConversion( &iwnnClass, conversionReading2, 1);
    if( ret < 0 ){
        releaseDic();
        Fprintf(logFile, L"error:conversion2 demo\n");
        logFile->Finalize();
        nn::fs::Unmount("sdmc:");
        delete logFile;
        dicHeap.Finalize();
        return -1;
    }

    NN_LOG("  iWnn Demo: Conversion demo3.\n");
    ret = demoConversion( &iwnnClass, conversionReading3, 1);
    if( ret < 0 ){
        releaseDic();
        Fprintf(logFile, L"error:conversion3 demo\n");
        logFile->Finalize();
        nn::fs::Unmount("sdmc:");
        delete logFile;
        dicHeap.Finalize();
        return -1;
    }
    
    /********************************/
    /* 7. Dictionary unmount           */
    /********************************/
    NN_LOG("  iWnn Demo: Unmounting dictionaries.\n");
    releaseDic();


    Fprintf(logFile,L"demo end!\n");
    logFile->Finalize();    
    nn::fs::Unmount("sdmc:");
    delete logFile;
    dicHeap.Finalize();
    NN_LOG("  iWnn Demo: Demo end.\n");
    while(true){}


}
/*
 
               
               
               
               

 
 

 
 */
s32 readDic(u8 **dicData,char *dicName )
{         
    size_t  fileSize;

    nn::fs::FileInputStream inFile(dicName);
    fileSize = inFile.GetSize();
 
    if(fileSize <= 0 ){
        Fprintf(logFile,L" Invalid file size(%d)\n", fileSize);
        inFile.Finalize();
        return -1;
    }
    
    *dicData = (IWNN_DIC_HANDLE)dicHeap.Allocate(fileSize);
    if(*dicData == NULL ){
        Fprintf(logFile,L" Failed to allocation \n");
        inFile.Finalize();
        return -1;
    }

    fileSize = (s32)inFile.Read(*dicData, fileSize);    
    inFile.Finalize();
    return(fileSize);
}
/*
 
               
               
               

 
 
 

 
 */
s32 loadReading( wchar_t *ptrReading,char *fileName, s32 stringSize){

    // Check arguments
    if ( ptrReading == NULL ){
        Fprintf(logFile, L" parameter error(ptrReading = NULL)\n");
        return -1;
    }

    nn::fs::FileInputStream inFile(fileName);
    size_t fileSize = inFile.GetSize();
    if (fileSize >= stringSize) {
        Fprintf(logFile,L" Failed to read file status (%d)\n", fileSize);
        inFile.Finalize();
        return -1;
    }
        
    if (fileSize <= 0) {
        Fprintf(logFile, L"Failed to read file status (%d)\n", fileSize);
        inFile.Finalize();
        return -1;
    }

    fileSize = inFile.Read(ptrReading, fileSize);
    inFile.Finalize();
    return(fileSize);
}
/*
 
               
               
               

 

 
 */
void releaseDic(void){
    u16 i;
    if(ptrUbase1 != NULL){
        dicHeap.Free(ptrUbase1);        
        dicSet.dic[1].handle=NULL;
        ptrUbase1 = NULL;
    }
    if(ptrUbase2 != NULL){
        dicHeap.Free(ptrUbase2);        
        dicSet.dic[2].handle=NULL;
        ptrUbase2 = NULL;
    }

    for(i = 0; i < dicCnt;i++){
        if(dicSet.dic[i].handle != NULL &&
           dicSet.dic[i].type== NJ_DIC_HANDLE_TYPE_NRM){
            dicHeap.Free(dicSet.dic[i].handle);
        }
    }
    if( dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION] != NULL){
        dicHeap.Free(dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION]);
    }
    if(inFileUbase1!=NULL){        
        inFileUbase1->Finalize();
        delete inFileUbase1;
    }    

    if(inFileUbase2!=NULL){
        inFileUbase2->Finalize();
        delete inFileUbase2;
    }
}
/*
 
               
               
               
               
               

 
 
 

 
 */
s16 demoAnalyze( IWNN_CLASS *environment, wchar_t *reading, u16 select)
{
    IWNN_RESULT   result[MAX_DEMO_CANDIDATE];
    wchar_t     *ptrReading;
    wchar_t     readingUtf16[NJ_MAX_LEN+NJ_TERM_SIZE];
    wchar_t     readingUtf16_2[NJ_MAX_LEN+NJ_TERM_SIZE];
    wchar_t     buf[NJ_MAX_RESULT_LEN+NJ_TERM_SIZE];
    s16    ret;
    u16   i;
    IWNN_ANALYZE_OPTION    analyze_option;    


    // Check arguments
    if ( environment == NULL ){
        Fprintf(logFile,L"error:demoAnalyze\n");
        return -1;
    }
    
    if ( reading == NULL ){
        Fprintf(logFile,L"error:demoAnalyze\n");
        return -1;
    }
    
    if ( wcharStringLen(reading) > NJ_MAX_LEN ){
        Fprintf(logFile,L"error:demoAnalyze\n");
        return -1;
    }

    if ( select >= NJ_MAX_CANDIDATE || select >= MAX_DEMO_CANDIDATE){
        Fprintf(logFile,L"error:demoAnalyze\n");
        return -1;
    }

    //* Prediction candidate option settings *
    analyze_option.mode = NJ_NO_MULTI_CONVERSION|NJ_RELATION_ON|NJ_NO_READING_ON;
    analyze_option.forecastLearnLimit = 20;
    analyze_option.forecastLimit = 100;
    analyze_option.charMin = 0;
    analyze_option.charMax = 55;

    memset( readingUtf16, 0x00, sizeof(readingUtf16) );
    memset( readingUtf16_2, 0x00, sizeof(readingUtf16_2) );

    Fprintf(logFile,L"-------------------------------------------------------\n");
    Fprintf(logFile,L"Prediction Demo -Start-\n");

    Fprintf(logFile,L" Input Reading : ");
    FprintUTF16BE(logFile,reading);
    Fprintf(logFile,L"\n");
 
    // Get predictive conversion candidates 
    Fprintf(logFile,L" Prediction List\n");
    memcpy(&readingUtf16[0], reading, sizeof(readingUtf16));
    ptrReading = &readingUtf16[0];
    for (i = 0; i < MAX_DEMO_CANDIDATE; i++) {
        memset(&result[i], 0x00, sizeof(IWNN_RESULT));
        ret = mw::iwnn::IWNN::NjxAnalyze(environment, &result[i], NULL, ptrReading, &analyze_option);
        if (ret > 0) {
            ret = mw::iwnn::IWNN::NjxGetCandidate(buf, environment, &result[i], sizeof(buf));
            
            if (ret < 0) {
                Fprintf(logFile,L"error:[%02d]NjxGetCandidate-%04x-\n", i, ret);
                return -1;
            }
            Fprintf(logFile,L" [%2d]-",i);
            FprintUTF16BE(logFile,buf);
            Fprintf(logFile,L"\n");

        } else if(ret == 0) {
            break;    
        } else {
            Fprintf(logFile,L"error:[%02d]NjxAnalyze-%04x-\n", i, ret);
            return ret;
        }

        ptrReading = NULL;
    }
    Fprintf(logFile,L" Prediction List-End-\n\n");

    // Learn prediction candidate having the specified index
    if(i == 0){
          Fprintf(logFile,L"error:NjxAnalyze No candidate\n");
          return -1;
    }else if(select > (i-1)){
        select = i-1;
    }
    Fprintf(logFile,L" Select [%2d]\n",select);
    ret = mw::iwnn::IWNN::NjxSelect(environment,  &result[select], &result[select], 0);
    if (ret < 0) {
        Fprintf(logFile,L"error:NjxSelect-%04x-\n",  ret);
        return ret;
    }

    // Get derived prediction candidate 
    Fprintf(logFile,L" Next Word Prediction List\n");
    readingUtf16_2[0] = NJ_CHAR_NUL;
    ptrReading = &readingUtf16_2[0];
    for (i = 0; i < MAX_DEMO_CANDIDATE; i++) {
        ret = mw::iwnn::IWNN::NjxAnalyze(environment, &result[i], NULL, ptrReading, &analyze_option);
        if (ret > 0) {
            ret = mw::iwnn::IWNN::NjxGetCandidate(buf, environment, &result[i], sizeof(buf));            
            if (ret < 0) {
                Fprintf(logFile,L"error:[%02d]NjxGetCandidate-%04x-\n", i, ret);
                return -1;
            }
            Fprintf(logFile,L" [%2d]-",i);
            FprintUTF16BE(logFile,buf);
            Fprintf(logFile,L"\n");
        } else if(ret == 0) {
            break;
        } else {
            Fprintf(logFile,L"error:[%02d]NjxAnalyze-%04x-\n", i, ret);
            return ret;
        }
        ptrReading = NULL;
    }
     Fprintf(logFile,L" Prediction List-End-\n\n");

    // Perform predictive conversion again to check learning 
    Fprintf(logFile,L" Prediction List\n");
    ptrReading = &readingUtf16[0];
    for (i = 0; i < MAX_DEMO_CANDIDATE; i++) {
        ret = mw::iwnn::IWNN::NjxAnalyze(environment, &result[i], NULL, ptrReading, &analyze_option);
        if (ret > 0) {
            ret = mw::iwnn::IWNN::NjxGetCandidate(buf, environment, &result[i], sizeof(buf));
            if (ret < 0) {
                Fprintf(logFile,L"error:[%02d]NjxGetCandidate-%04x-\n", i, ret);
                return -1;
            }
            Fprintf(logFile,L" [%2d]-",i);
            FprintUTF16BE(logFile,buf);
            Fprintf(logFile,L"\n");
        } else if(ret == 0) {
            break;
        } else {
            Fprintf(logFile,L"error:[%02d]NjxAnalyze-%04x-\n", i, ret);
            return ret;
        }
        ptrReading = NULL;
    }
    Fprintf(logFile,L" Prediction List-End-\n");
    Fprintf(logFile,L"Prediction Demo -End-\n\n");
    Fprintf(logFile,L"-------------------------------------------------------\n");
    
    return i;
}

/*
 
               
               
               
               

 
 
 

 
 */
s16 demoConversion( IWNN_CLASS *environment, wchar_t *reading, u16 select)
{
    IWNN_RESULT   phraseResult[NJ_MAX_PHRASE];
    IWNN_RESULT   candidateResult[MAX_DEMO_CANDIDATE];
    IWNN_RESULT   *ptrResult;
    wchar_t     readingUtf16[NJ_MAX_LEN+NJ_TERM_SIZE];
    wchar_t     readingUtf16_2[NJ_MAX_LEN+NJ_TERM_SIZE];
    wchar_t     buf[NJ_MAX_RESULT_LEN+NJ_TERM_SIZE];
    s16    ret;
    s16    cnt;
    u16   i, j;
    u8    connect = 0;
    u16 select_pos;

    // Check arguments
    if ( environment == NULL ){
        Fprintf(logFile,L"error:demoAnalyze\n");
        return -1;
    }

    if ( reading == NULL ){
        Fprintf(logFile,L"error:demoAnalyze\n");
        return -1;
    }

    if ( wcharStringLen(reading) > NJ_MAX_LEN ){
        Fprintf(logFile,L"error:demoAnalyze\n");
        return -1;
    }

    if ( select >= NJ_MAX_CANDIDATE || select >= MAX_DEMO_CANDIDATE){
        Fprintf(logFile,L"error:demoAnalyze\n");
        return -1;
    }


    memset( readingUtf16, 0x00, sizeof(readingUtf16) );
    memset( readingUtf16_2, 0x00, sizeof(readingUtf16_2) );

    Fprintf(logFile,L"-------------------------------------------------------\n");    
    Fprintf(logFile,L"Kana-Kanji Conversion Demo -Start-\n");

    Fprintf(logFile,L" Input Reading : ");
    FprintUTF16BE(logFile,reading);
    Fprintf(logFile,L"\n");

    // Convert with automatic clause-parsing
    Fprintf(logFile,L" Conversion\n");
    
    memcpy(&readingUtf16[0], reading, sizeof(readingUtf16));

    cnt = mw::iwnn::IWNN::NjxConversion(environment, phraseResult, readingUtf16, NJ_MAX_PHRASE, 0);
        
    if (cnt <= 0){
        Fprintf(logFile,L"error:NjxConversion-%04x-\n", cnt );
        return -1;
    }
    
    Fprintf(logFile,L" Conversion List\n");
    for (i = 0; i < cnt; i++) {
        ret = mw::iwnn::IWNN::NjxGetCandidate(buf, environment, &phraseResult[i], sizeof(buf));       
        if (ret < 0) {
            Fprintf(logFile,L"error:[%02d]NjxGetCandidate-%04x-\n", i, ret);
            return -1;
        }
        Fprintf(logFile,L" [%2d]-",i);
        FprintUTF16BE(logFile,buf);
        Fprintf(logFile,L"\n");
    }
    Fprintf(logFile,L" Conversion List-End-\n\n");

    // Get multiple phrase candidates
    for (i = 0; i < cnt; i++) {
        ptrResult = &phraseResult[i];
        Fprintf(logFile,L" Candidate List [%02d]\n", i);
        for (j = 0; j < MAX_DEMO_CANDIDATE; j++){
            ret = mw::iwnn::IWNN::NjxAllCandidates(environment, &candidateResult[j], ptrResult, j); 
            if (ret > 0) {            

                ret = mw::iwnn::IWNN::NjxGetCandidate(buf, environment, &candidateResult[j], sizeof(buf));
                if (ret < 0) {
                    Fprintf(logFile,L"error:[%02d]NjxGetCandidate-%04x-\n", j, ret);
                    return -1;
                }
                Fprintf(logFile,L" [%2d]-",j);
                FprintUTF16BE(logFile,buf);
                Fprintf(logFile,L"\n");
            } else if(ret == 0) {
                break;
            } else {
                Fprintf(logFile,L"error:[%02d]NjxAllCandidates-%04x-\n", j, ret);
                return ret;
            }
            ptrResult = NULL;
        }
        Fprintf(logFile,L" Candidate List-End-\n\n");

        select_pos = select;
        if(j == 0){
            Fprintf(logFile,L"error:NjxGetCandidate No candidate\n");
            return -1;
        }else if(select > (j-1)){
            select_pos = j-1;
        }
        // Learn candidate having the specified index     
        Fprintf(logFile,L" Select [%2d]\n\n",select_pos);
        ret = mw::iwnn::IWNN::NjxSelect(environment,  &candidateResult[select_pos], &candidateResult[select_pos], connect);
        if (ret < 0) {
            Fprintf(logFile,L"error:NjxSelect-%04x-\n",  ret);
            return ret;
        }
        connect = 1;
    }


    // Perform multiple phrase conversion again to check learning 
    Fprintf(logFile,L" Re-Conversion\n");
    cnt = mw::iwnn::IWNN::NjxConversion(environment, phraseResult, readingUtf16, NJ_MAX_PHRASE, 0);
    if (cnt <= 0){
        Fprintf(logFile,L"error:NjxConversion-%04x-\n", cnt );
        return -1;
    }
   
    Fprintf(logFile,L" Conversion List\n");
    for (i = 0; i < cnt; i++) {
        ret = mw::iwnn::IWNN::NjxGetCandidate(buf, environment, &phraseResult[i], sizeof(buf));
        if (ret < 0) {
            Fprintf(logFile,L"error:[%02d]NjxGetCandidate-%04x-\n", i, ret);
            return -1;
        }
        Fprintf(logFile,L" [%2d]-",i);
        FprintUTF16BE(logFile,buf);
        Fprintf(logFile,L"\n");
    }
    Fprintf(logFile,L" Conversion List-End-\n");
    Fprintf(logFile,L"Kana-Kanji Conversion Demo -End-\n");
    Fprintf(logFile,L"-------------------------------------------------------\n");

    return i;
}
/*
 

 
 

 
 

 */
s16 MyCharTypeFilter(IWNN_CLASS * iwnn, IWNN_PHASE1_FILTER_MESSAGE *message)
{
    s16 ret;
    u8 stemType;
    u8 ancillaryType;


    ret = mw::iwnn::IWNN::NjxGetCharType(&stemType,&ancillaryType,iwnn,message->result);
    if(ret < 0){
        return 0;
    }

    if( stemType == NJ_TYPE_HALF_KATAKANA || ancillaryType == NJ_TYPE_HALF_KATAKANA){
        return 0;
    }
        
    return 1;
}

/*
 

 
 

 
 

 */
s16 MyCandidateLenFilter(IWNN_CLASS * iwnn, IWNN_PHASE2_FILTER_MESSAGE *message)
{
    s16 ret;
    wchar_t  candidate[NJ_MAX_RESULT_LEN+NJ_TERM_SIZE];


    ret = mw::iwnn::IWNN::NjxGetCandidate(candidate,iwnn,message->result,sizeof(candidate));
    if(ret < 0){
        return 0;
    }

    if(ret < MIN_DEMO_CANDIDATE_LEN){
        return 0;
    }

    return 1;
}


/*
 

 

 
 
 

 
 */
u16 wcharStringLen(wchar_t *c) {

    u16 count = 0;

    while (*c++ != NULL) {
        count++;
    }
    return count;
}
