﻿/*-----------------------------------------------------------------------*
 Project: iWnn API Morpholize Demo: Main Function
 File:    main.cpp

 Copyright (C)2009-2012 Nintendo Co., Ltd.  All rights reserved.

 These coded instructions, statements, and computer programs contain
 proprietary information of Nintendo of America Inc. and/or Nintendo
 Company Ltd., and are protected by Federal copyright law.  They may
 not be disclosed to third parties or copied or duplicated in any form,
 in whole or in part, without the prior written consent of Nintendo.

 $Revision: $
 *-----------------------------------------------------------------------*/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <nn/os.h>
#include <nn/dbg.h>
#include <nn/fs.h>
#include <nn/fnd.h>
#include <nn/applet.h>
#include <mw/iwnn/iwnnCTR.h>
#include "fprintf.h"


#define DEMO_MAX_STRING_LEN 512  //Maximum input string length

#define DEMO_PARAM_ERR -1    //Argument error
#define DEMO_SIZE_OVER -2    //Size error

#define MAX_LEARN_COUNT 1000 //Learning dictionary registration count
#define LEARN_DIC_SIZE (80 + 42*MAX_LEARN_COUNT) //Learning dictionary size

s32 readDic(u8 **dicData,char *dicName);
s32 loadString(wchar_t *ptrString,char *fileName, s32 stringSize);
s16 demoMorpho( IWNN_CLASS *environment, wchar_t *reading);
void releaseDic(void);

IWNN_CLASS      iwnnClass;    // iWnn class structure definition
IWNN_DIC_SET  dicSet;                                   //Dictionary set structure
s16    dicCnt;                                   //Mounted dictionary count
IWNN_FLASH_DIC_INFO    flashDicInfo[NJ_MAX_DIC];       //FLASH type dictionary information storage region

nn::fnd::ExpHeap            dicHeap;       //Instance for dynamic memory allocation
nn::fs::FileOutputStream*   logFile=NULL;  //Log file output instance
nn::fs::FileStream*    inFileUbase1=NULL;  //Instance for working with Integrated Dictionary File 1.
nn::fs::FileStream*    inFileUbase2=NULL;  //Instance for working with Integrated Dictionary File 2.
u8* ptrUbase1=NULL;                        //Integrated Dictionary 1 handle pointer  
u8* ptrUbase2=NULL;                        //Integrated Dictionary 2 handle pointer

wchar_t        inputString[DEMO_MAX_STRING_LEN+NJ_TERM_SIZE];                 //Storage region for source string being parsed

u16 wcharStringLen(wchar_t *c);
wchar_t *wcharStrncpy(wchar_t *dst, wchar_t *src, u16 n);





/*
 
               
               
               
               
               

 

 
 */
extern "C" int nnMain()
{
    s16    ret = 0;
    s32 fileSize;
    u8     bom[2]={0xff,0xfe};

    //Enabling the applet library
    nn::applet::Enable();

    //fs initialization
    nn::fs::Initialize();

    //SD Card initialization
    nn::Result result = nn::fs::MountSdmc();
    if( result.IsFailure() ){
        return -1;
    }    

    //Result output file (Result3.txt) initialization
    logFile = new (nn::fs::FileOutputStream);
    if(!logFile){
        nn::fs::Unmount("sdmc:");
        return -1;
    }

    logFile->Initialize(FPRINTF_FILE_PATH,true);
    //Write BOM symbol
    logFile->Write(bom,sizeof(bom),true);

    Fprintf(logFile,L"demo start!\n");

    //Initialize the heap region
    dicHeap.Initialize(nn::os::GetDeviceMemoryAddress(), nn::os::GetDeviceMemorySize(), nn::os::ALLOCATE_OPTION_LINEAR);
    
    dicCnt = 0;
    memset(&iwnnClass, 0x00, sizeof(IWNN_CLASS));
    memset(&dicSet, 0x00, sizeof(IWNN_DIC_SET));
    memset(&flashDicInfo, 0x00, (NJ_MAX_DIC) * sizeof(IWNN_FLASH_DIC_INFO));        


    /********************************/
    /* 1. Dictionary set structure settings       */
    /********************************/
    NN_LOG("  iWnn Demo: Mounting dictionaries.\n");


    // Learning dictionary        
    dicSet.dic[dicCnt].handle = (IWNN_DIC_HANDLE)dicHeap.Allocate(LEARN_DIC_SIZE);
    if (dicSet.dic[dicCnt].handle == NULL) {
        Fprintf(logFile, L"error:learn-dic malloc\n");
        logFile->Finalize();
        nn::fs::Unmount("sdmc:");
        delete logFile;
        dicHeap.Finalize();
        return -1;
    }
    
    ret = mw::iwnn::IWNN::NjxCreateDic(dicSet.dic[dicCnt].handle, &iwnnClass, NJ_CREATE_DIC_TYPE_LEARN, LEARN_DIC_SIZE);
    
    if (ret < 0) {
        releaseDic();
        Fprintf(logFile, L"error:learn-dic create\n");
        logFile->Finalize();
        nn::fs::Unmount("sdmc:");
        delete logFile;
        dicHeap.Finalize();
        return -1;
    }
    
    dicSet.dic[dicCnt].type            = NJ_DIC_HANDLE_TYPE_NRM;
    dicSet.dic[dicCnt].limit        = 0;
    dicSet.dic[dicCnt].extensionArea        = NULL;
    dicSet.dic[dicCnt].searchCache     = NULL;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_MORPHOLIZE].base = 10;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_MORPHOLIZE].high = 0;
 
    dicCnt++;

    //Integrated dictionary 1 (njubase1.a)
    inFileUbase1 = new(nn::fs::FileStream);
    inFileUbase1->Initialize(L"sdmc:/iWnn/dic/JA/32/njubase1.a",nn::fs::OPEN_MODE_READ);
    
    fileSize = mw::iwnn::IWNN::NjxGetFlashDicCacheSize(&iwnnClass, (IWNN_FILE*)inFileUbase1);    
    
    if(fileSize <= 0){   
        releaseDic();
        Fprintf(logFile, L"error:[%d]dic open\n", dicCnt);
        logFile->Finalize();
        nn::fs::Unmount("sdmc:");
        delete logFile;
        dicHeap.Finalize();
        return -1;
    }    
    
    ptrUbase1 = NULL;
    ptrUbase1  = (u8*)(dicHeap.Allocate(fileSize));
    if (mw::iwnn::IWNN::NjxSetFlashDicInfo(&(flashDicInfo[dicCnt]), &iwnnClass, (IWNN_FILE*)inFileUbase1, ptrUbase1, fileSize) < 0) {       
        releaseDic();
        Fprintf(logFile, L"error:[%d]dic open\n", dicCnt);
        logFile->Finalize();
        nn::fs::Unmount("sdmc:");
        delete logFile;
        dicHeap.Finalize();
        return -1;
        
    } else {
        dicSet.dic[dicCnt].handle = (IWNN_DIC_HANDLE)&(flashDicInfo[dicCnt]);
        dicSet.dic[dicCnt].addInfo[0] = NULL;
        dicSet.dic[dicCnt].addInfo[1] = NULL;
    }

    /* In this iWnn library, all of the dictionary data is expanded to memory.
       The iWnn functions that issue file access are:
        - NjxGetFlashDicCacheSize
        - NjxSetFlashDicInfo
        - NjxCheckDic
       and those functions only.
       With this program, inFileUbase1 after this will become unnecessary so, the file is closed here
    */
    inFileUbase1->Finalize();
    delete inFileUbase1;
    inFileUbase1=NULL;

    dicSet.dic[dicCnt].type                               = NJ_DIC_HANDLE_TYPE_ON_FLASH;
    dicSet.dic[dicCnt].limit                              = 0;
    dicSet.dic[dicCnt].extensionArea                           = NULL;
    dicSet.dic[dicCnt].searchCache                           = NULL;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_MORPHOLIZE].base = NJ_DEFAULT_FREQUENCY_DIC_UNION_MORPHOLIZE_BASE;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_MORPHOLIZE].high = NJ_DEFAULT_FREQUENCY_DIC_UNION_MORPHOLIZE_HIGH;

    dicCnt++;

    //Integrated dictionary 2 (njubase2.a)
    inFileUbase2 = new(nn::fs::FileStream);
    inFileUbase2->Initialize(L"sdmc:/iWnn/dic/JA/32/njubase2.a",nn::fs::OPEN_MODE_READ);
        
    fileSize = mw::iwnn::IWNN::NjxGetFlashDicCacheSize(&iwnnClass, (IWNN_FILE*)inFileUbase2);     
   
    if(fileSize <= 0){ 
        releaseDic();
        Fprintf(logFile, L"error:[%d]dic open\n", dicCnt);
        logFile->Finalize();
        nn::fs::Unmount("sdmc:");
        delete logFile;
        dicHeap.Finalize();
        return -1;
    }
    
    ptrUbase2 = NULL;
    ptrUbase2 = (u8 *)dicHeap.Allocate(fileSize);
    if (mw::iwnn::IWNN::NjxSetFlashDicInfo(&(flashDicInfo[dicCnt]), &iwnnClass, (IWNN_FILE*)inFileUbase2, ptrUbase2, fileSize) < 0) {  
        releaseDic();
        Fprintf(logFile, L"error:[%d]dic open\n", dicCnt);
        logFile->Finalize();
        nn::fs::Unmount("sdmc:");
        delete logFile;
        dicHeap.Finalize();
        return -1;        
    } else {
        dicSet.dic[dicCnt].handle = (IWNN_DIC_HANDLE)&(flashDicInfo[dicCnt]);
        dicSet.dic[dicCnt].addInfo[0] = NULL;
        dicSet.dic[dicCnt].addInfo[1] = NULL;
    }

    /* In this iWnn library, all of the dictionary data is expanded to memory.
       The iWnn functions that issue file access are:
        - NjxGetFlashDicCacheSize
        - NjxSetFlashDicInfo
        - NjxCheckDic
       and those functions only.
       With this program, inFileUbase2 after this will become unnecessary so, the file is closed here
    */
    inFileUbase2->Finalize();
    delete inFileUbase2;
    inFileUbase2=NULL;

    dicSet.dic[dicCnt].type                               = NJ_DIC_HANDLE_TYPE_ON_FLASH;
    dicSet.dic[dicCnt].limit                              = 2;
    dicSet.dic[dicCnt].extensionArea                           = NULL;
    dicSet.dic[dicCnt].searchCache                           = NULL;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_MORPHOLIZE].base = NJ_DEFAULT_FREQUENCY_DIC_UNION_MORPHOLIZE_BASE;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_MORPHOLIZE].high = NJ_DEFAULT_FREQUENCY_DIC_UNION_MORPHOLIZE_HIGH;

    dicCnt++;

    // Auxiliary dictionary (njfzk.a)
    if( readDic(  &dicSet.dic[dicCnt].handle,"sdmc:/iWnn/dic/JA/32/njfzk.a") <= 0 ){
        releaseDic();
        Fprintf(logFile, L"error:[%d]dic open\n", dicCnt);
        logFile->Finalize();
        nn::fs::Unmount("sdmc:");
        delete logFile;
        dicHeap.Finalize();
        return -1;
    }

    dicSet.dic[dicCnt].type            = NJ_DIC_HANDLE_TYPE_NRM;
    dicSet.dic[dicCnt].limit        = 0;
    dicSet.dic[dicCnt].extensionArea        = NULL;
    dicSet.dic[dicCnt].searchCache     = NULL;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_MORPHOLIZE].base = NJ_DEFAULT_FREQUENCY_DIC_UNION_MORPHOLIZE_BASE;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_MORPHOLIZE].high = NJ_DEFAULT_FREQUENCY_DIC_UNION_MORPHOLIZE_HIGH;
    dicCnt++;    

    // Single kanji dictionary (njtan.a)
    if( readDic( &dicSet.dic[dicCnt].handle,"sdmc:/iWnn/dic/JA/32/njtan.a") <= 0 ){
        releaseDic();
        Fprintf(logFile, L"error:[%d]dic open\n", dicCnt);
        logFile->Finalize();
        nn::fs::Unmount("sdmc:");
        delete logFile;
        dicHeap.Finalize();
        return -1;
    }

    dicSet.dic[dicCnt].type            = NJ_DIC_HANDLE_TYPE_NRM;
    dicSet.dic[dicCnt].limit        = 0;
    dicSet.dic[dicCnt].extensionArea        = NULL;
    dicSet.dic[dicCnt].searchCache     = NULL;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_MORPHOLIZE].base = 0;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_MORPHOLIZE].high = 10;
    dicCnt++;



    // Rule dictionary (njcon.a)   
    if( readDic( &dicSet.ruleHandle[NJ_MODE_TYPE_MORPHOLIZE],"sdmc:/iWnn/dic/JA/njcon.a") <= 0 ){
        releaseDic();
        Fprintf(logFile, L"error:rule dic open\n");
        logFile->Finalize();
        nn::fs::Unmount("sdmc:");
        delete logFile;
        dicHeap.Finalize();
        return -1;
    }
    dicSet.mode  =  NJ_CACHE_MODE_NONE;

    /**************************/
    /* 2. Parsing string load     */
    /**************************/
    memset(inputString,0x00,sizeof(u16)*(DEMO_MAX_STRING_LEN+NJ_TERM_SIZE));
    NN_LOG("  iWnn Demo: Loading reading string for morpholize.\n");
    if( loadString(  inputString,"sdmc:/iWnn/input_str.txt",sizeof(u16)*(DEMO_MAX_STRING_LEN+NJ_TERM_SIZE)) <= 0 ){
        releaseDic();
        Fprintf(logFile, L"error:demo input string open\n");
        logFile->Finalize();
        nn::fs::Unmount("sdmc:");
        delete logFile;
        dicHeap.Finalize();
        return -1;
    }
   Fprintf(logFile, L"*****************************input string*************************\n");
    FprintUTF16BE(logFile,inputString);
    Fprintf(logFile,L"\n");
    Fprintf(logFile, L"******************************************************************\n");
    Fprintf(logFile, L"Candidate\tReading\tPartsOfSpeech\tLearningStem\n");

    /********************************/
    /* 3. iWnn engine initialization       */
    /********************************/
    NN_LOG("  iWnn Demo: Initializing iWnn engine.\n");
    mw::iwnn::IWNN::NjxInit(&iwnnClass, NULL);   
    iwnnClass.dicSet = dicSet;

    
    /********************************/
    /* 4. Morphological Analyisis demo             */
    /********************************/    
    NN_LOG("  iWnn Demo: Morpholize demo.\n");
    ret = demoMorpho( &iwnnClass, inputString);
    if( ret < 0 ){
        releaseDic();
        if(ret == DEMO_PARAM_ERR){
            Fprintf(logFile, L"error:morpho demo (param error)\n");
        }else if(ret == DEMO_SIZE_OVER){
            Fprintf(logFile, L"error:morpho demo (size error)\n");
        }else{
            Fprintf(logFile, L"error:morpho demo (error code=0x%04x)\n", NJ_GET_ERR_CODE(ret));
        }
        
        logFile->Finalize();
        nn::fs::Unmount("sdmc:");
        delete logFile;
        dicHeap.Finalize();
        return -1;
    }

    
    /********************************/
    /* 5. Dictionary unmount           */
    /********************************/
    NN_LOG("  iWnn Demo: Unmounting dictionaries.\n");
    releaseDic();


    Fprintf(logFile, L"******************************************************************\n");
    Fprintf(logFile,L"demo end!\n");
    logFile->Finalize();    
    nn::fs::Unmount("sdmc:");
    delete logFile;    
    dicHeap.Finalize();
    NN_LOG("  iWnn Demo: Demo end.\n");
    while(true){}
}


/*
 
               
               
               
               

 
 

 
 */
s32 readDic(u8 **dicData,char *dicName )
{         
    size_t  fileSize;

    nn::fs::FileInputStream inFile(dicName);
    fileSize = inFile.GetSize();
 
    if(fileSize <= 0 ){
        inFile.Finalize();
        Fprintf(logFile,L" Invalid file size(%d)\n", fileSize);
        return -1;
    }
    
    *dicData = (IWNN_DIC_HANDLE)dicHeap.Allocate(fileSize);
    if(*dicData == NULL ){
        inFile.Finalize();
        Fprintf(logFile,L" Failed to allocation \n");
        return -1;
    }

    fileSize = (s32)inFile.Read(*dicData, fileSize);    
    inFile.Finalize();
    return(fileSize);
}

/*
 
               
               
               

 
 
 

 
 */
s32 loadString(wchar_t *ptrString,char *fileName, s32 stringSize){
 // Check arguments
    if ( ptrString == NULL ){
        Fprintf(logFile,L" parameter error (ptrString = NULL)\n");
        return -1;
    }
    nn::fs::FileInputStream inFile(fileName);
    size_t fileSize = inFile.GetSize();
    if (fileSize <= 0) {
        Fprintf(logFile, L"error:loadReading\n");
        inFile.Finalize();
        return -1;
    }
    if (fileSize >= stringSize) {
        Fprintf(logFile, L" Failed to read file status (%d)\n", fileSize);
        inFile.Finalize();
        return -1;
    }

    fileSize = inFile.Read(ptrString, stringSize);
    inFile.Finalize();
    return(fileSize);
}

/*
 
               
               
               

 

 
 */
void releaseDic(void){
    u16 i;
    if(ptrUbase1 != NULL){
        dicHeap.Free(ptrUbase1);        
        dicSet.dic[1].handle=NULL;
        ptrUbase1 = NULL;
    }
    if(ptrUbase2 != NULL){
        dicHeap.Free(ptrUbase2);        
        dicSet.dic[2].handle=NULL;
        ptrUbase2 = NULL;
    }

    for(i = 0; i < dicCnt;i++){
        if(dicSet.dic[i].handle != NULL &&
           dicSet.dic[i].type== NJ_DIC_HANDLE_TYPE_NRM){
            dicHeap.Free(dicSet.dic[i].handle);
        }
    }
    if( dicSet.ruleHandle[NJ_MODE_TYPE_MORPHOLIZE] != NULL){
        dicHeap.Free(dicSet.ruleHandle[NJ_MODE_TYPE_MORPHOLIZE]);
    }
    if(inFileUbase1!=NULL){        
        inFileUbase1->Finalize();
        delete inFileUbase1;
    }    

    if(inFileUbase2!=NULL){
        inFileUbase2->Finalize();
        delete inFileUbase2;
    }
}

/*
 
               
               
               

 
 

 
 */


IWNN_RESULT results[MM_MAX_MORPHOLIZE_LEN]; //Process result structure
wchar_t candidate[NJ_MAX_CANDIDATE + NJ_TERM_SIZE]; //Candidate string storage region
wchar_t reading[NJ_MAX_LEN+NJ_TERM_SIZE]; //Reading string storage region
IWNN_WORD_INFO wordInfo; //Word information

s16 demoMorpho( IWNN_CLASS *environment, wchar_t *inputString)
{
    s32 i;
    u8 processLen;
    s16 ret, partsOfSpeech;
    s16 stringLen;
    wchar_t* partsOfSpeechName=NULL;
    
    u8 stemLen;
    s16 cnt=0;
    s16 phraseNum=0;
    wchar_t* ptrString=NULL;
    u16 doneLen;
    u16 candidateLen=0;
    u16 readingLen=0;
    // Check arguments 
    if ((environment == NULL)||(inputString == NULL)) {
        return DEMO_PARAM_ERR;
    }
    stringLen = wcharStringLen(inputString);
    if(stringLen > DEMO_MAX_STRING_LEN){
        return DEMO_SIZE_OVER;
    }

    ptrString = inputString;
    doneLen=0;
    while(doneLen < stringLen){
        // Morphological analysis 
        processLen = 0;
        memset(results,0,sizeof(results));
        ret = mw::iwnn::IWNN::MmxSplitWord(environment, results,&processLen,ptrString);
        if (ret < 0) {
            return ret;
        }
        cnt = ret;        
        for (i = 0;  i < cnt; i++) {
            //Get part of speech

            partsOfSpeech = mw::iwnn::IWNN::MmxGetPartsOfSpeech(environment, &results[i]);
            switch (partsOfSpeech) {
                // Noun type 
                case MM_PARTS_OF_SPEECH_GROUP_NOUN :
                    partsOfSpeechName = L"NOUN";
                    break;
                // Pseudo-candidate 
                case MM_PARTS_OF_SPEECH_GROUP_PSEUDO :
                    partsOfSpeechName = L"PSEUDO";
                    break;
                // From other to not extracted 
                case MM_PARTS_OF_SPEECH_GROUP_OTHER:
                default:
                    partsOfSpeechName = L"OTHER";
                    break;
            }
            memset(candidate,0,sizeof(candidate));
            ret = mw::iwnn::IWNN::NjxGetCandidate(candidate,environment,&results[i],sizeof(candidate));
            if( ret < 0 ){
                return ret;
            }

            stemLen = 0;
            memset(reading,0,sizeof(reading));
            ret =  mw::iwnn::IWNN::MmxGetReading(environment,reading,&stemLen,&results[i],sizeof(reading));
            if( ret < 0 ){
                return ret;
            }

            //Register independent words in process result in the learning dictionary
            candidateLen =wcharStringLen(candidate);
            readingLen = wcharStringLen(reading);

            if( (candidateLen > 0 && candidateLen < NJ_MAX_RESULT_LEN) && 
                (readingLen > 0 && readingLen < NJ_MAX_LEN) && 
                stemLen > 0 ){
                ret = mw::iwnn::IWNN::NjxGetWordInfo(&wordInfo,environment,&results[i]);
                if(ret < 0 ){
                    return ret;
                }
                ret = mw::iwnn::IWNN::MmxSelect(environment,&results[i],reading,1,0);
                if( ret < 0 ){
                    return ret;
                }
            }
            FprintUTF16BE(logFile, candidate);
            Fprintf(logFile,L"\t");
            FprintUTF16BE(logFile, reading);
            Fprintf(logFile,L"\t");
            FprintUTF16LE(logFile,partsOfSpeechName);
            Fprintf(logFile,L"\t");
            memset(candidate,0,sizeof(candidate));
            wcharStrncpy(candidate,wordInfo.candidate,wordInfo.stem.candidateLen);
            FprintUTF16BE(logFile, candidate);
            Fprintf(logFile,L"(");
            memset(reading,0,sizeof(reading));
            wcharStrncpy(reading,wordInfo.reading,wordInfo.stem.readingLen);
            FprintUTF16BE(logFile, reading);
            Fprintf(logFile,L")\n");
        }
        ptrString += processLen;
        doneLen+=processLen;    
        phraseNum+=cnt;
    }
    return phraseNum;

}

/*
 

 

 
 
 

 
 */
u16 wcharStringLen(wchar_t *c) {

    u16 count = 0;

    while (*c++ != NULL) {
        count++;
    }
    return count;
}
/*
 
 
 
 
  
 
 
 
 
 
 */
wchar_t *wcharStrncpy(wchar_t *dst, wchar_t *src, u16 n) {

    wchar_t *d = dst;


    while (n != 0) {
        if (*src == NULL) {
            while (n != 0) {
                *d++ = NULL;
                n--;
            }
            break;
        } else {
            *d++ = *src++;
        }
        n--;
    }
    return dst;
}
