/*-----------------------------------------------------------------------*
 Project: iWnn API Multi Language Demo: Multi Language Framework Common Definition
 File:    multidic_com.h

 Copyright (C)2009-2012 Nintendo Co., Ltd.  All rights reserved.

 These coded instructions, statements, and computer programs contain
 proprietary information of Nintendo of America Inc. and/or Nintendo
 Company Ltd., and are protected by Federal copyright law. They may
 not be disclosed to third parties or copied or duplicated in any form,
 in whole or in part, without the prior written consent of Nintendo.

 $Revision: $
 *-----------------------------------------------------------------------*/

#ifndef _MULTI_COM_H_
#define _MULTI_COM_H_

#include <mw/iwnn/iwnnCTR.h>
#include "multidic_sys.h"


#define MULTI_LANG_TYPE_USE_MAX       1       // Maximum number of language to use simultaneously
#define MULTI_LANG_TYPE_SAME_MAX       3       // Maximum language TYPE count when using the same dictionary

#define MULTI_LEARN_MAX_CNT   1000     // Learning dictionary registration count
#define MULTI_LEARN_DIC_SIZE (80 + 42*MULTI_LEARN_MAX_CNT) //Learning dictionary size

// iWnn engine option settings 
#define MULTI_OPTION_MULTI_CONVERSION     10     // Multiple conversion auto start candidate count 

// Dictionary file-related definitions 
#define MULTI_DIC_DIR         "sdmc:/iWnn/dic"    // Dictionary folder       
#define MULTI_LANG_DIR_JA     "JA"     // Japanese             
#define MULTI_LANG_DIR_EN     "EN"     // English               
#define MULTI_LANG_DIR_EN_US  "US"     // English (US)           
#define MULTI_LANG_DIR_EN_GB  "GB"     // English (Great Britain)           
#define MULTI_LANG_DIR_ZH     "ZH"     // Chinese             
#define MULTI_LANG_DIR_CN     "CN"     // Chinese (simplified)     
#define MULTI_LANG_DIR_TW     "TW"     // Chinese (traditional)     
#define MULTI_LANG_DIR_KO     "KO"     // Korean             
#define MULTI_LANG_DIR_FR     "FR"     // French         
#define MULTI_LANG_DIR_DE     "DE"     // German           
#define MULTI_LANG_DIR_ES     "ES"     // Spanish         
#define MULTI_LANG_DIR_PT     "PT"     // Portuguese       
#define MULTI_LANG_DIR_PT_PT  "PT"     // Portuguese (Portugal)   
#define MULTI_LANG_DIR_PT_BR  "BR"     // Portuguese (Brazil)   
#define MULTI_LANG_DIR_IT     "IT"     // Italian         
#define MULTI_LANG_DIR_RU     "RU"     // Russian           
#define MULTI_LANG_DIR_NL     "NL"     // Dutch
#define MULTI_LANG_DIR_HK     "HK"     // Chinese (Cantonese) //2011/12/05 Support for the addition of the Cantonese (Cangjie) Dictionary to the package.
#define MULTI_LANG_DIR_CA     "CA"     // Quebecois           

#define MULTI_REWRITE_DIR     "sdmc:/iWnn/rewrite"  // Re-writable dictionary 
#define MULTI_LARGE_DIR       "large"    // large type      
#define MULTI_REGULAR_DIR     "regular"  // regular type    
#define MULTI_16_DIR          "16"       // 16Byte type     
#define MULTI_32_DIR          "32"       // 32Byte type     

#define MULTI_GB18030_DIR     "GB18030"  // Simplified kanji standard

// Input method definition 
#define MULTI_INTYPE_PINYIN   "pinyin"   // Pinyin         
#define MULTI_INTYPE_BOPOMOFO "bopomofo" // Bopomofo
//2011/10/11 Commented out the Cantonese dictionary because it was deleted from the package.
//#define MULTI_INTYPE_JYUTPING "jyutping" // Cantonese romanization scheme            
#define MULTI_INTYPE_CANGJIE  "cangjie"  // 2011/12/05 Support for additions of the Cantonese (Cangjie) Dictionary to the package.

// Dictionary file name definitions 
#define MULTI_RULE_DIC_FNAME      "njcon.a"     // Root dictionary file name       
#define MULTI_UBASE_DIC_FNAME     "njubase.a"   // Integrated dictionary 1 file name       
#define MULTI_UBASE1_DIC_FNAME    "njubase1.a"  // Integrated dictionary 1 file name       
#define MULTI_UBASE2_DIC_FNAME    "njubase2.a"  // Integrated dictionary 2 file name       
#define MULTI_FUZOKU_DIC_FNAME    "njfzk.a"     // Auxiliary word dictionary file name       
#define MULTI_TANKAN_DIC_FNAME    "njtan.a"     // Single kanji dictionary file name       
#define MULTI_NOYOMI_DIC_FNAME    "njyomi.a"    // No reading prediction dictionary file name 
#define MULTI_LEARN_DIC_FNAME     "njuserl.a"   // Learning dictionary file name         
#define MULTI_USER_DIC_FNAME      "njuserw.a"   // User dictionary file name       

// Dictionary auto restore definition 
#define MULTI_DIC_RESTORE_OFF     0    // Dictionary auto restore flag off      
#define MULTI_DIC_RESTORE_ON      1    // Dictionary auto restore flag on       

// Define whether there is a cache 
#define MULTI_DIC_SRHCACHE_OFF    0    // No cache         
#define MULTI_DIC_SRHCACHE_ON     1    // There is a cache         

// Items related to prediction candidate condition definitions 
#define MULTI_SET_FORECAST_LEARN    15   // Maximum number of learning dictionary prediction candidates to be obtained 
#define MULTI_SET_FORECAST_LIMIT    50   // Maximum number of prediction candidates to be obtained         
#define MULTI_SET_FORECAST_LIMITMAX 100  // Maximum number of prediction candidates to be obtained         

// Associative learning flag definition 
#define MULTI_CONNECT_OFF         0    // Disable associative learning               
#define MULTI_CONNECT_ON          1    // Enable associative learning               

// Filtered search flag definitions 
#define MULTI_RELATION_OFF        0    // Disable filtered search               
#define MULTI_RELATION_ON         1    // Enable filtered search               

// Morphological analysis and learning independent word specification definitions 
#define MULTI_MM_SELECT_SEGMENT   0    // Learn process results by phrase     
#define MULTI_MM_SELECT_STEM      1    // Learn only independent words in process results   

#define MULTI_PINYIN_MAX_CNT      100   // Maximum number of PinYin selection lists     

// Error Definitions 
#define MULTI_ERR_INVALID_PARAM     -1   // Invalid parameter           
#define MULTI_ERR_NO_MEMORY         -2   // Insufficient memory.               
#define MULTI_ERR_FAILED_FILE_OPEN  -3   // Failed to open file     
#define MULTI_ERR_FAILED_FILE_SEEK  -4   // File load location setting failed 
#define MULTI_ERR_FAILED_FILE_READ  -5   // Failed to load file         
#define MULTI_ERR_FAILED_FILE_WRITE -6   // File write failed     
#define MULTI_ERR_FAILED_DIC_MOUNT  -7   // Dictionary file mount failed 
#define MULTI_ERR_DIC_NO_MOUNT      -8   // No target dictionary data     
#define MULTI_ERR_INVALID_RESULT    -9   // Invalid data             
#define MULTI_ERR_NO_DATA           -10  // No target data         

// Uppercase/lowercase control 
#define MULTI_STRING_UPPER_NONE    0x00    // Lowercase           
#define MULTI_STRING_UPPER_TOP     0x01    // Initial cap only   
#define MULTI_STRING_UPPER_ALL     0x02    // All caps       

// Extended region setting type 
#define MULTI_EXT_TYPE_FREQ_LEARN  1       // Frequency learning region type 

// Hash size 
#define MULTI_STRING_HASH_MAX  (63)

#define MULTI_CHARSET_FROM_SIZE   4       // Character size definition before fuzzy search and replace 
#define MULTI_CHARSET_TO_SIZE     8       // Character size definition after fuzzy search and replace 

// Stroke delimit character definition 
static const wchar_t MULTI_CHAR_NUMSIGN = 0x0023;

// Stroke wildcard character definition 
static const wchar_t MULTI_CHAR_ASTERISK = 0x002A;

// Simplified delimit character definition 
static const wchar_t MULTI_CHAR_ROM1    = 0x2160;
static const wchar_t MULTI_CHAR_ROM4    = 0x2163;


/*
  
  
  

  
  
 
  
 */
#define MULTI_CONV_TO_WCHAR(to, from)                                            \
    { ((u8*)(to))[0] = (u8)(((from) >> 8) & 0x00ff);        \
        ((u8*)(to))[1] = (u8)((from) & 0x00ff); }

/* Please see man pages for details 
 
 */
typedef struct {
    IWNN_ANALYZE_OPTION analyzeInfo;      // Prediction options             
    u16         flexibleSearch;         // Enable/disable fuzzy searches             
    NN_PADDING2;
} MULTI_PREDICT;

/* Please see man pages for details 
 
 */
typedef struct {
    u8     devidePos;           // Phrase delimit position             
    u8     fixSegmentCnt;       // Number of phrases confirmed             
    u8     fixStrokeLen;        // Number of characters confirmed             
    NN_PADDING1;
} MULTI_CONVERSION;

/* Please see man pages for details 
 
 */
typedef struct {
    IWNN_RESULT    candidate[NJ_MAX_CANDIDATE];     // Process result structure (for candidate lists) 
    IWNN_RESULT    segment[NJ_MAX_PHRASE];          // Process result structure (for phrases)       
    u16    candidateCnt;                            // Number of process result structures stored         
    u16    segmentCnt;                              // Number of process result structures stored         
} MULTI_CANDIDATE;

/* Please see man pages for details 
 
 */
typedef struct {
    IWNN_DIC_SET  dicSet;                   // Dictionary set structure           
    IWNN_FLASH_DIC_INFO flashDicInfo[NJ_MAX_DIC];     // FLASH dictionary information              
    MULTI_FILE    *fileStream[NJ_MAX_DIC];          // The File Information Structure         
    u32   userDicSize;                      // User dictionary size           
    u32   learnDicSize;                     // Learning dictionary size             
    s16    userDicIndex;                    // User dictionary handle Index    
    s16    learnDicIndex;                   // Learning dictionary handle Index      
    u8    extareaType[NJ_MAX_DIC];          // Extended region setting type         
    u8    language;                         // Language type                 
    NN_PADDING3;
} MULTI_LANG_INFO;


/* Please see man pages for details 
 
 */
typedef struct {
    wchar_t   string[NJ_MAX_RESULT_LEN + NJ_TERM_LEN];
    s16  stringLen;
    NN_PADDING2;
} MULTI_STRING_ENTITY_T;

/* Please see man pages for details 
 
 */
typedef struct {
    MULTI_STRING_ENTITY_T*    entity[NJ_MAX_CANDIDATE];
    s32               count;
} MULTI_STRING_HASH_T;
/* Please see man pages for details 
 
 */
typedef struct {
    s16   separatorPos[NJ_MAX_LEN + NJ_TERM_LEN]; // Reading sting including delimit character   
    u8    separatorFlg;                             // Delimit character check flag (count) 
    u8    strokeFlg;                                // Load and store flag             
    u16   strokeLen;                                // Reading string length               
    wchar_t      stroke[NJ_MAX_LEN + NJ_TERM_LEN];          /* Please see man pages for details */
} MULTI_SIMPLIFIED_FILTER_OPTION;

/* Please see man pages for details 
 
 */
typedef struct {
    u8    from[MULTI_CHARSET_FROM_SIZE];        // Character before replacement     
    u8    to[MULTI_CHARSET_TO_SIZE];            // Character after replacement     
} MULTI_CHARSET;

/* Please see man pages for details 
 
 */
typedef struct {
    u8                      mode;                                               // Operation mode                         
    u8                      language;                                           // Language to use                           
    u8                      caps;                                               // Uppercase/lowercase determination                   
    u8                      connect;                                            // Associative learning flag                     
    u8                      relation;                                           // Filtered search flag                   
    u8                      candidateFlag;                                      // Candidate yes/no flag                         
    u16                     inputStrokeLen;                                     // Reading string length                       
    u16                     searchWordCnt;                                      // Number of word search results stored                 
    NN_PADDING2;
    wchar_t                 inputStroke[NJ_MAX_LEN+NJ_TERM_SIZE];               // Reading string                         
    wchar_t                 inputStroke2[NJ_MAX_LEN+NJ_TERM_SIZE];              // Reading string                             
    MULTI_LANG_INFO         languageInfo[MULTI_LANG_TYPE_USE_MAX];              // Storage region for already loaded language settings         
    IWNN_DIC_SET            dicSet;                                             // Dictionary set structure                   
    IWNN_OPTION             option;                                             // Option Settings                     
    IWNN_CHARSET            charSet;                                            // Fuzzy character set                     
    IWNN_CLASS              iwnn;                                               // Parsing class structure                   
    MULTI_PREDICT           predictInfo;                                        // Prediction information storage structure                 
    MULTI_CONVERSION        conversionInfo;                                     // Conversion information storage structure                 
    MULTI_CANDIDATE         candidateInfo;                                      // Structure for storing predictive conversion candidate information         
    IWNN_RESULT             searchWord[MULTI_LEARN_MAX_CNT];                    // Region for storing word search results               
    MULTI_STRING_ENTITY_T   hashEntity[NJ_MAX_CANDIDATE];                       // Hash region for deleting duplicate candidates         
    MULTI_STRING_HASH_T     hashString[MULTI_STRING_HASH_MAX];                  // Hash index for deleting duplicate candidates 
    MULTI_SIMPLIFIED_FILTER_OPTION simplifiedOption;                            // Simplified filtering option */
    wchar_t                 tone4Char;                                          // Toggle information for determining the tone */
    NN_PADDING2;
    u8   *memHandle;                                                            // Memory management handler               
    IWNN_CURSOR           cursor;                                               // Dictionary search cursor structure             
    IWNN_FLASH_DIC_INFO       flashDicInfo[NJ_MAX_DIC];                         // FLASH dictionary information              
    MULTI_FILE              fileStream[NJ_MAX_DIC];                             // The File Information Structure         
} MULTI_CLASS;


#endif // _MULTI_COM_H_ 
