/*-----------------------------------------------------------------------*
 Project: iWnn API Multi Language Demo: Multi Language Framework
 File:    multidic_engine.cpp

 Copyright (C)2009-2012 Nintendo Co., Ltd.  All rights reserved.

 These coded instructions, statements, and computer programs contain
 proprietary information of Nintendo of America Inc. and/or Nintendo
 Company Ltd., and are protected by Federal copyright law. They may
 not be disclosed to third parties or copied or duplicated in any form,
 in whole or in part, without the prior written consent of Nintendo.

 $Revision: $
 *-----------------------------------------------------------------------*/

#include "multidic_engine.h"



// Common/unique tables (Single tap definitions for English and European languages)
#include "multidic_table_com.h"
/* English-specific table */
#include "multidic_table_en.h"

/* Simplified Chinese-specific table */
#include "multidic_table_cn.h"

/* Latin-specific table */
#include "multidic_table_latin.h"

/* Russian-specific table */
#include "multidic_table_ru.h"

/*
 
 
 
 */
#define MULTI_SET_INIT_PREDICT(pre)                                             \
           ((pre)->analyzeInfo.mode                 = 0,                        \
            (pre)->analyzeInfo.forecastLearnLimit   = MULTI_SET_FORECAST_LEARN, \
            (pre)->analyzeInfo.forecastLimit        = MULTI_SET_FORECAST_LIMIT, \
            (pre)->analyzeInfo.charMin              = 0,                        \
            (pre)->analyzeInfo.charMax              = NJ_MAX_LEN,                \
            (pre)->flexibleSearch                   = MULTI_FLEXIBLE_SEARCH_ON)

/*
 
 
 
 */
#define MULTI_SET_INIT_CONVERT(conv)        \
           ((conv)->devidePos       = 0,    \
            (conv)->fixSegmentCnt   = 0,    \
            (conv)->fixStrokeLen    = 0)


/*
 
 
 
 */
#define MULTI_SET_INIT_CANDIDATE(cand)  \
           ((cand)->candidateCnt = 0,   \
            (cand)->segmentCnt   = 0)




//**************************************************************************
//                          Prototype Declarations                                
//**************************************************************************


//**************************************************************************
//                            Function definitions                                      
//**************************************************************************

/*
 
 
  
   
   
   
   
   
   
   
 
 
 
 
 
 
 
   
     
 
 */
s16 initializeLanguage(u8 *languageEngineClass)
{
    u8  i, j;
    MULTI_CLASS   *languageEngine;

    if (languageEngineClass == NULL) {
        return MULTI_ERR_INVALID_PARAM;
    }
    languageEngine = (MULTI_CLASS *)languageEngineClass;    


    // Initialize language settings 
    languageEngine->language = MULTI_LANG_TYPE_NONE;
    for (i = 0; i < MULTI_LANG_TYPE_USE_MAX; i++) {
        languageEngine->languageInfo[i].language        = MULTI_LANG_TYPE_NONE;
        languageEngine->languageInfo[i].userDicIndex    = -1;
        languageEngine->languageInfo[i].learnDicIndex   = -1;
        for (j = 0; j < NJ_MAX_DIC; j++) {
            languageEngine->languageInfo[i].extareaType[j] = 0;
            internalMemset(&(languageEngine->languageInfo[i].flashDicInfo[j]), 
                            0x00, 
                            sizeof(IWNN_FLASH_DIC_INFO));
        }
    }

    // iWnn engine option settings (number of multiple phrase conversion auto start candidates) 
    languageEngine->option.autoConversionCnt     = MULTI_OPTION_MULTI_CONVERSION;
    languageEngine->option.phase2Filter    = NULL;
    languageEngine->option.phase2Option    = (void*)languageEngine;
    languageEngine->option.phase1Filter    = NULL;
    languageEngine->option.phase1Option    = (void*)languageEngine;

    // Initialize hash for deleting duplicate candidates 
    internalHashFilterInit(languageEngine);

    // Initial setting for predictive search conditions 
    MULTI_SET_INIT_PREDICT(&(languageEngine->predictInfo));

    // Engine initialization process 
    return mw::iwnn::IWNN::NjxInit(&(languageEngine->iwnn), &(languageEngine->option));

}


/*
 

 
  
  
  

 

 
 */
void terminateLanguage(u8 *languageEngineClass)
{
    u8  i;
    MULTI_CLASS *languageEngine;

    if (languageEngineClass == NULL) {
        return;
    }
    // Cast to MULTI_CLASS type 
    languageEngine = (MULTI_CLASS *)languageEngineClass;

    for (i = 0; i < MULTI_LANG_TYPE_USE_MAX; i++) {
        // Dictionary unmount process 
        freeLanguage(languageEngineClass, languageEngine->languageInfo[i].language);
    }

    // Destroy the European language filter region 
    internalHashFilterDestroy(languageEngine);

    // MULTI_CLASS region initialization 
    internalMemset(languageEngine, 0x00, sizeof(MULTI_CLASS));

    return;
}


/*
 

 
  

 
 
 
 
 
 */
s16 loadLanguage(u8 *languageEngineClass, u8 language)
{
    u8  dicCnt;
    u8  fileName[256];
    u8  addInfo[256];
    s32 size;
    s32 i;    
    MULTI_CLASS     *languageEngine;
    MULTI_LANG_INFO *languageInfo = NULL;


    if (languageEngineClass == NULL) {
        return -1;
    }
    // Cast to MULTI_CLASS type 
    languageEngine = (MULTI_CLASS *)languageEngineClass;

    // Check whether data is already loaded
    for (i = 0; i < MULTI_LANG_TYPE_USE_MAX; i++) {
        if (languageEngine->languageInfo[i].language == language) {
            // Dictionary already loaded
            return -1;
        } else {
            if (languageEngine->languageInfo[i].language == MULTI_LANG_TYPE_NONE) {
                // Save storage location 
                languageInfo = &(languageEngine->languageInfo[i]);

                break;
            }
        }
    }
    if (i == MULTI_LANG_TYPE_USE_MAX) {
        // If there is nowhere to mount the dictionary 
        return -1;
    }
    
    dicCnt = 0;
    switch (language) {
    case MULTI_LANG_TYPE_JP_FULL:
        // Rule dictionary mount 
        internalSprintf (fileName, "%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_JA, "njcon.a");
        size = internalReadBinary(&(languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION]),languageEngine, fileName);
        if (languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION] == NULL || size == 0) {
            // Rule dictionary load error 
            return -1;
        }
        languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_PREDICTION] = languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION];
        languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_MORPHOLIZE] = languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION];

        // Learning dictionary mount 
        internalSprintf (fileName, "%s/%s_%s.a", MULTI_REWRITE_DIR, "njuserl", MULTI_LANG_DIR_JA);
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_LEARN, fileName, NULL, NULL, NULL,
                             NJ_DEFAULT_FREQUENCY_DIC_LEARN_CONVERSION_BASE, NJ_DEFAULT_FREQUENCY_DIC_LEARN_CONVERSION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_LEARN_PREDICTION_BASE, NJ_DEFAULT_FREQUENCY_DIC_LEARN_PREDICTION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_LEARN_MORPHOLIZE_BASE, NJ_DEFAULT_FREQUENCY_DIC_LEARN_MORPHOLIZE_HIGH,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // User dictionary mount 
        internalSprintf (fileName, "%s/%s_%s.a", MULTI_REWRITE_DIR, "njuserw", MULTI_LANG_DIR_JA);
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_USER, fileName, NULL, NULL, NULL,
                             NJ_DEFAULT_FREQUENCY_DIC_USER_CONVERSION_BASE,  NJ_DEFAULT_FREQUENCY_DIC_USER_CONVERSION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_USER_PREDICTION_BASE,  NJ_DEFAULT_FREQUENCY_DIC_USER_PREDICTION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_USER_MORPHOLIZE_BASE,  NJ_DEFAULT_FREQUENCY_DIC_USER_MORPHOLIZE_HIGH,
                             0, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // Integrated Dictionary 1 mount 
        internalSprintf (fileName, "%s/%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_JA, MULTI_32_DIR, "njubase1.a");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_FLASH, fileName, NULL, NULL, NULL,
                             NJ_DEFAULT_FREQUENCY_DIC_UNION_CONVERSION_BASE, NJ_DEFAULT_FREQUENCY_DIC_UNION_CONVERSION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_UNION_PREDICTION_BASE, NJ_DEFAULT_FREQUENCY_DIC_UNION_PREDICTION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_UNION_MORPHOLIZE_BASE, NJ_DEFAULT_FREQUENCY_DIC_UNION_MORPHOLIZE_HIGH,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // Integrated Dictionary 2 mount 
        internalSprintf (fileName, "%s/%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_JA, MULTI_32_DIR, "njubase2.a");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_FLASH, fileName, NULL, NULL, NULL,
                             NJ_DEFAULT_FREQUENCY_DIC_UNION_CONVERSION_BASE, NJ_DEFAULT_FREQUENCY_DIC_UNION_CONVERSION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_UNION_PREDICTION_BASE, NJ_DEFAULT_FREQUENCY_DIC_UNION_PREDICTION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_UNION_MORPHOLIZE_BASE, NJ_DEFAULT_FREQUENCY_DIC_UNION_MORPHOLIZE_HIGH,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // Auxiliary word dictionary mount 
        internalSprintf (fileName, "%s/%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_JA, MULTI_32_DIR, "njfzk.a");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_NORMAL, fileName, NULL, NULL, NULL,
                             NJ_DEFAULT_FREQUENCY_DIC_ANCILLARY_CONVERSION_BASE,   NJ_DEFAULT_FREQUENCY_DIC_ANCILLARY_CONVERSION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_ANCILLARY_PREDICTION_BASE,   NJ_DEFAULT_FREQUENCY_DIC_ANCILLARY_PREDICTION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_ANCILLARY_MORPHOLIZE_BASE,   NJ_DEFAULT_FREQUENCY_DIC_ANCILLARY_MORPHOLIZE_HIGH,
                             0, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // Independent word dictionary (single kanji dictionary) mount 
        internalSprintf (fileName, "%s/%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_JA, MULTI_32_DIR, "njtan.a");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_SINGLE_KANJI, fileName, NULL, NULL, NULL,
                             NJ_DEFAULT_FREQUENCY_DIC_SINGLE_CONVERSION_BASE,   NJ_DEFAULT_FREQUENCY_DIC_SINGLE_CONVERSION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_SINGLE_PREDICTION_BASE,   NJ_DEFAULT_FREQUENCY_DIC_SINGLE_PREDICTION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_SINGLE_MORPHOLIZE_BASE,   NJ_DEFAULT_FREQUENCY_DIC_SINGLE_MORPHOLIZE_HIGH,
                             0, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // Extended no reading analysis dictionary mount 
        internalSprintf (fileName, "%s/%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_JA, MULTI_32_DIR, "njexyomi.a");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_NORMAL, fileName, NULL, NULL, NULL,
                             NJ_DEFAULT_FREQUENCY_DIC_EXTENSION_READING_CONVERSION_BASE, NJ_DEFAULT_FREQUENCY_DIC_EXTENSION_READING_CONVERSION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_EXTENSION_READING_PREDICTION_BASE, NJ_DEFAULT_FREQUENCY_DIC_EXTENSION_READING_PREDICTION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_EXTENSION_READING_MORPHOLIZE_BASE, NJ_DEFAULT_FREQUENCY_DIC_EXTENSION_READING_MORPHOLIZE_HIGH,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;
        
        // Implementation of pseudo-dictionary sample 
        languageInfo->dicSet.dic[dicCnt].type = NJ_DIC_HANDLE_TYPE_PROGRAM;
        languageInfo->dicSet.dic[dicCnt].extensionArea = NULL;
        languageInfo->dicSet.dic[dicCnt].addInfo[0] = NULL;
        languageInfo->dicSet.dic[dicCnt].addInfo[1] = NULL;
        languageInfo->dicSet.dic[dicCnt].handle = (u8*)(mw::iwnn::IWNN::NjexPseudoDic);
        languageInfo->dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_CONVERSION].base = 
                                                        NJ_DEFAULT_FREQUENCY_DIC_PRORAM_CONVERSION_BASE;
        languageInfo->dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_CONVERSION].high = 
                                                        NJ_DEFAULT_FREQUENCY_DIC_PRORAM_CONVERSION_HIGH;
        languageInfo->dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_PREDICTION].base = 
                                                        NJ_DEFAULT_FREQUENCY_DIC_PRORAM_PREDICTION_BASE;
        languageInfo->dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_PREDICTION].high = 
                                                        NJ_DEFAULT_FREQUENCY_DIC_PRORAM_PREDICTION_HIGH;
        languageInfo->dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_MORPHOLIZE].base = 
                                                        NJ_DEFAULT_FREQUENCY_DIC_PRORAM_MORPHOLIZE_BASE;
        languageInfo->dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_MORPHOLIZE].high = 
                                                        NJ_DEFAULT_FREQUENCY_DIC_PRORAM_MORPHOLIZE_HIGH;
        languageInfo->dicSet.dic[dicCnt].searchCache = NULL;
        languageInfo->dicSet.dic[dicCnt].limit = 0;
        dicCnt++;

        // Implementation of pseudo-dictionary sample 
        languageInfo->dicSet.dic[dicCnt].type = NJ_DIC_HANDLE_TYPE_PROGRAM;
        languageInfo->dicSet.dic[dicCnt].extensionArea = NULL;
        languageInfo->dicSet.dic[dicCnt].addInfo[0] = NULL;
        languageInfo->dicSet.dic[dicCnt].addInfo[1] = NULL;
        languageInfo->dicSet.dic[dicCnt].handle = (u8*)(mw::iwnn::IWNN::NjexPredictionPseudoDic);
        languageInfo->dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_CONVERSION].base =
                                                    NJ_DEFAULT_FREQUENCY_DIC_LIMIT_PREDICTION_PRORAM_CONVERSION_BASE;
        languageInfo->dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_CONVERSION].high = 
                                                    NJ_DEFAULT_FREQUENCY_DIC_LIMIT_PREDICTION_PRORAM_CONVERSION_HIGH;
        languageInfo->dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_PREDICTION].base = 
                                                    NJ_DEFAULT_FREQUENCY_DIC_LIMIT_PREDICTION_PRORAM_PREDICTION_BASE;
        languageInfo->dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_PREDICTION].high = 
                                                    NJ_DEFAULT_FREQUENCY_DIC_LIMIT_PREDICTION_PRORAM_PREDICTION_HIGH;
        languageInfo->dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_MORPHOLIZE].base =
                                                    NJ_DEFAULT_FREQUENCY_DIC_LIMIT_PREDICTION_PRORAM_MORPHOLIZE_BASE;
        languageInfo->dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_MORPHOLIZE].high =
                                                    NJ_DEFAULT_FREQUENCY_DIC_LIMIT_PREDICTION_PRORAM_MORPHOLIZE_HIGH;
        languageInfo->dicSet.dic[dicCnt].searchCache = NULL;
        languageInfo->dicSet.dic[dicCnt].limit = 0;
        dicCnt++;
        
        languageInfo->language = language;
        break;

    case MULTI_LANG_TYPE_CN_10S:      
    case MULTI_LANG_TYPE_CN_FULL:
        // Rule dictionary mount 
        internalSprintf (fileName, "%s/%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_ZH, MULTI_LANG_DIR_CN, "njcon.a");
        size = internalReadBinary(&(languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION]),languageEngine, fileName);
        if (languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION] == NULL || size == 0) {
            // Rule dictionary load error 
            return -1;
        }
        languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_PREDICTION] = languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION];
        languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_MORPHOLIZE] = languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION];

        // Learning dictionary mount 
        internalSprintf (fileName, "%s/%s_%s_%s_%s.a", 
                         MULTI_REWRITE_DIR, "njuserl", MULTI_LANG_DIR_ZH, MULTI_LANG_DIR_CN, MULTI_INTYPE_PINYIN);
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_LEARN_ADD, fileName, NULL, NULL, NULL,
                             NJ_DEFAULT_FREQUENCY_DIC_LEARN_CONVERSION_BASE, NJ_DEFAULT_FREQUENCY_DIC_LEARN_CONVERSION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_LEARN_PREDICTION_BASE, NJ_DEFAULT_FREQUENCY_DIC_LEARN_PREDICTION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_LEARN_MORPHOLIZE_BASE, NJ_DEFAULT_FREQUENCY_DIC_LEARN_MORPHOLIZE_HIGH,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // User dictionary mount 
        internalSprintf (fileName, "%s/%s_%s_%s_%s.a", MULTI_REWRITE_DIR, 
                         "njuserw", MULTI_LANG_DIR_ZH, MULTI_LANG_DIR_CN, MULTI_INTYPE_PINYIN);
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_USER, fileName, NULL, NULL, NULL,
                             NJ_DEFAULT_FREQUENCY_DIC_USER_CONVERSION_BASE,  NJ_DEFAULT_FREQUENCY_DIC_USER_CONVERSION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_USER_PREDICTION_BASE,  NJ_DEFAULT_FREQUENCY_DIC_USER_PREDICTION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_USER_MORPHOLIZE_BASE,  NJ_DEFAULT_FREQUENCY_DIC_USER_MORPHOLIZE_HIGH,
                             0, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // Conversion dictionary mount 
        internalSprintf (fileName, "%s/%s/%s/%s/%s/%s", MULTI_DIC_DIR, 
                         MULTI_LANG_DIR_ZH, MULTI_LANG_DIR_CN, 
                         MULTI_INTYPE_PINYIN, MULTI_GB18030_DIR, "njubase1.a");
        internalSprintf (addInfo, "%s/%s/%s/%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_ZH, 
                         MULTI_LANG_DIR_CN, MULTI_INTYPE_PINYIN, MULTI_GB18030_DIR, "njubase1.ad1");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_FLASH, fileName, NULL, addInfo, NULL,
                             100, 400,
                             100, 560,
                             10,    0,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // Conversion dictionary mount 
        internalSprintf (fileName, "%s/%s/%s/%s/%s/%s", MULTI_DIC_DIR, 
                         MULTI_LANG_DIR_ZH, MULTI_LANG_DIR_CN, 
                         MULTI_INTYPE_PINYIN, MULTI_GB18030_DIR, "njubase2.a");
        internalSprintf (addInfo, "%s/%s/%s/%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_ZH, 
                         MULTI_LANG_DIR_CN, MULTI_INTYPE_PINYIN, MULTI_GB18030_DIR, "njubase2.ad1");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_FLASH, fileName, NULL, addInfo, NULL,
                             100, 400,
                             100, 560,
                             10,    0,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // Conversion dictionary mount 
        internalSprintf (fileName, "%s/%s/%s/%s/%s/%s", MULTI_DIC_DIR, 
                         MULTI_LANG_DIR_ZH, MULTI_LANG_DIR_CN, MULTI_INTYPE_PINYIN, MULTI_GB18030_DIR, "njubase3.a");
        internalSprintf (addInfo, "%s/%s/%s/%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_ZH, 
                         MULTI_LANG_DIR_CN, MULTI_INTYPE_PINYIN, MULTI_GB18030_DIR, "njubase3.ad1");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_FLASH, fileName, NULL, addInfo, NULL,
                             100, 400,
                             100, 560,
                             10,    0,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // Single kanji dictionary mount 
        internalSprintf (fileName, "%s/%s/%s/%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_ZH, 
                         MULTI_LANG_DIR_CN, MULTI_INTYPE_PINYIN, MULTI_GB18030_DIR, "njtan.a");
        internalSprintf (addInfo, "%s/%s/%s/%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_ZH, 
                         MULTI_LANG_DIR_CN, MULTI_INTYPE_PINYIN, MULTI_GB18030_DIR, "njtan.ad1");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_FLASH, fileName, NULL, addInfo, NULL,
                             100, 400,
                             400, 800,
                             10,    0,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // Extended no reading analysis dictionary mount 
        internalSprintf (fileName, "%s/%s/%s/%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_ZH, 
                         MULTI_LANG_DIR_CN, MULTI_INTYPE_PINYIN, MULTI_GB18030_DIR, "njexyomi.a");
        internalSprintf (addInfo, "%s/%s/%s/%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_ZH, 
                         MULTI_LANG_DIR_CN, MULTI_INTYPE_PINYIN, MULTI_GB18030_DIR, "njexyomi.ad1");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_NORMAL, fileName, NULL, addInfo, NULL,
                             10,    0,
                             0,   100,
                             10,    0,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        languageInfo->language = language;
        break;

    case MULTI_LANG_TYPE_TW_P10S:
    case MULTI_LANG_TYPE_TW_PFULL:
        // Rule dictionary mount 
        internalSprintf (fileName, "%s/%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_ZH, MULTI_LANG_DIR_TW, "njcon.a");
        size = internalReadBinary( &(languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION]),languageEngine, fileName);
        if (languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION] == NULL || size == 0) {
            // Rule dictionary load error 
            return -1;
        }
        languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_PREDICTION] = languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION];
        languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_MORPHOLIZE] = languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION];

        // Learning dictionary mount 
        internalSprintf (fileName, "%s/%s_%s_%s_%s.a", MULTI_REWRITE_DIR, 
                         "njuserl", MULTI_LANG_DIR_ZH, MULTI_LANG_DIR_TW, MULTI_INTYPE_PINYIN);
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_LEARN, fileName, NULL, NULL, NULL,
                             NJ_DEFAULT_FREQUENCY_DIC_LEARN_CONVERSION_BASE, NJ_DEFAULT_FREQUENCY_DIC_LEARN_CONVERSION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_LEARN_PREDICTION_BASE, NJ_DEFAULT_FREQUENCY_DIC_LEARN_PREDICTION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_LEARN_MORPHOLIZE_BASE, NJ_DEFAULT_FREQUENCY_DIC_LEARN_MORPHOLIZE_HIGH,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // User dictionary mount 
        internalSprintf (fileName, "%s/%s_%s_%s_%s.a", MULTI_REWRITE_DIR, 
                         "njuserw", MULTI_LANG_DIR_ZH, MULTI_LANG_DIR_TW, MULTI_INTYPE_PINYIN);
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_USER, fileName, NULL, NULL, NULL,
                             NJ_DEFAULT_FREQUENCY_DIC_USER_CONVERSION_BASE,  NJ_DEFAULT_FREQUENCY_DIC_USER_CONVERSION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_USER_PREDICTION_BASE,  NJ_DEFAULT_FREQUENCY_DIC_USER_PREDICTION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_USER_MORPHOLIZE_BASE,  NJ_DEFAULT_FREQUENCY_DIC_USER_MORPHOLIZE_HIGH,
                             0, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // Conversion dictionary mount 
        internalSprintf (fileName, "%s/%s/%s/%s/%s", MULTI_DIC_DIR, 
                         MULTI_LANG_DIR_ZH, MULTI_LANG_DIR_TW, MULTI_INTYPE_PINYIN, "njubase1.a");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_FLASH, fileName, NULL, NULL, NULL,
                             100, 400,
                             100, 560,
                             10,    0,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // Predictive conversion dictionary mount 
        internalSprintf (fileName, "%s/%s/%s/%s/%s", MULTI_DIC_DIR, 
                         MULTI_LANG_DIR_ZH, MULTI_LANG_DIR_TW, MULTI_INTYPE_PINYIN, "njubase2.a");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_FLASH, fileName, NULL, NULL, NULL,
                             100, 400,
                             100, 560,
                             10,    0,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // Extended no reading analysis dictionary mount 
        internalSprintf (fileName, "%s/%s/%s/%s/%s", MULTI_DIC_DIR, 
                         MULTI_LANG_DIR_ZH, MULTI_LANG_DIR_TW, MULTI_INTYPE_PINYIN, "njexyomi.a");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_NORMAL, fileName, NULL, NULL, NULL,
                             10,    0,
                             0,   100,
                             10,    0,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // Single kanji dictionary mount 
        internalSprintf (fileName, "%s/%s/%s/%s/%s", MULTI_DIC_DIR, 
                         MULTI_LANG_DIR_ZH, MULTI_LANG_DIR_TW, MULTI_INTYPE_PINYIN, "njtan.a");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_FLASH, fileName, NULL, NULL, NULL,
                             100, 400,
                             400, 800,
                             10,    0,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;       
        languageInfo->language = language;
        break;

    case MULTI_LANG_TYPE_TW_B10S:
    case MULTI_LANG_TYPE_TW_BFULL:
        // Rule dictionary mount 
        internalSprintf (fileName, "%s/%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_ZH, MULTI_LANG_DIR_TW, "njcon.a");
        size = internalReadBinary(&(languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION]),languageEngine, fileName);
        if (languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION] == NULL || size == 0) {
            // Rule dictionary load error 
            return -1;
        }
        languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_PREDICTION] = languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION];
        languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_MORPHOLIZE] = languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION];

        // Learning dictionary mount 
        internalSprintf (fileName, "%s/%s_%s_%s_%s.a", MULTI_REWRITE_DIR, 
                         "njuserl", MULTI_LANG_DIR_ZH, MULTI_LANG_DIR_TW, MULTI_INTYPE_BOPOMOFO);
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_LEARN, fileName, NULL, NULL, NULL,
                             NJ_DEFAULT_FREQUENCY_DIC_LEARN_CONVERSION_BASE, NJ_DEFAULT_FREQUENCY_DIC_LEARN_CONVERSION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_LEARN_PREDICTION_BASE, NJ_DEFAULT_FREQUENCY_DIC_LEARN_PREDICTION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_LEARN_MORPHOLIZE_BASE, NJ_DEFAULT_FREQUENCY_DIC_LEARN_MORPHOLIZE_HIGH,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // User dictionary mount 
        internalSprintf (fileName, "%s/%s_%s_%s_%s.a", MULTI_REWRITE_DIR, 
                         "njuserw", MULTI_LANG_DIR_ZH, MULTI_LANG_DIR_TW, MULTI_INTYPE_BOPOMOFO);
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_USER, fileName, NULL, NULL, NULL,
                             NJ_DEFAULT_FREQUENCY_DIC_USER_CONVERSION_BASE,  NJ_DEFAULT_FREQUENCY_DIC_USER_CONVERSION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_USER_PREDICTION_BASE,  NJ_DEFAULT_FREQUENCY_DIC_USER_PREDICTION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_USER_MORPHOLIZE_BASE,  NJ_DEFAULT_FREQUENCY_DIC_USER_MORPHOLIZE_HIGH,
                             0, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // Conversion dictionary mount 
        internalSprintf (fileName, "%s/%s/%s/%s/%s", MULTI_DIC_DIR, 
                         MULTI_LANG_DIR_ZH, MULTI_LANG_DIR_TW, MULTI_INTYPE_BOPOMOFO, "njubase1.a");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_FLASH, fileName, NULL, NULL, NULL,
                             100, 400,
                             100, 560,
                             10,    0,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // Predictive conversion dictionary mount 
        internalSprintf (fileName, "%s/%s/%s/%s/%s", MULTI_DIC_DIR, 
                         MULTI_LANG_DIR_ZH, MULTI_LANG_DIR_TW, MULTI_INTYPE_BOPOMOFO, "njubase2.a");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_FLASH, fileName, NULL, NULL, NULL,
                             100, 400,
                             100, 560,
                             10,    0,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // Single kanji dictionary mount 
        internalSprintf (fileName, "%s/%s/%s/%s/%s", MULTI_DIC_DIR, 
                         MULTI_LANG_DIR_ZH, MULTI_LANG_DIR_TW, MULTI_INTYPE_BOPOMOFO, "njtan.a");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_FLASH, fileName, NULL, NULL, NULL,
                             100, 400,
                             400, 800,
                             10,    0,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // Extended no reading analysis dictionary mount 
        internalSprintf (fileName, "%s/%s/%s/%s/%s", MULTI_DIC_DIR, 
                         MULTI_LANG_DIR_ZH, MULTI_LANG_DIR_TW, MULTI_INTYPE_BOPOMOFO, "njexyomi.a");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_NORMAL, fileName, NULL, NULL, NULL,
                             10,    0,
                             0,   100,
                             10,    0,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;
        
        languageInfo->language = language;
        break;
//2011/10/11 Commented out the Cantonese dictionary because it was deleted from the package.
//    case MULTI_LANG_TYPE_HK_10S:
/*    case MULTI_LANG_TYPE_HK_FULL:        
        // Rule dictionary mount 
        internalSprintf (fileName, "%s/%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_ZH, MULTI_LANG_DIR_HK, "njcon.a");
        size = internalReadBinary(&(languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION]),languageEngine, fileName);
        if (languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION] == NULL || size == 0) {
            // Rule dictionary load error 
            return -1;
        }
        languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_PREDICTION] = languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION];
        languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_MORPHOLIZE] = languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION];

        // Learning dictionary mount 
        internalSprintf (fileName, "%s/%s_%s_%s_%s.a", 
                         MULTI_REWRITE_DIR, "njuserl", MULTI_LANG_DIR_ZH, MULTI_LANG_DIR_HK, MULTI_INTYPE_JYUTPING);
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_LEARN_ADD, fileName, NULL, NULL, NULL,
                             NJ_DEFAULT_FREQUENCY_DIC_LEARN_CONVERSION_BASE, NJ_DEFAULT_FREQUENCY_DIC_LEARN_CONVERSION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_LEARN_PREDICTION_BASE, NJ_DEFAULT_FREQUENCY_DIC_LEARN_PREDICTION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_LEARN_MORPHOLIZE_BASE, NJ_DEFAULT_FREQUENCY_DIC_LEARN_MORPHOLIZE_HIGH,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // User dictionary mount 
        internalSprintf (fileName, "%s/%s_%s_%s_%s.a", MULTI_REWRITE_DIR, 
                         "njuserw", MULTI_LANG_DIR_ZH, MULTI_LANG_DIR_HK, MULTI_INTYPE_JYUTPING);
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_USER, fileName, NULL, NULL, NULL,
                             NJ_DEFAULT_FREQUENCY_DIC_USER_CONVERSION_BASE,  NJ_DEFAULT_FREQUENCY_DIC_USER_CONVERSION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_USER_PREDICTION_BASE,  NJ_DEFAULT_FREQUENCY_DIC_USER_PREDICTION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_USER_MORPHOLIZE_BASE,  NJ_DEFAULT_FREQUENCY_DIC_USER_MORPHOLIZE_HIGH,
                             0, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // Conversion dictionary mount 
        internalSprintf (fileName, "%s/%s/%s/%s/%s", MULTI_DIC_DIR, 
                         MULTI_LANG_DIR_ZH, MULTI_LANG_DIR_HK, 
                         MULTI_INTYPE_JYUTPING, "njubase1.a");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_FLASH, fileName, NULL, NULL, NULL,
                             100, 400,
                             100, 560,
                             10,    0,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // Conversion dictionary mount 
        internalSprintf (fileName, "%s/%s/%s/%s/%s", MULTI_DIC_DIR, 
                         MULTI_LANG_DIR_ZH, MULTI_LANG_DIR_HK, 
                         MULTI_INTYPE_JYUTPING, "njubase2.a");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_FLASH, fileName, NULL, NULL, NULL,
                             100, 400,
                             100, 560,
                             10,    0,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // Single kanji dictionary mount 
        internalSprintf (fileName, "%s/%s/%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_ZH, 
                         MULTI_LANG_DIR_HK, MULTI_INTYPE_JYUTPING, "njtan.a");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_FLASH, fileName, NULL, NULL, NULL,
                             100, 400,
                             100, 800,
                             10,    0,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // Extended no reading analysis dictionary mount 
        internalSprintf (fileName, "%s/%s/%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_ZH, 
                         MULTI_LANG_DIR_HK, MULTI_INTYPE_JYUTPING, "njexyomi.a");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_NORMAL, fileName, NULL, NULL, NULL,
                             10,    0,
                             0,   100,
                             10,    0,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        languageInfo->language = language;
        break;
*/
//2011/12/05 Added support for the Cantonese dictionary (Cangjie) package
    case MULTI_LANG_TYPE_HK_CANGJIE:
        // Rule dictionary mount 
        internalSprintf (fileName, "%s/%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_ZH, MULTI_LANG_DIR_HK, "njcon.a");
        size = internalReadBinary(&(languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION]),languageEngine, fileName);
        if (languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION] == NULL || size == 0) {
            // Rule dictionary load error 
            return -1;
        }
        languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_PREDICTION] = languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION];
        languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_MORPHOLIZE] = languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION];

        // Learning dictionary mount 
        internalSprintf (fileName, "%s/%s_%s_%s_%s.a", 
                         MULTI_REWRITE_DIR, "njuserl", MULTI_LANG_DIR_ZH, MULTI_LANG_DIR_HK, MULTI_INTYPE_CANGJIE);
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_LEARN_ADD, fileName, NULL, NULL, NULL,
                             NJ_DEFAULT_FREQUENCY_DIC_LEARN_CONVERSION_BASE, NJ_DEFAULT_FREQUENCY_DIC_LEARN_CONVERSION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_LEARN_PREDICTION_BASE, NJ_DEFAULT_FREQUENCY_DIC_LEARN_PREDICTION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_LEARN_MORPHOLIZE_BASE, NJ_DEFAULT_FREQUENCY_DIC_LEARN_MORPHOLIZE_HIGH,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // User dictionary mount 
        internalSprintf (fileName, "%s/%s_%s_%s_%s.a", MULTI_REWRITE_DIR, 
                         "njuserw", MULTI_LANG_DIR_ZH, MULTI_LANG_DIR_HK, MULTI_INTYPE_CANGJIE);
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_USER, fileName, NULL, NULL, NULL,
                             NJ_DEFAULT_FREQUENCY_DIC_USER_CONVERSION_BASE,  NJ_DEFAULT_FREQUENCY_DIC_USER_CONVERSION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_USER_PREDICTION_BASE,  NJ_DEFAULT_FREQUENCY_DIC_USER_PREDICTION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_USER_MORPHOLIZE_BASE,  NJ_DEFAULT_FREQUENCY_DIC_USER_MORPHOLIZE_HIGH,
                             0, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;
        // Mount the Cangjie dictionary 
        internalSprintf (fileName, "%s/%s/%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_ZH, 
                         MULTI_LANG_DIR_HK, MULTI_INTYPE_CANGJIE, "njcangjie.a");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_NORMAL, fileName, NULL, NULL, NULL,
                             100, 400,
                             100, 560,
                             10,    0,
                             1, 0, dicCnt) < 0){
            return -1;
        }

        dicCnt++;
        languageInfo->language = language;
        break;
    case MULTI_LANG_TYPE_KO_FULL:        
        // Rule dictionary mount 
        internalSprintf (fileName, "%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_KO, "njcon.a");
        size = internalReadBinary(&(languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION]),languageEngine, fileName);
        if (languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION] == NULL || size == 0) {
            // Rule dictionary load error 
            return -1;
        }
        languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_PREDICTION] = languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION];
        languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_MORPHOLIZE] = languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION];

        // Learning dictionary mount 
        internalSprintf (fileName, "%s/%s_%s.a", MULTI_REWRITE_DIR, "njuserl", MULTI_LANG_DIR_KO);
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_LEARN, fileName, NULL, NULL, NULL,
                             NJ_DEFAULT_FREQUENCY_DIC_LEARN_CONVERSION_BASE, NJ_DEFAULT_FREQUENCY_DIC_LEARN_CONVERSION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_LEARN_PREDICTION_BASE, NJ_DEFAULT_FREQUENCY_DIC_LEARN_PREDICTION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_LEARN_MORPHOLIZE_BASE, NJ_DEFAULT_FREQUENCY_DIC_LEARN_MORPHOLIZE_HIGH,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // User dictionary mount 
        internalSprintf (fileName, "%s/%s_%s.a", MULTI_REWRITE_DIR, "njuserw", MULTI_LANG_DIR_KO);
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_USER, fileName, NULL, NULL, NULL,
                             NJ_DEFAULT_FREQUENCY_DIC_USER_CONVERSION_BASE,  NJ_DEFAULT_FREQUENCY_DIC_USER_CONVERSION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_USER_PREDICTION_BASE,  NJ_DEFAULT_FREQUENCY_DIC_USER_PREDICTION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_USER_MORPHOLIZE_BASE,  NJ_DEFAULT_FREQUENCY_DIC_USER_MORPHOLIZE_HIGH,
                             0, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // Conversion dictionary mount 
        internalSprintf (fileName, "%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_KO, "njubase1.a");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_FLASH, fileName, NULL, NULL, NULL,
                             400, 500,
                             100, 560,
                             10,    0,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // Conversion dictionary mount 
        internalSprintf (fileName, "%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_KO, "njubase2.a");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_FLASH, fileName, NULL, NULL, NULL,
                             400, 500,
                             100, 560,
                             10,    0,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

/*2010/11/05 Commented out because the font does not exist in the target
        // Single kanji dictionary mount 
        internalSprintf (fileName, "%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_KO, "njtan.a");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_NORMAL, fileName, NULL, NULL, NULL,
                             0, 10,
                             10, 0,
                             0, 10,
                             0, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;
*/
        // Extended no reading analysis dictionary mount 
        internalSprintf (fileName, "%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_KO, "njexyomi.a");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_NORMAL, fileName, NULL, NULL, NULL,
                             10,    0,
                             0,   100,
                             10,    0,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;
        languageInfo->language = language;
        break;

    case MULTI_LANG_TYPE_EN_US_10S:
    case MULTI_LANG_TYPE_EN_US_FULL:
        // Rule dictionary mount 
        internalSprintf (fileName, "%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_EN, "njcon.a");
        size = internalReadBinary(&(languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION]),languageEngine, fileName);
        if (languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION] == NULL || size == 0) {
            // Rule dictionary load error 
            return -1;
        }
        languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_PREDICTION] = languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION];
        languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_MORPHOLIZE] = languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION];

        // Learning dictionary mount 
        internalSprintf (fileName, "%s/%s_%s_%s.a", MULTI_REWRITE_DIR, "njuserl", MULTI_LANG_DIR_EN, MULTI_LANG_DIR_EN_US);
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_LEARN, fileName, NULL, NULL, NULL,
                             NJ_DEFAULT_FREQUENCY_DIC_LEARN_CONVERSION_BASE, NJ_DEFAULT_FREQUENCY_DIC_LEARN_CONVERSION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_LEARN_PREDICTION_BASE, NJ_DEFAULT_FREQUENCY_DIC_LEARN_PREDICTION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_LEARN_MORPHOLIZE_BASE, NJ_DEFAULT_FREQUENCY_DIC_LEARN_MORPHOLIZE_HIGH,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // User dictionary mount 
        internalSprintf (fileName, "%s/%s_%s_%s.a", MULTI_REWRITE_DIR, "njuserw", MULTI_LANG_DIR_EN, MULTI_LANG_DIR_EN_US);
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_USER, fileName, NULL, NULL, NULL,
                             NJ_DEFAULT_FREQUENCY_DIC_USER_CONVERSION_BASE,  NJ_DEFAULT_FREQUENCY_DIC_USER_CONVERSION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_USER_PREDICTION_BASE,  NJ_DEFAULT_FREQUENCY_DIC_USER_PREDICTION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_USER_MORPHOLIZE_BASE,  NJ_DEFAULT_FREQUENCY_DIC_USER_MORPHOLIZE_HIGH,
                             0, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // Predictive conversion dictionary mount 
        internalSprintf (fileName, "%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_EN, "njubase1.a");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_FLASH, fileName, NULL, NULL, NULL,
                             100, 400,
                             100, 560,
                             10,    0,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // Predictive conversion dictionary mount 
        internalSprintf (fileName, "%s/%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_EN, MULTI_LANG_DIR_EN_US, "njubase1us.a");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_FLASH, fileName, NULL, NULL, NULL,
                             100, 400,
                             100, 560,
                             10,    0,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // Predictive conversion dictionary mount 
        internalSprintf (fileName, "%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_EN, "njubase2.a");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_FLASH, fileName, NULL, NULL, NULL,
                             100, 400,
                             100, 560,
                             10,    0,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // Predictive conversion dictionary mount 
        internalSprintf (fileName, "%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_EN, "njubase3.a");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_FLASH, fileName, NULL, NULL, NULL,
                             100, 400,
                             100, 560,
                             10,    0,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // Extended no reading analysis dictionary mount 
        internalSprintf (fileName, "%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_EN, "njyomi.a");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_NORMAL, fileName, NULL, NULL, NULL,
                             10,    0,
                             0,   100,
                             10,    0,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;
        
        languageInfo->language = language;
        break;

    case MULTI_LANG_TYPE_EN_GB_10S:
    case MULTI_LANG_TYPE_EN_GB_FULL:
        // Rule dictionary mount 
        internalSprintf (fileName, "%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_EN, "njcon.a");
        size = internalReadBinary(&(languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION]),languageEngine, fileName);
        if (languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION] == NULL || size == 0) {
            // Rule dictionary load error 
            return -1;
        }
        languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_PREDICTION] = languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION];
        languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_MORPHOLIZE] = languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION];

        // Learning dictionary mount 
        internalSprintf (fileName, "%s/%s_%s_%s.a", MULTI_REWRITE_DIR, "njuserl", MULTI_LANG_DIR_EN, MULTI_LANG_DIR_EN_GB);
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_LEARN, fileName, NULL, NULL, NULL,
                             NJ_DEFAULT_FREQUENCY_DIC_LEARN_CONVERSION_BASE, NJ_DEFAULT_FREQUENCY_DIC_LEARN_CONVERSION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_LEARN_PREDICTION_BASE, NJ_DEFAULT_FREQUENCY_DIC_LEARN_PREDICTION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_LEARN_MORPHOLIZE_BASE, NJ_DEFAULT_FREQUENCY_DIC_LEARN_MORPHOLIZE_HIGH,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // User dictionary mount 
        internalSprintf (fileName, "%s/%s_%s_%s.a", MULTI_REWRITE_DIR, "njuserw", MULTI_LANG_DIR_EN, MULTI_LANG_DIR_EN_GB);
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_USER, fileName, NULL, NULL, NULL,
                             NJ_DEFAULT_FREQUENCY_DIC_USER_CONVERSION_BASE,  NJ_DEFAULT_FREQUENCY_DIC_USER_CONVERSION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_USER_PREDICTION_BASE,  NJ_DEFAULT_FREQUENCY_DIC_USER_PREDICTION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_USER_MORPHOLIZE_BASE,  NJ_DEFAULT_FREQUENCY_DIC_USER_MORPHOLIZE_HIGH,
                             0, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // Predictive conversion dictionary mount 
        internalSprintf (fileName, "%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_EN, "njubase1.a");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_FLASH, fileName, NULL, NULL, NULL,
                             100, 400,
                             100, 560,
                             10,    0,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // Predictive conversion dictionary mount 
        internalSprintf (fileName, "%s/%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_EN, MULTI_LANG_DIR_EN_GB, "njubase1gb.a");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_FLASH, fileName, NULL, NULL, NULL,
                             100, 400,
                             100, 560,
                             10,    0,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // Predictive conversion dictionary mount 
        internalSprintf (fileName, "%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_EN, "njubase2.a");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_FLASH, fileName, NULL, NULL, NULL,
                             100, 400,
                             100, 560,
                             10,    0,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // Predictive conversion dictionary mount 
        internalSprintf (fileName, "%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_EN, "njubase3.a");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_FLASH, fileName, NULL, NULL, NULL,
                             100, 400,
                             100, 560,
                             10,    0,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // Extended no reading analysis dictionary mount 
        internalSprintf (fileName, "%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_EN, "njyomi.a");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_NORMAL, fileName, NULL, NULL, NULL,
                             10,    0,
                             0,   100,
                             10,    0,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        languageInfo->dicSet.dic[dicCnt].limit = 0;
        dicCnt++;
        
        languageInfo->language = language;
        break;

    case MULTI_LANG_TYPE_FR_10S:
    case MULTI_LANG_TYPE_FR_FULL:
        internalMemset(&(languageInfo->dicSet), 0x00, sizeof(IWNN_DIC_SET));
        // Rule dictionary mount 
        internalSprintf (fileName, "%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_FR, "njcon.a");
        size = internalReadBinary( &(languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION]),languageEngine, fileName);
        if (languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION] == NULL || size == 0) {
            // Rule dictionary load error 
            return -1;
        }
        languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_PREDICTION] = languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION];
        languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_MORPHOLIZE] = languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION];

        // Learning dictionary mount 
        internalSprintf (fileName, "%s/%s_%s.a", MULTI_REWRITE_DIR, "njuserl", MULTI_LANG_DIR_FR);
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_LEARN, fileName, NULL, NULL, NULL,
                             NJ_DEFAULT_FREQUENCY_DIC_LEARN_CONVERSION_BASE, NJ_DEFAULT_FREQUENCY_DIC_LEARN_CONVERSION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_LEARN_PREDICTION_BASE, NJ_DEFAULT_FREQUENCY_DIC_LEARN_PREDICTION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_LEARN_MORPHOLIZE_BASE, NJ_DEFAULT_FREQUENCY_DIC_LEARN_MORPHOLIZE_HIGH,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // User dictionary mount 
        internalSprintf (fileName, "%s/%s_%s.a", MULTI_REWRITE_DIR, "njuserw", MULTI_LANG_DIR_FR);
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_USER, fileName, NULL, NULL, NULL,
                             NJ_DEFAULT_FREQUENCY_DIC_USER_CONVERSION_BASE,  NJ_DEFAULT_FREQUENCY_DIC_USER_CONVERSION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_USER_PREDICTION_BASE,  NJ_DEFAULT_FREQUENCY_DIC_USER_PREDICTION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_USER_MORPHOLIZE_BASE,  NJ_DEFAULT_FREQUENCY_DIC_USER_MORPHOLIZE_HIGH,
                             0, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // Predictive conversion dictionary mount 
        internalSprintf (fileName, "%s/%s/%s/%s/%s", MULTI_DIC_DIR, 
                         MULTI_LANG_DIR_FR, MULTI_LANG_DIR_FR, MULTI_REGULAR_DIR, "njubase1.a");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_FLASH, fileName, NULL, NULL, NULL,
                             100, 400,
                             100, 560,
                             10,    0,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // Predictive conversion dictionary mount 
        internalSprintf (fileName, "%s/%s/%s/%s/%s", MULTI_DIC_DIR, 
                         MULTI_LANG_DIR_FR, MULTI_LANG_DIR_FR, MULTI_REGULAR_DIR, "njubase2.a");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_FLASH, fileName, NULL, NULL, NULL,
                             100, 400,
                             100, 560,
                             10,    0,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // Predictive conversion dictionary mount 
        internalSprintf (fileName, "%s/%s/%s/%s/%s", MULTI_DIC_DIR, 
                         MULTI_LANG_DIR_FR, MULTI_LANG_DIR_FR, MULTI_REGULAR_DIR, "njubase3.a");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_FLASH, fileName, NULL, NULL, NULL,
                             100, 400,
                             100, 560,
                             10,    0,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // Extended no reading analysis dictionary mount 
        internalSprintf (fileName, "%s/%s/%s/%s/%s", MULTI_DIC_DIR, 
                         MULTI_LANG_DIR_FR, MULTI_LANG_DIR_FR, MULTI_REGULAR_DIR, "njexyomi.a");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_NORMAL, fileName, NULL, NULL, NULL,
                             10,    0,
                             0,   100,
                             10,    0,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;
        
        languageInfo->language = language;
        break;

    case MULTI_LANG_TYPE_CA_10S:
    case MULTI_LANG_TYPE_CA_FULL:    
        internalMemset(&(languageInfo->dicSet), 0x00, sizeof(IWNN_DIC_SET));
        // Rule dictionary mount 
        internalSprintf (fileName, "%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_FR, "njcon.a");
        size = internalReadBinary( &(languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION]),languageEngine, fileName);
        if (languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION] == NULL || size == 0) {
            // Rule dictionary load error 
            return -1;
        }
        languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_PREDICTION] = languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION];
        languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_MORPHOLIZE] = languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION];

        // Learning dictionary mount 
        internalSprintf (fileName, "%s/%s_%s.a", MULTI_REWRITE_DIR, "njuserl", MULTI_LANG_DIR_CA);
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_LEARN, fileName, NULL, NULL, NULL,
                             NJ_DEFAULT_FREQUENCY_DIC_LEARN_CONVERSION_BASE, NJ_DEFAULT_FREQUENCY_DIC_LEARN_CONVERSION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_LEARN_PREDICTION_BASE, NJ_DEFAULT_FREQUENCY_DIC_LEARN_PREDICTION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_LEARN_MORPHOLIZE_BASE, NJ_DEFAULT_FREQUENCY_DIC_LEARN_MORPHOLIZE_HIGH,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // User dictionary mount 
        internalSprintf (fileName, "%s/%s_%s.a", MULTI_REWRITE_DIR, "njuserw", MULTI_LANG_DIR_CA);
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_USER, fileName, NULL, NULL, NULL,
                             NJ_DEFAULT_FREQUENCY_DIC_USER_CONVERSION_BASE,  NJ_DEFAULT_FREQUENCY_DIC_USER_CONVERSION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_USER_PREDICTION_BASE,  NJ_DEFAULT_FREQUENCY_DIC_USER_PREDICTION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_USER_MORPHOLIZE_BASE,  NJ_DEFAULT_FREQUENCY_DIC_USER_MORPHOLIZE_HIGH,
                             0, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // Predictive conversion dictionary mount 
        internalSprintf (fileName, "%s/%s/%s/%s/%s", MULTI_DIC_DIR, 
                         MULTI_LANG_DIR_FR, MULTI_LANG_DIR_CA, MULTI_REGULAR_DIR, "njubase1.a");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_FLASH, fileName, NULL, NULL, NULL,
                             100, 400,
                             100, 560,
                             10,    0,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // Predictive conversion dictionary mount 
        internalSprintf (fileName, "%s/%s/%s/%s/%s", MULTI_DIC_DIR, 
                         MULTI_LANG_DIR_FR, MULTI_LANG_DIR_CA, MULTI_REGULAR_DIR, "njubase2.a");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_FLASH, fileName, NULL, NULL, NULL,
                             100, 400,
                             100, 560,
                             10,    0,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // Predictive conversion dictionary mount 
        internalSprintf (fileName, "%s/%s/%s/%s/%s", MULTI_DIC_DIR, 
                         MULTI_LANG_DIR_FR, MULTI_LANG_DIR_CA, MULTI_REGULAR_DIR, "njubase3.a");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_FLASH, fileName, NULL, NULL, NULL,
                             100, 400,
                             100, 560,
                             10,    0,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // Extended no reading analysis dictionary mount 
        internalSprintf (fileName, "%s/%s/%s/%s/%s", MULTI_DIC_DIR, 
                         MULTI_LANG_DIR_FR, MULTI_LANG_DIR_CA, MULTI_REGULAR_DIR, "njexyomi.a");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_NORMAL, fileName, NULL, NULL, NULL,
                             10,    0,
                             0,   100,
                             10,    0,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;
        languageInfo->language = language;
        break;

    case MULTI_LANG_TYPE_DE_10S:
    case MULTI_LANG_TYPE_DE_FULL:  
        internalMemset(&(languageInfo->dicSet), 0x00, sizeof(IWNN_DIC_SET));
        // Rule dictionary mount 
        internalSprintf (fileName, "%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_DE, "njcon.a");
        size = internalReadBinary(&(languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION]),languageEngine, fileName);
        if (languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION] == NULL || size == 0) {
            // Rule dictionary load error 
            return -1;
        }
        languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_PREDICTION] = languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION];
        languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_MORPHOLIZE] = languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION];

        // Learning dictionary mount 
        internalSprintf (fileName, "%s/%s_%s.a", MULTI_REWRITE_DIR, "njuserl", MULTI_LANG_DIR_DE);
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_LEARN, fileName, NULL, NULL, NULL,
                             NJ_DEFAULT_FREQUENCY_DIC_LEARN_CONVERSION_BASE, NJ_DEFAULT_FREQUENCY_DIC_LEARN_CONVERSION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_LEARN_PREDICTION_BASE, NJ_DEFAULT_FREQUENCY_DIC_LEARN_PREDICTION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_LEARN_MORPHOLIZE_BASE, NJ_DEFAULT_FREQUENCY_DIC_LEARN_MORPHOLIZE_HIGH,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // User dictionary mount 
        internalSprintf (fileName, "%s/%s_%s.a", MULTI_REWRITE_DIR, "njuserw", MULTI_LANG_DIR_DE);
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_USER, fileName, NULL, NULL, NULL,
                             NJ_DEFAULT_FREQUENCY_DIC_USER_CONVERSION_BASE,  NJ_DEFAULT_FREQUENCY_DIC_USER_CONVERSION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_USER_PREDICTION_BASE,  NJ_DEFAULT_FREQUENCY_DIC_USER_PREDICTION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_USER_MORPHOLIZE_BASE,  NJ_DEFAULT_FREQUENCY_DIC_USER_MORPHOLIZE_HIGH,
                             0, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // Predictive conversion dictionary mount 
        internalSprintf (fileName, "%s/%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_DE, MULTI_REGULAR_DIR, "njubase1.a");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_FLASH, fileName, NULL, NULL, NULL,
                             100, 400,
                             100, 560,
                             10,    0,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // Predictive conversion dictionary mount 
        internalSprintf (fileName, "%s/%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_DE, MULTI_REGULAR_DIR, "njubase2.a");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_FLASH, fileName, NULL, NULL, NULL,
                             100, 400,
                             100, 560,
                             10,    0,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // Predictive conversion dictionary mount 
        internalSprintf (fileName, "%s/%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_DE, MULTI_REGULAR_DIR, "njubase3.a");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_FLASH, fileName, NULL, NULL, NULL,
                             100, 400,
                             100, 560,
                             10,    0,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // Extended no reading analysis dictionary mount 
        internalSprintf (fileName, "%s/%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_DE, MULTI_REGULAR_DIR, "njexyomi.a");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_NORMAL, fileName, NULL, NULL, NULL,
                             10,    0,
                             0,   100,
                             10,    0,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;
        
        languageInfo->language = language;
        break;

    case MULTI_LANG_TYPE_ES_10S:
    case MULTI_LANG_TYPE_ES_FULL: 
        internalMemset(&(languageInfo->dicSet), 0x00, sizeof(IWNN_DIC_SET));
        // Rule dictionary mount 
        internalSprintf (fileName, "%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_ES, "njcon.a");
        size = internalReadBinary(&(languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION]),languageEngine, fileName);
        if (languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION] == NULL || size == 0) {
            // Rule dictionary load error 
            return -1;
        }
        languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_PREDICTION] = languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION];
        languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_MORPHOLIZE] = languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION];

        // Learning dictionary mount 
        internalSprintf (fileName, "%s/%s_%s.a", MULTI_REWRITE_DIR, "njuserl", MULTI_LANG_DIR_ES);
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_LEARN, fileName, NULL, NULL, NULL,
                             NJ_DEFAULT_FREQUENCY_DIC_LEARN_CONVERSION_BASE, NJ_DEFAULT_FREQUENCY_DIC_LEARN_CONVERSION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_LEARN_PREDICTION_BASE, NJ_DEFAULT_FREQUENCY_DIC_LEARN_PREDICTION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_LEARN_MORPHOLIZE_BASE, NJ_DEFAULT_FREQUENCY_DIC_LEARN_MORPHOLIZE_HIGH,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // User dictionary mount 
        internalSprintf (fileName, "%s/%s_%s.a", MULTI_REWRITE_DIR, "njuserw", MULTI_LANG_DIR_ES);
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_USER, fileName, NULL, NULL, NULL,
                             NJ_DEFAULT_FREQUENCY_DIC_USER_CONVERSION_BASE,  NJ_DEFAULT_FREQUENCY_DIC_USER_CONVERSION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_USER_PREDICTION_BASE,  NJ_DEFAULT_FREQUENCY_DIC_USER_PREDICTION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_USER_MORPHOLIZE_BASE,  NJ_DEFAULT_FREQUENCY_DIC_USER_MORPHOLIZE_HIGH,
                             0, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // Predictive conversion dictionary mount 
        internalSprintf (fileName, "%s/%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_ES, MULTI_LANG_DIR_ES, "njubase1.a");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_FLASH, fileName, NULL, NULL, NULL,
                             100, 400,
                             100, 560,
                             10,    0,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // Predictive conversion dictionary mount 
        internalSprintf (fileName, "%s/%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_ES, MULTI_LANG_DIR_ES, "njubase2.a");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_FLASH, fileName, NULL, NULL, NULL,
                             100, 400,
                             100, 560,
                             10,    0,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // Predictive conversion dictionary mount 
        internalSprintf (fileName, "%s/%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_ES, MULTI_LANG_DIR_ES, "njubase3.a");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_FLASH, fileName, NULL, NULL, NULL,
                             100, 400,
                             100, 560,
                             10,    0,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // Extended no reading analysis dictionary mount 
        internalSprintf (fileName, "%s/%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_ES, MULTI_LANG_DIR_ES, "njexyomi.a");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_NORMAL, fileName, NULL, NULL, NULL,
                             10,    0,
                             0,   100,
                             10,    0,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;
        
        languageInfo->language = language;
        break;
    case MULTI_LANG_TYPE_PT_PT_10S:
    case MULTI_LANG_TYPE_PT_PT_FULL:
        internalMemset(&(languageInfo->dicSet), 0x00, sizeof(IWNN_DIC_SET));
        // Rule dictionary mount 
        internalSprintf (fileName, "%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_PT, "njcon.a");
        size = internalReadBinary(&(languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION]),languageEngine, fileName);
        if (languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION] == NULL || size == 0) {
            // Rule dictionary load error 
            return -1;
        }
        languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_PREDICTION] = languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION];
        languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_MORPHOLIZE] = languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION];

        // Learning dictionary mount 
        internalSprintf (fileName, "%s/%s_%s_%s.a", MULTI_REWRITE_DIR, "njuserl", MULTI_LANG_DIR_PT, MULTI_LANG_DIR_PT_PT);
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_LEARN, fileName, NULL, NULL, NULL,
                             NJ_DEFAULT_FREQUENCY_DIC_LEARN_CONVERSION_BASE, NJ_DEFAULT_FREQUENCY_DIC_LEARN_CONVERSION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_LEARN_PREDICTION_BASE, NJ_DEFAULT_FREQUENCY_DIC_LEARN_PREDICTION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_LEARN_MORPHOLIZE_BASE, NJ_DEFAULT_FREQUENCY_DIC_LEARN_MORPHOLIZE_HIGH,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // User dictionary mount 
        internalSprintf (fileName, "%s/%s_%s_%s.a", MULTI_REWRITE_DIR, "njuserw", MULTI_LANG_DIR_PT, MULTI_LANG_DIR_PT_PT);
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_USER, fileName, NULL, NULL, NULL,
                             NJ_DEFAULT_FREQUENCY_DIC_USER_CONVERSION_BASE,  NJ_DEFAULT_FREQUENCY_DIC_USER_CONVERSION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_USER_PREDICTION_BASE,  NJ_DEFAULT_FREQUENCY_DIC_USER_PREDICTION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_USER_MORPHOLIZE_BASE,  NJ_DEFAULT_FREQUENCY_DIC_USER_MORPHOLIZE_HIGH,
                             0, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // Predictive conversion dictionary mount 
        internalSprintf (fileName, "%s/%s/%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_PT, 
                         MULTI_LANG_DIR_PT_PT, MULTI_REGULAR_DIR, "njubase1.a");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_FLASH, fileName, NULL, NULL, NULL,
                             100, 400,
                             100, 560,
                             10,    0,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // Predictive conversion dictionary mount 
        internalSprintf (fileName, "%s/%s/%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_PT, 
                         MULTI_LANG_DIR_PT_PT, MULTI_REGULAR_DIR, "njubase2.a");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_FLASH, fileName, NULL, NULL, NULL,
                             100, 400,
                             100, 560,
                             10,    0,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // Predictive conversion dictionary mount 
        internalSprintf (fileName, "%s/%s/%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_PT, 
                         MULTI_LANG_DIR_PT_PT, MULTI_REGULAR_DIR, "njubase3.a");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_FLASH, fileName, NULL, NULL, NULL,
                             100, 400,
                             100, 560,
                             10,    0,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // Extended no reading analysis dictionary mount 
        internalSprintf (fileName, "%s/%s/%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_PT, 
                         MULTI_LANG_DIR_PT_PT, MULTI_REGULAR_DIR, "njexyomi.a");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_NORMAL, fileName, NULL, NULL, NULL,
                             10,    0,
                             0,   100,
                             10,    0,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;
        languageInfo->language = language;
        break;

    case MULTI_LANG_TYPE_PT_BR_10S:
    case MULTI_LANG_TYPE_PT_BR_FULL:
        internalMemset(&(languageInfo->dicSet), 0x00, sizeof(IWNN_DIC_SET));
        // Rule dictionary mount 
        internalSprintf (fileName, "%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_PT, "njcon.a");
        size = internalReadBinary(&(languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION]),languageEngine, fileName);
        if (languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION] == NULL || size == 0) {
            // Rule dictionary load error 
            return -1;
        }
        languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_PREDICTION] = languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION];
        languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_MORPHOLIZE] = languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION];

        // Learning dictionary mount 
        internalSprintf (fileName, "%s/%s_%s_%s.a", MULTI_REWRITE_DIR, "njuserl", MULTI_LANG_DIR_PT, MULTI_LANG_DIR_PT_BR);
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_LEARN, fileName, NULL, NULL, NULL,
                             NJ_DEFAULT_FREQUENCY_DIC_LEARN_CONVERSION_BASE, NJ_DEFAULT_FREQUENCY_DIC_LEARN_CONVERSION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_LEARN_PREDICTION_BASE, NJ_DEFAULT_FREQUENCY_DIC_LEARN_PREDICTION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_LEARN_MORPHOLIZE_BASE, NJ_DEFAULT_FREQUENCY_DIC_LEARN_MORPHOLIZE_HIGH,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // User dictionary mount 
        internalSprintf (fileName, "%s/%s_%s_%s.a", MULTI_REWRITE_DIR, "njuserw", MULTI_LANG_DIR_PT, MULTI_LANG_DIR_PT_BR);
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_USER, fileName, NULL, NULL, NULL,
                             NJ_DEFAULT_FREQUENCY_DIC_USER_CONVERSION_BASE,  NJ_DEFAULT_FREQUENCY_DIC_USER_CONVERSION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_USER_PREDICTION_BASE,  NJ_DEFAULT_FREQUENCY_DIC_USER_PREDICTION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_USER_MORPHOLIZE_BASE,  NJ_DEFAULT_FREQUENCY_DIC_USER_MORPHOLIZE_HIGH,
                             0, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // Predictive conversion dictionary mount 
        internalSprintf (fileName, "%s/%s/%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_PT,
                          MULTI_LANG_DIR_PT_BR, MULTI_REGULAR_DIR, "njubase1.a");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_FLASH, fileName, NULL, NULL, NULL,
                             100, 400,
                             100, 560,
                             10,    0,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // Predictive conversion dictionary mount 
        internalSprintf (fileName, "%s/%s/%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_PT,
                         MULTI_LANG_DIR_PT_BR, MULTI_REGULAR_DIR, "njubase2.a");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_FLASH, fileName, NULL, NULL, NULL,
                             100, 400,
                             100, 560,
                             10,    0,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // Predictive conversion dictionary mount 
        internalSprintf (fileName, "%s/%s/%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_PT, 
                         MULTI_LANG_DIR_PT_BR, MULTI_REGULAR_DIR, "njubase3.a");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_FLASH, fileName, NULL, NULL, NULL,
                             100, 400,
                             100, 560,
                             10,    0,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // Extended no reading analysis dictionary mount 
        internalSprintf (fileName, "%s/%s/%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_PT, 
                         MULTI_LANG_DIR_PT_BR, MULTI_REGULAR_DIR, "njexyomi.a");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_NORMAL, fileName, NULL, NULL, NULL,
                             10,    0,
                             0,   100,
                             10,    0,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;
        languageInfo->language = language;
        break;

    case MULTI_LANG_TYPE_IT_10S:
    case MULTI_LANG_TYPE_IT_FULL:
        internalMemset(&(languageInfo->dicSet), 0x00, sizeof(IWNN_DIC_SET));
        // Rule dictionary mount 
        internalSprintf (fileName, "%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_IT, "njcon.a");
        size = internalReadBinary( &(languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION]),languageEngine, fileName);
        if (languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION] == NULL || size == 0) {
            // Rule dictionary load error 
            return -1;
        }
        languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_PREDICTION] = languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION];
        languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_MORPHOLIZE] = languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION];

        // Learning dictionary mount 
        internalSprintf (fileName, "%s/%s_%s.a", MULTI_REWRITE_DIR, "njuserl", MULTI_LANG_DIR_IT);
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_LEARN, fileName, NULL, NULL, NULL,
                             NJ_DEFAULT_FREQUENCY_DIC_LEARN_CONVERSION_BASE, NJ_DEFAULT_FREQUENCY_DIC_LEARN_CONVERSION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_LEARN_PREDICTION_BASE, NJ_DEFAULT_FREQUENCY_DIC_LEARN_PREDICTION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_LEARN_MORPHOLIZE_BASE, NJ_DEFAULT_FREQUENCY_DIC_LEARN_MORPHOLIZE_HIGH,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // User dictionary mount 
        internalSprintf (fileName, "%s/%s_%s.a", MULTI_REWRITE_DIR, "njuserw", MULTI_LANG_DIR_IT);
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_USER, fileName, NULL, NULL, NULL,
                             NJ_DEFAULT_FREQUENCY_DIC_USER_CONVERSION_BASE,  NJ_DEFAULT_FREQUENCY_DIC_USER_CONVERSION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_USER_PREDICTION_BASE,  NJ_DEFAULT_FREQUENCY_DIC_USER_PREDICTION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_USER_MORPHOLIZE_BASE,  NJ_DEFAULT_FREQUENCY_DIC_USER_MORPHOLIZE_HIGH,
                             0, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // Predictive conversion dictionary mount 
        internalSprintf (fileName, "%s/%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_IT, MULTI_LANG_DIR_IT, "njubase1.a");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_FLASH, fileName, NULL, NULL, NULL,
                             100, 400,
                             100, 560,
                             10,    0,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // Predictive conversion dictionary mount 
        internalSprintf (fileName, "%s/%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_IT, MULTI_LANG_DIR_IT, "njubase2.a");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_FLASH, fileName, NULL, NULL, NULL,
                             100, 400,
                             100, 560,
                             10,    0,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // Predictive conversion dictionary mount 
        internalSprintf (fileName, "%s/%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_IT, MULTI_LANG_DIR_IT, "njubase3.a");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_FLASH, fileName, NULL, NULL, NULL,
                             100, 400,
                             100, 560,
                             10,    0,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // Extended no reading analysis dictionary mount 
        internalSprintf (fileName, "%s/%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_IT, MULTI_LANG_DIR_IT, "njexyomi.a");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_NORMAL, fileName, NULL, NULL, NULL,
                             10,    0,
                             0,   100,
                             10,    0,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;
        languageInfo->language = language;
        break;

    case MULTI_LANG_TYPE_RU_10S:
    case MULTI_LANG_TYPE_RU_FULL:
        internalMemset(&(languageInfo->dicSet), 0x00, sizeof(IWNN_DIC_SET));
        // Rule dictionary mount 
        internalSprintf (fileName, "%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_RU, "njcon.a");
        size = internalReadBinary(&(languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION]),languageEngine, fileName);
        if (languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION] == NULL || size == 0) {
            // Rule dictionary load error 
            return -1;
        }
        languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_PREDICTION] = languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION];
        languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_MORPHOLIZE] = languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION];

        // Learning dictionary mount 
        internalSprintf (fileName, "%s/%s_%s.a", MULTI_REWRITE_DIR, "njuserl", MULTI_LANG_DIR_RU);
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_LEARN, fileName, NULL, NULL, NULL,
                             NJ_DEFAULT_FREQUENCY_DIC_LEARN_CONVERSION_BASE, NJ_DEFAULT_FREQUENCY_DIC_LEARN_CONVERSION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_LEARN_PREDICTION_BASE, NJ_DEFAULT_FREQUENCY_DIC_LEARN_PREDICTION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_LEARN_MORPHOLIZE_BASE, NJ_DEFAULT_FREQUENCY_DIC_LEARN_MORPHOLIZE_HIGH,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // User dictionary mount 
        internalSprintf (fileName, "%s/%s_%s.a", MULTI_REWRITE_DIR, "njuserw", MULTI_LANG_DIR_RU);
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_USER, fileName, NULL, NULL, NULL,
                             NJ_DEFAULT_FREQUENCY_DIC_USER_CONVERSION_BASE,  NJ_DEFAULT_FREQUENCY_DIC_USER_CONVERSION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_USER_PREDICTION_BASE,  NJ_DEFAULT_FREQUENCY_DIC_USER_PREDICTION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_USER_MORPHOLIZE_BASE,  NJ_DEFAULT_FREQUENCY_DIC_USER_MORPHOLIZE_HIGH,
                             0, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // Predictive conversion dictionary mount 
        internalSprintf (fileName, "%s/%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_RU, MULTI_REGULAR_DIR, "njubase1.a");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_FLASH, fileName, NULL, NULL, NULL,
                             100, 400,
                             100, 560,
                             10,    0,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // Predictive conversion dictionary mount 
        internalSprintf (fileName, "%s/%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_RU, MULTI_REGULAR_DIR, "njubase2.a");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_FLASH, fileName, NULL, NULL, NULL,
                             100, 400,
                             100, 560,
                             10,    0,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // Predictive conversion dictionary mount 
        internalSprintf (fileName, "%s/%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_RU, MULTI_REGULAR_DIR, "njubase3.a");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_FLASH, fileName, NULL, NULL, NULL,
                             100, 400,
                             100, 560,
                             10,    0,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // Extended no reading analysis dictionary mount 
        internalSprintf (fileName, "%s/%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_RU, MULTI_REGULAR_DIR, "njexyomi.a");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_NORMAL, fileName, NULL, NULL, NULL,
                             10,    0,
                             0,   100,
                             10,    0,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;
        languageInfo->language = language;
        break;

    case MULTI_LANG_TYPE_NL_10S:
    case MULTI_LANG_TYPE_NL_FULL:    // Dutch (FullKey)
        internalMemset(&(languageInfo->dicSet), 0x00, sizeof(IWNN_DIC_SET));
        // Rule dictionary mount 
        internalSprintf (fileName, "%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_NL, "njcon.a");
        size = internalReadBinary(&(languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION]),languageEngine, fileName);
        if (languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION] == NULL || size == 0) {
            // Rule dictionary load error 
            return -1;
        }
        languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_PREDICTION] = languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION];
        languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_MORPHOLIZE] = languageInfo->dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION];

        // Learning dictionary mount 
        internalSprintf (fileName, "%s/%s_%s.a", MULTI_REWRITE_DIR, "njuserl", MULTI_LANG_DIR_NL);
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_LEARN, fileName, NULL, NULL, NULL,
                             NJ_DEFAULT_FREQUENCY_DIC_LEARN_CONVERSION_BASE, NJ_DEFAULT_FREQUENCY_DIC_LEARN_CONVERSION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_LEARN_PREDICTION_BASE, NJ_DEFAULT_FREQUENCY_DIC_LEARN_PREDICTION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_LEARN_MORPHOLIZE_BASE, NJ_DEFAULT_FREQUENCY_DIC_LEARN_MORPHOLIZE_HIGH,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // User dictionary mount 
        internalSprintf (fileName, "%s/%s_%s.a", MULTI_REWRITE_DIR, "njuserw", MULTI_LANG_DIR_NL);
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_USER, fileName, NULL, NULL, NULL,
                             NJ_DEFAULT_FREQUENCY_DIC_USER_CONVERSION_BASE,  NJ_DEFAULT_FREQUENCY_DIC_USER_CONVERSION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_USER_PREDICTION_BASE,  NJ_DEFAULT_FREQUENCY_DIC_USER_PREDICTION_HIGH,
                             NJ_DEFAULT_FREQUENCY_DIC_USER_MORPHOLIZE_BASE,  NJ_DEFAULT_FREQUENCY_DIC_USER_MORPHOLIZE_HIGH,
                             0, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // Predictive conversion dictionary mount 
        internalSprintf (fileName, "%s/%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_NL, MULTI_LANG_DIR_NL, "njubase1.a");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_FLASH, fileName, NULL, NULL, NULL,
                             100, 400,
                             100, 560,
                             10,    0,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // Predictive conversion dictionary mount 
        internalSprintf (fileName, "%s/%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_NL, MULTI_LANG_DIR_NL, "njubase2.a");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_FLASH, fileName, NULL, NULL, NULL,
                             100, 400,
                             100, 560,
                             10,    0,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;

        // Predictive conversion dictionary mount 
        internalSprintf (fileName, "%s/%s/%s/%s", MULTI_DIC_DIR, MULTI_LANG_DIR_NL, MULTI_LANG_DIR_NL, "njubase3.a");
        if(internalLoadDictionary (languageEngine, languageInfo, MULTI_DIC_TYPE_FLASH, fileName, NULL, NULL, NULL,
                             100, 400,
                             100, 560,
                             10,    0,
                             1, 0, dicCnt) < 0){
            return -1;
        }
        dicCnt++;
        languageInfo->language = language;
        break;

    default:       
        break;
    }
    return 0;
}


/*
 

 
  

 
 

 
 
 */
s16 freeLanguage(u8 *languageEngineClass, u8 language)
{
    u8  i, j, cnt;
    MULTI_CLASS     *languageEngine;
    MULTI_LANG_INFO *languageInfo = NULL;
    MULTI_LANG_INFO *languageInfoSame[MULTI_LANG_TYPE_SAME_MAX];
    IWNN_DIC_HANDLE   saveRuleHandle = NULL;

    if (languageEngineClass == NULL) {
        return -1;
    }
    // Cast to MULTI_CLASS type 
    languageEngine = (MULTI_CLASS *)languageEngineClass;

    /*
     * If the current language and language to be released are the same
     * Clear the current language
     */
    if (languageEngine->language == language) {
        internalMemset(&(languageEngine->dicSet), 0x00, sizeof(IWNN_DIC_SET));
    }

    // Region for searching language types used with the same dictionary 
    for (i = 0; i < MULTI_LANG_TYPE_SAME_MAX; i++) {
        languageInfoSame[i] = NULL;
    }
    
    // Search for the language to be released and languages used by the same dictionary
    cnt = 0;
    switch (language) {
    case MULTI_LANG_TYPE_JP_FULL:
        for (i = 0; i < MULTI_LANG_TYPE_USE_MAX; i++) {
            if (languageEngine->languageInfo[i].language == language) {
                languageInfo = &(languageEngine->languageInfo[i]);
            } else if ((languageEngine->languageInfo[i].language != language) &&
                       (languageEngine->languageInfo[i].language == MULTI_LANG_TYPE_JP_FULL)) {
                languageInfoSame[cnt] = &(languageEngine->languageInfo[i]);
                cnt++;
            }
        }
        break;

    case MULTI_LANG_TYPE_EN_US_10S:
    case MULTI_LANG_TYPE_EN_US_FULL:
        for (i = 0; i < MULTI_LANG_TYPE_USE_MAX; i++) {
            if (languageEngine->languageInfo[i].language == language) {
                languageInfo = &(languageEngine->languageInfo[i]);
            } else if ((languageEngine->languageInfo[i].language != language) &&
                       (languageEngine->languageInfo[i].language == MULTI_LANG_TYPE_EN_US_FULL)) {
                languageInfoSame[cnt] = &(languageEngine->languageInfo[i]);
                cnt++;
            }
        }
        break;

    case MULTI_LANG_TYPE_EN_GB_10S:
    case MULTI_LANG_TYPE_EN_GB_FULL:
        for (i = 0; i < MULTI_LANG_TYPE_USE_MAX; i++) {
            if (languageEngine->languageInfo[i].language == language) {
                languageInfo = &(languageEngine->languageInfo[i]);
            } else if ((languageEngine->languageInfo[i].language != language) &&
                       (languageEngine->languageInfo[i].language == MULTI_LANG_TYPE_EN_GB_FULL)) {
                languageInfoSame[cnt] = &(languageEngine->languageInfo[i]);
                cnt++;
            }
        }
        break;

    case MULTI_LANG_TYPE_CN_10S:
    case MULTI_LANG_TYPE_CN_FULL:
        for (i = 0; i < MULTI_LANG_TYPE_USE_MAX; i++) {
            if (languageEngine->languageInfo[i].language == language) {
                languageInfo = &(languageEngine->languageInfo[i]);
            } else if ((languageEngine->languageInfo[i].language != language) &&
                       (languageEngine->languageInfo[i].language == MULTI_LANG_TYPE_CN_FULL)) {
                languageInfoSame[cnt] = &(languageEngine->languageInfo[i]);
                cnt++;
            }
        }
        break;

    case MULTI_LANG_TYPE_TW_P10S:
    case MULTI_LANG_TYPE_TW_PFULL:
        for (i = 0; i < MULTI_LANG_TYPE_USE_MAX; i++) {
            if (languageEngine->languageInfo[i].language == language) {
                languageInfo = &(languageEngine->languageInfo[i]);
            } else if ((languageEngine->languageInfo[i].language != language) &&
                       (languageEngine->languageInfo[i].language == MULTI_LANG_TYPE_TW_PFULL)) {
                languageInfoSame[cnt] = &(languageEngine->languageInfo[i]);
                cnt++;
            }
        }
        break;

    case MULTI_LANG_TYPE_TW_B10S:
    case MULTI_LANG_TYPE_TW_BFULL:
        for (i = 0; i < MULTI_LANG_TYPE_USE_MAX; i++) {
            if (languageEngine->languageInfo[i].language == language) {
                languageInfo = &(languageEngine->languageInfo[i]);
            } else if ((languageEngine->languageInfo[i].language != language) &&
                       (languageEngine->languageInfo[i].language == MULTI_LANG_TYPE_TW_BFULL)) {
                languageInfoSame[cnt] = &(languageEngine->languageInfo[i]);
                cnt++;
            }
        }
        break;
//2011/10/11 Commented out the Cantonese dictionary because it was deleted from the package.
//    case MULTI_LANG_TYPE_HK_10S:
/*    case MULTI_LANG_TYPE_HK_FULL:
        for (i = 0; i < MULTI_LANG_TYPE_USE_MAX; i++) {
            if (languageEngine->languageInfo[i].language == language) {
                languageInfo = &(languageEngine->languageInfo[i]);
            } else if ((languageEngine->languageInfo[i].language != language) &&
                       (languageEngine->languageInfo[i].language == MULTI_LANG_TYPE_HK_FULL)) {
                languageInfoSame[cnt] = &(languageEngine->languageInfo[i]);
                cnt++;
            }
        }
        break;
*/
    case MULTI_LANG_TYPE_KO_FULL:
        for (i = 0; i < MULTI_LANG_TYPE_USE_MAX; i++) {
            if (languageEngine->languageInfo[i].language == language) {
                languageInfo = &(languageEngine->languageInfo[i]);
            } else if ((languageEngine->languageInfo[i].language != language) &&
                       (languageEngine->languageInfo[i].language == MULTI_LANG_TYPE_KO_FULL)) {
                languageInfoSame[cnt] = &(languageEngine->languageInfo[i]);
                cnt++;
            }
        }
        break;

    case MULTI_LANG_TYPE_FR_10S:
    case MULTI_LANG_TYPE_FR_FULL:
    case MULTI_LANG_TYPE_CA_FULL:
    case MULTI_LANG_TYPE_CA_10S:
    case MULTI_LANG_TYPE_DE_10S:
    case MULTI_LANG_TYPE_DE_FULL:
    case MULTI_LANG_TYPE_ES_10S:
    case MULTI_LANG_TYPE_ES_FULL:
    case MULTI_LANG_TYPE_PT_PT_10S:
    case MULTI_LANG_TYPE_PT_PT_FULL:
    case MULTI_LANG_TYPE_PT_BR_10S:
    case MULTI_LANG_TYPE_PT_BR_FULL:
    case MULTI_LANG_TYPE_IT_10S:
    case MULTI_LANG_TYPE_IT_FULL:
    case MULTI_LANG_TYPE_RU_10S:
    case MULTI_LANG_TYPE_RU_FULL:
    case MULTI_LANG_TYPE_NL_10S:
    case MULTI_LANG_TYPE_NL_FULL:    // Dutch (FullKey)
    case MULTI_LANG_TYPE_HK_CANGJIE: //2011/12/05 Added support for the Cantonese dictionary (Cangjie) package
        for (i = 0; i < MULTI_LANG_TYPE_USE_MAX; i++) {
            if (languageEngine->languageInfo[i].language == language) {
                languageInfo = &(languageEngine->languageInfo[i]);
                break;
            }
        }
        break;
        
    default:
        break;
    }

    if (languageInfo == NULL) {
        // If an attempt is made to release an unloaded language
        return 0;
    }
    
    // Release dictionaries for language to be released
    for (i = 0; i < NJ_MAX_DIC; i++) {
        if ((languageInfo->dicSet.dic[i].type == NJ_DIC_HANDLE_TYPE_NRM) &&
            (languageInfo->dicSet.dic[i].handle != NULL)) {
            j = NJ_MAX_DIC;
            for (cnt = 0; cnt < MULTI_LANG_TYPE_SAME_MAX; cnt++) {
                if (languageInfoSame[cnt] == NULL) {
                    continue;
                }
                
                for (j = 0; j < NJ_MAX_DIC; j++) {
                    if ((languageInfoSame[cnt]->dicSet.dic[j].type == NJ_DIC_HANDLE_TYPE_NRM) &&
                        (languageInfoSame[cnt]->dicSet.dic[j].handle == languageInfo->dicSet.dic[i].handle)) {
                        // If the same handle as the dictionary set scheduled for release has been set elsewhere
                        break;
                    }
                }
                if (j < NJ_MAX_DIC) {
                    // If the same dictionary handle exists
                    break;
                }
            }
            if (j == NJ_MAX_DIC) {
                /* 
                 * Could not find the same dictionary handle, so release memory.
                 * 
                 */
                internalFree((u8*)(languageEngine->memHandle), languageInfo->dicSet.dic[i].handle);
                if (languageInfo->dicSet.dic[i].searchCache != NULL) {
                    internalFree((u8*)(languageEngine->memHandle), languageInfo->dicSet.dic[i].searchCache);
                }
                if (languageInfo->dicSet.dic[i].addInfo[0] != NULL) {
                    internalFree((u8*)(languageEngine->memHandle), languageInfo->dicSet.dic[i].addInfo[0]);
                }
            }
        } else if ((languageInfo->dicSet.dic[i].type == NJ_DIC_HANDLE_TYPE_ON_FLASH) &&
                   (languageInfo->dicSet.dic[i].handle != NULL)) {
            j = NJ_MAX_DIC;
            for (cnt = 0; cnt < MULTI_LANG_TYPE_SAME_MAX; cnt++) {
                if (languageInfoSame[cnt] == NULL) {
                    continue;
                }
                
                for (j = 0; j < NJ_MAX_DIC; j++) {
                    if ((languageInfoSame[cnt]->dicSet.dic[j].type == NJ_DIC_HANDLE_TYPE_ON_FLASH) &&
                        (languageInfoSame[cnt]->dicSet.dic[j].handle == languageInfo->dicSet.dic[i].handle)) {
                        // If the same handle as the dictionary set scheduled for release has been set elsewhere
                        break;
                    }
                }
                if (j < NJ_MAX_DIC) {
                    // If the same dictionary handle exists
                    break;
                }
            }
            if (j == NJ_MAX_DIC) {
                /* 
                 * Could not find the same dictionary handle, so release memory.
                 * 
                 */
                internalFree((u8*)(languageEngine->memHandle), 
                                 ((IWNN_FLASH_DIC_INFO*)(languageInfo->dicSet.dic[i].handle))->cacheArea[0]);
                if (languageInfo->dicSet.dic[i].searchCache != NULL) {
                    internalFree((u8*)(languageEngine->memHandle), languageInfo->dicSet.dic[i].searchCache);
                }
                if (languageInfo->dicSet.dic[i].addInfo[0] != NULL) {
                    internalFree((u8*)(languageEngine->memHandle), languageInfo->dicSet.dic[i].addInfo[0]);
                }
            }
        }
        
        // Initialize dictionary settings 
        languageInfo->dicSet.dic[i].type = NJ_DIC_HANDLE_TYPE_NRM;
        languageInfo->dicSet.dic[i].handle  = NULL;
        languageInfo->dicSet.dic[i].extensionArea= NULL;
        languageInfo->dicSet.dic[i].searchCache= NULL;
    }
    // Release rule dictionary 
    for (i = 0; i < NJ_MODE_TYPE_MAX; i++) {
        if (languageInfo->dicSet.ruleHandle[i] != NULL) {
            j = NJ_MODE_TYPE_MAX;
            for (cnt = 0; cnt < MULTI_LANG_TYPE_SAME_MAX; cnt++) {
                if (languageInfoSame[cnt] == NULL) {
                    continue;
                }
                
                for (j = 0; j < NJ_MODE_TYPE_MAX; j++) {
                    if (languageInfoSame[cnt]->dicSet.ruleHandle[j] == languageInfo->dicSet.ruleHandle[i]) {
                        // If the same handle as the dictionary set scheduled for release has been set elsewhere
                        break;
                    }
                }
                if (j < NJ_MODE_TYPE_MAX) {
                    // If the same dictionary handle exists
                    break;
                }
            }
            if (j == NJ_MODE_TYPE_MAX) {
                /* 
                 * Could not find same dictionary handle, so set it such that once the rule dictionary is released, it is not released again, because there is a chance of using the same dictionary for analysis, conversion, and morphological elements.
                 * 
                 * 
                 */
                if (saveRuleHandle != languageInfo->dicSet.ruleHandle[i]) {
                    saveRuleHandle = languageInfo->dicSet.ruleHandle[i];
                    internalFree((u8*)(languageEngine->memHandle), languageInfo->dicSet.ruleHandle[i]);
                }
            }
        }
        
        // Initialize dictionary settings 
        languageInfo->dicSet.ruleHandle[i] = NULL;
    }
    languageInfo->language         = MULTI_LANG_TYPE_NONE;
    languageInfo->userDicIndex = -1;
    languageInfo->learnDicIndex= -1;
    for (j = 0; j < NJ_MAX_DIC; j++) {
        languageInfo->extareaType[j] = 0;
    }
    return 0;
}


/*
 

 
  
  

 
 

 
 
 */
s16 setLanguage(u8 *languageEngineClass, u8 language)
{
    u8  i;
    s16 ret;
    MULTI_CLASS *languageEngine;

    if (languageEngineClass == NULL) {
        return -1;
    }
    languageEngine = (MULTI_CLASS *)languageEngineClass;

    for (i = 0; i < MULTI_LANG_TYPE_USE_MAX; i++) {
        if (languageEngine->languageInfo[i].language == language) {
            break;
        }
    }
    if (i == MULTI_LANG_TYPE_USE_MAX) {
        // If an attempt is made to release a language not loaded
        return -1;
    }
    // Specify the language to use
    languageEngine->language    = language;
    languageEngine->connect     = MULTI_CONNECT_OFF;
    languageEngine->relation    = MULTI_RELATION_OFF;

    // Copy the dictionary set of the setting for the language being used 
    languageEngine->dicSet = languageEngine->languageInfo[i].dicSet;
    NJ_SET_ENVIRONMENT(languageEngine->iwnn, languageEngine->dicSet);

    // Uppercase/lowercase control 
    switch (languageEngine->language) {
    case MULTI_LANG_TYPE_EN_GB_FULL:
    case MULTI_LANG_TYPE_EN_US_FULL:
    case MULTI_LANG_TYPE_FR_FULL:
    case MULTI_LANG_TYPE_CA_FULL:
    case MULTI_LANG_TYPE_DE_FULL:
    case MULTI_LANG_TYPE_ES_FULL:
    case MULTI_LANG_TYPE_PT_PT_FULL:
    case MULTI_LANG_TYPE_PT_BR_FULL:
    case MULTI_LANG_TYPE_IT_FULL:
    case MULTI_LANG_TYPE_RU_FULL:    // Russian (FullKey)

    case MULTI_LANG_TYPE_NL_FULL:    // Dutch (FullKey)   
        languageEngine->option.autoConversionCnt  = NJ_MAX_CANDIDATE;
        languageEngine->option.extensionMode      = NJ_OPTION_FORECAST_TOP_FREQUENCY;
        languageEngine->option.phase2Filter = NULL;
        languageEngine->option.phase2Option = NULL;
        languageEngine->option.phase1Filter = (void*)internalHashFilter;
        languageEngine->option.phase1Option = (void*)languageEngine;
        break;
    default:
        languageEngine->option.autoConversionCnt  = MULTI_OPTION_MULTI_CONVERSION;
        languageEngine->option.extensionMode      = NULL;
        languageEngine->option.phase2Filter = NULL;
        languageEngine->option.phase2Option = NULL;
        languageEngine->option.phase1Filter = NULL;
        languageEngine->option.phase1Option = NULL;
        break;
    }


    // Engine initialization process 
    ret =mw::iwnn::IWNN::NjxInit(&(languageEngine->iwnn), &(languageEngine->option));
    
    // Fuzzy search string setting 
    internalSetFlexibleSearchCharset(languageEngine);
    return ret;
}


/*
 

 
 

 
 
                                      
                                      
 
 
                                      
                                      
 
 

 
 
 */
s16 getConversion(u8 *languageEngineClass, u8 mode, wchar_t *stroke, u8 devidePos, u8 strokeMin, u8 strokeMax)
{
    s16 ret = 0;
    u16 resultCnt; 
    MULTI_CLASS *languageEngine;
    wchar_t     buf[NJ_MAX_LEN+NJ_TERM_SIZE];
    

    if ((languageEngineClass == NULL)  ||
        (stroke    == NULL)) {
        // Invalid parameter 
        return MULTI_ERR_INVALID_PARAM;
    }

    // Cast to MULTI_CLASS type 
    languageEngine = (MULTI_CLASS *)languageEngineClass;

    // Operation mode 
    // Triggered by an error when status is other than operational mode 
    languageEngine->mode     = mode;
    languageEngine->candidateFlag = 0;

    // Set filter options 
    ret = mw::iwnn::IWNN::NjxSetOption(&(languageEngine->iwnn), &(languageEngine->option));
    if (ret < 0) {
        /* Error */
        return ret;
    }
    
    // Initialize candidate information 
    MULTI_SET_INIT_CONVERT  (&languageEngine->conversionInfo);
    MULTI_SET_INIT_CANDIDATE(&languageEngine->candidateInfo);

    // Delete the duplicate candidate deletion buffer 
    internalHashFilterInit(languageEngine);

    // Input character string setting 
    internalMemset(&languageEngine->inputStroke, 0x00,   sizeof(languageEngine->inputStroke));
    internalConvertInternal(languageEngine, languageEngine->inputStroke,stroke );

    // Measure the length of the input string 
    languageEngine->inputStrokeLen = internalStrlen(languageEngine->inputStroke);

    languageEngine->predictInfo.analyzeInfo.mode = NJ_NO_SINGLE_CONVERSION|NJ_RELATION_ON|NJ_NO_READING_ON;

    switch (languageEngine->mode) {
    case MULTI_MODE_PREDICTION :
        if (languageEngine->language  == MULTI_LANG_TYPE_CN_10S ||
            languageEngine->language  == MULTI_LANG_TYPE_TW_P10S
            ) {
            languageEngine->predictInfo.analyzeInfo.charMin             = 0;
            languageEngine->predictInfo.analyzeInfo.charMax             = NJ_MAX_LEN;
            languageEngine->predictInfo.analyzeInfo.forecastLearnLimit = 50;
            languageEngine->predictInfo.analyzeInfo.forecastLimit       = 100;
            resultCnt = 1;
            ret = internalGetPredictionCN(languageEngine);
            if (ret < 0) {
                /* Error */
                return ret;
            }

        } else if (languageEngine->language  == MULTI_LANG_TYPE_CN_FULL || 
                   languageEngine->language  == MULTI_LANG_TYPE_TW_PFULL ) {
            /* When using full keyboard input for simplified Chinese */
            if (internalStrlen(languageEngine->inputStroke) != 0) {
                languageEngine->predictInfo.analyzeInfo.charMin         = strokeMin;
                languageEngine->predictInfo.analyzeInfo.charMax         = strokeMax;
            } else {
                languageEngine->predictInfo.analyzeInfo.charMin         = 0;
                languageEngine->predictInfo.analyzeInfo.charMax         = NJ_MAX_LEN;
            }
            languageEngine->predictInfo.analyzeInfo.forecastLearnLimit = 50;
            languageEngine->predictInfo.analyzeInfo.forecastLimit       = 100;
            resultCnt = 1;
            ret = internalGetPredictionCNFull(languageEngine);
            if (ret < 0) {
                /* Error */
                return ret;
            }

        } else {

            
            languageEngine->predictInfo.analyzeInfo.charMin             = strokeMin;
            languageEngine->predictInfo.analyzeInfo.charMax             = strokeMax;
            languageEngine->predictInfo.analyzeInfo.forecastLearnLimit = 50;
            languageEngine->predictInfo.analyzeInfo.forecastLimit       = 100;

            if (languageEngine->inputStrokeLen == strokeMax) {
                languageEngine->predictInfo.analyzeInfo.mode |= NJ_NO_PREDICTION;
            } else {
                languageEngine->predictInfo.analyzeInfo.mode &= ~NJ_NO_PREDICTION;
            }
        
            // Uppercase/lowercase control 
            switch (languageEngine->language) {
            case MULTI_LANG_TYPE_EN_GB_FULL:
            case MULTI_LANG_TYPE_EN_US_FULL:
            case MULTI_LANG_TYPE_FR_FULL:
            case MULTI_LANG_TYPE_CA_FULL:
            case MULTI_LANG_TYPE_DE_FULL:
            case MULTI_LANG_TYPE_ES_FULL:
            case MULTI_LANG_TYPE_PT_PT_FULL:
            case MULTI_LANG_TYPE_PT_BR_FULL:
            case MULTI_LANG_TYPE_IT_FULL:
            case MULTI_LANG_TYPE_RU_FULL:    // Russian (FullKey)
            case MULTI_LANG_TYPE_NL_FULL:    // Dutch (FullKey) 
                languageEngine->predictInfo.analyzeInfo.mode |= NJ_NO_MULTI_CONVERSION | NJ_NO_SINGLE_CONVERSION;
                break;
            default:
                break;
            }
            resultCnt = NJ_MAX_CANDIDATE;
            ret = internalGetPrediction(languageEngine, resultCnt);
            if (ret < 0) {
                // Error 
                return ret;
            }
        }
        break;

    case MULTI_MODE_CONVERSION :
        languageEngine->conversionInfo.devidePos=devidePos;
        // Convert with automatic clause-parsing 
        ret = mw::iwnn::IWNN::NjxConversion(&(languageEngine->iwnn), 
                     languageEngine->candidateInfo.segment,    
                     languageEngine->inputStroke, 
                     NJ_MAX_PHRASE, 
                     languageEngine->conversionInfo.devidePos);
         
        
        if (ret < 0) {
            // Error 
            return ret;
        } else if (ret == 0) {
            languageEngine->candidateInfo.segmentCnt = 0;
            return ret;  
        } else {
            // Conversion succeeded. 
            languageEngine->candidateInfo.segmentCnt = ret; 
        }

        // Get phrase delimit locations 
        

        ret = mw::iwnn::IWNN::NjxGetStroke(buf, 
                             &languageEngine->iwnn, 
                             &languageEngine->candidateInfo.segment[0], 
                             sizeof(buf));
        if (ret < 0) {
            // Error
            return ret;
        }
        languageEngine->conversionInfo.devidePos = (u8)internalStrlen(buf);

        ret = languageEngine->candidateInfo.segmentCnt;
        break;
    
    default :
        break;
    }

    if (ret != 0) {
        // If there was some kind of candidate 
        languageEngine->candidateFlag = 1;
    }
    return ret;
}


/*
 

 
  
  
  

 
 
 
 
 

 
 
 
 */
s16 getString(u8 *languageEngineClass, wchar_t *buf, u16 segmentPos, u16 index, u16 size) {
    u16     resultCnt;      // Number of searches 
    u16     tempCnt;
    s16     candidateCnt;      // Number of candidates obtained 
    s16     ret = 1;    
    MULTI_CLASS     *languageEngine;
    MULTI_CANDIDATE      *ptrCandidate;
    MULTI_CONVERSION     *ptrConversion;
    wchar_t         tmp[NJ_MAX_RESULT_LEN+NJ_TERM_SIZE];

    if ((languageEngineClass == NULL)  ||
        (buf       == NULL) || size < sizeof(tmp) ) {
        // Invalid parameter 
        return MULTI_ERR_INVALID_PARAM;
    }
    // Cast to MULTI_CLASS type 
    languageEngine = (MULTI_CLASS *)languageEngineClass;

    if (languageEngine->candidateFlag == 0) {
        // If there is no candidate
        return 0;
    }
    
    ptrCandidate = &(languageEngine->candidateInfo);
    ptrConversion = &(languageEngine->conversionInfo);

    switch (languageEngine->mode) {
    case MULTI_MODE_PREDICTION :
        if (segmentPos != 0) {
            // Multiple phrases do not exist in prediction mode
            *buf++ = 0x00;
            *buf   = 0x00;
            return (-1);
        }
        
        if (index >= ptrCandidate->candidateCnt) {
            // Get any candidates not gotten yet
            if ((languageEngine->language == MULTI_LANG_TYPE_CN_10S) ||
                (languageEngine->language == MULTI_LANG_TYPE_CN_FULL) ||
                (languageEngine->language == MULTI_LANG_TYPE_TW_P10S) ||
                (languageEngine->language == MULTI_LANG_TYPE_TW_PFULL)) {
                return (0);
            } else {
                resultCnt = (index + 1) - ptrCandidate->candidateCnt;
                candidateCnt = internalGetPrediction(languageEngine, resultCnt);
                if (candidateCnt < 0) {
                    // Error 
                    return candidateCnt;
                }
                if (resultCnt != candidateCnt) {
                    // If the number of searches differs from the number of candidates obtained 
                    *buf++ = 0x00;
                    *buf   = 0x00;
                    return (0);
                }
            }
        }
        
        ret = mw::iwnn::IWNN::NjxGetCandidate(tmp, &languageEngine->iwnn, &ptrCandidate->candidate[index], sizeof(tmp));
        if (ret < 0) {
            // Error 
            return ret;
        }
 
        break;
        
    case MULTI_MODE_CONVERSION :
        if (index == 0) {
            // If getting phrase candidates 
            if (segmentPos < (ptrCandidate->segmentCnt - ptrConversion->fixSegmentCnt)) {
                ret = mw::iwnn::IWNN::NjxGetCandidate(tmp, &languageEngine->iwnn, &ptrCandidate->segment[segmentPos], sizeof(tmp));
                if (ret < 0) {
                    // Error 
                    return ret;
                }
            } else {
                // An invalid number of phrases was specified 
                *buf++ = 0x00;
                *buf   = 0x00;
                return (0);
            }
        } else {
            if (segmentPos != 0) {
                // Currently set so that only the first unconfirmed phrase can be obtained
                *buf++ = 0x00;
                *buf   = 0x00;
                return (-1);
            }
            
            if (index >= ptrCandidate->candidateCnt) {
                // Get any candidates not gotten yet 
                resultCnt = (index + 1) - ptrCandidate->candidateCnt;
                candidateCnt = internalGetAllCandidate(languageEngine, &tempCnt,resultCnt );
                if (candidateCnt < 0) {
                    // Error 
                    return candidateCnt;
                }
                if (resultCnt != candidateCnt) {
                    // If the number of searches differs from the number of candidates obtained 
                    *buf++ = 0x00;
                    *buf   = 0x00;
                    return (0);
                }
                ret = tempCnt;
            }
            if( mw::iwnn::IWNN::NjxGetCandidate(tmp, &languageEngine->iwnn, &ptrCandidate->candidate[index], sizeof(tmp)) < 0 ){
                return (-1);
            }
        }
        break;
                
    default:
        *buf++ = 0x00;
        *buf   = 0x00;
        return (0);
    }
    // Internal character code > External character code 
    internalConvertExternal(languageEngine,  buf,tmp);    
    return ret;
}


/*
 

 
  
  

 
 
                            
                            
                            

 
 
 */
s16 clearDic(u8 *languageEngineClass, u16 type)
{
    s16     ret;
    MULTI_CLASS     *languageEngine;

    if ((languageEngineClass == NULL) ||
        (type > MULTI_DIC_TYPE_MORPHO)) {
        // Invalid parameter 
        return MULTI_ERR_INVALID_PARAM;
    }
    // Cast to MULTI_CLASS type 
    languageEngine = (MULTI_CLASS *)languageEngineClass;

    // Dictionary initialization process 
    ret = internalCreateDic(languageEngine, type);
    if (ret < 0) {
        // Error 
        return ret;
    }
    // Dictionary check process 
    ret = internalCheckDic(languageEngine, type);
    if (ret < 0) {
        // Error 
        return ret;
    }
    return 0;
}


/*
 

 
  
  
    
  
    
    

 
 
 

 
 
 */
s16 learnString(u8 *languageEngineClass, u16 segmentPos, u16 index) {
    u8      type;
    s16     ret;
    wchar_t         buf[NJ_MAX_LEN+NJ_TERM_SIZE];    
    MULTI_CLASS     *languageEngine;
    MULTI_CANDIDATE      *ptrCandidate;

    if (languageEngineClass == NULL) {
        // Invalid parameter
        return MULTI_ERR_INVALID_PARAM;
    }

    // Cast to MULTI_CLASS type 
    languageEngine = (MULTI_CLASS *)languageEngineClass;
    ptrCandidate = &(languageEngine->candidateInfo);

    switch (languageEngine->mode) {
    case MULTI_MODE_PREDICTION :
        if (segmentPos != 0) {
            return MULTI_ERR_INVALID_PARAM;
        }
        if (index >= ptrCandidate->candidateCnt) {
            // index is greater than the number of candidates obtained 
            return (-1);
        }
        type = MULTI_STR_TYPE_CANDIDATE;
        break;
        
    case MULTI_MODE_CONVERSION :
        if (index == 0) {
            if (segmentPos != 0) {
                // Only the first phrase can be learned
                return (-1);
            }
            type = MULTI_STR_TYPE_SEGMENT;
        } else {
            if (segmentPos == 0) {
                // [For future expandability] Use this location when getting all candidates other than for the first phrase
                return (-1);
            }
            if (index >= ptrCandidate->candidateCnt) {
                // index is greater than the number of candidates obtained
                return (-1);
            }
            type = MULTI_STR_TYPE_CANDIDATE;
        }
        break;
    default :
        return (-1);
    }

    ret = internalSelect(languageEngine, index, index, type);
    
    if (ret < 0) {
        // Failed
        return ret;
    }
    // Enable associative learning and filtered searches 
    languageEngine->connect  = MULTI_CONNECT_ON;
    languageEngine->relation = MULTI_RELATION_ON;

    if ((type == MULTI_STR_TYPE_CANDIDATE) &&
        (languageEngine->mode == MULTI_MODE_PREDICTION)) {
        // Initialize candidate information 
        MULTI_SET_INIT_CONVERT  (&(languageEngine->conversionInfo));
        MULTI_SET_INIT_CANDIDATE(&(languageEngine->candidateInfo));
        ret = 0;
    } else {
        // Update confirmed phrase information 
        languageEngine->conversionInfo.fixSegmentCnt++;
        if (languageEngine->candidateInfo.segmentCnt <= languageEngine->conversionInfo.fixSegmentCnt) {
            // Confirm all phrases 

            // Initialize candidate information 
            MULTI_SET_INIT_CONVERT  (&(languageEngine->conversionInfo));
            MULTI_SET_INIT_CANDIDATE(&(languageEngine->candidateInfo));
            ret = 0;
        } else {
            // Update confirmed reading information 
    
            ret = mw::iwnn::IWNN::NjxGetStroke(buf, &(languageEngine->iwnn), 
                                       &(languageEngine->candidateInfo.segment[index]), sizeof(buf));
            
            if (ret < 0) {
                // Failed to get reading
                return ret;
            }
            // Update number of reading characters already confirmed 
            languageEngine->conversionInfo.fixStrokeLen += internalStrlen(buf);

            // Update phrase delimit positions 
            languageEngine->conversionInfo.devidePos = (u8)internalStrlen(buf);
            ret = (languageEngine->candidateInfo.segmentCnt - languageEngine->conversionInfo.fixSegmentCnt);
        }
    }
    return ret;
}


//**************************************************************************
//                            Internal Function Definitions                                  
//**************************************************************************


/*
 

 
  
  
  
  
  
  

 
 
 

 
 
 */
s32 internalReadBinary(u8 **dicdata,MULTI_CLASS *languageEngine, u8 *dicb)
{
    u8      *ptr;
    s32     size;    
    MULTI_FILE    fileStream;

    if ((dicb    == NULL) ||
        (dicdata == NULL)) {
        // Invalid parameter 
        return MULTI_ERR_INVALID_PARAM;
    }
    if (internalFopen(&fileStream, (s8 *)dicb, (s8 *)"rb") == NULL) {
        // Failed to open file
        return MULTI_ERR_FAILED_FILE_OPEN;
    }

    // Get the file size 
    size = internalFtell(&fileStream);
    if(size == 0){
        internalFclose(&fileStream);
        // NULL set for dictionary handle 
        *dicdata = NULL;
        return 0;
    }
    // Allocate enough memory for the file size 
    ptr = (u8 *)internalMallocation((u8*)(languageEngine->memHandle), size);
    if (ptr == NULL) {
        // Memory allocation failure 
        internalFclose(&fileStream);
        return MULTI_ERR_NO_MEMORY;
    }

    // Load file 
    if (internalFread(ptr, sizeof(s8), size, &fileStream) != size) {
        // Failed to load file 
        internalFree((u8*)(languageEngine->memHandle), ptr);
        internalFclose(&fileStream);
        return MULTI_ERR_FAILED_FILE_READ;
    }
    internalFclose(&fileStream);

    // Set region for dictionary handle 
    *dicdata = ptr;
    return size;
}


/*
 

 
  

 

 
 */
void internalSetFlexibleSearchCharset(MULTI_CLASS *languageEngine)
{
    u16   i;
    MULTI_CHARSET *ptrCharSet = NULL;

    // Hold only English full-keyboard fuzzy search processing 
    switch (languageEngine->language) {
    case MULTI_LANG_TYPE_EN_US_10S:
    case MULTI_LANG_TYPE_EN_GB_10S:
    case MULTI_LANG_TYPE_FR_10S:
    case MULTI_LANG_TYPE_DE_10S:
    case MULTI_LANG_TYPE_ES_10S:
    case MULTI_LANG_TYPE_PT_PT_10S:
    case MULTI_LANG_TYPE_PT_BR_10S:
    case MULTI_LANG_TYPE_IT_10S:
    case MULTI_LANG_TYPE_NL_10S:
    case MULTI_LANG_TYPE_CA_10S:
        // Set number of characters for fuzzy search 
        languageEngine->charSet.charSetCnt = MULTI_FLEXBLE_CHARSET_COM_10S;
        ptrCharSet = (MULTI_CHARSET*)MULTI_CHARSET_TABLE_COM_10S;
        break;
    case MULTI_LANG_TYPE_EN_US_FULL:
    case MULTI_LANG_TYPE_EN_GB_FULL:
        /* Set number of characters for fuzzy search */
        languageEngine->charSet.charSetCnt = MULTI_FLEXBLE_CHARSET_EN_FULL;
        ptrCharSet = (MULTI_CHARSET*)MULTI_CHARSET_TABLE_EN_FULL;
        break;
    case MULTI_LANG_TYPE_TW_P10S:
    case MULTI_LANG_TYPE_CN_10S:
        /* Set number of characters for fuzzy search */
        languageEngine->charSet.charSetCnt = MULTI_FLEXBLE_CHARSET_CN_10S;
        ptrCharSet = (MULTI_CHARSET*)MULTI_CHARSET_TABLE_CN_10S;
        break;

    case MULTI_LANG_TYPE_TW_PFULL:
    case MULTI_LANG_TYPE_CN_FULL:
        /* Set number of characters for fuzzy search */
        languageEngine->charSet.charSetCnt = MULTI_FLEXBLE_CHARSET_CN_FULL;
        ptrCharSet = NULL;
        break;

    case MULTI_LANG_TYPE_RU_FULL:     // Russian (FullKey)
        /* Set number of characters for fuzzy search */
        languageEngine->charSet.charSetCnt = MULTI_FLEXBLE_CHARSET_RU_FULL;
        ptrCharSet = (MULTI_CHARSET*)MULTI_CHARSET_TABLE_RU_FULL;
        break;
    case MULTI_LANG_TYPE_RU_10S:     // Russian (10-key single)                      
        /* Set number of characters for fuzzy search */
        languageEngine->charSet.charSetCnt = MULTI_FLEXBLE_CHARSET_RU_10S;
        ptrCharSet = (MULTI_CHARSET*)MULTI_CHARSET_TABLE_RU_10S;
        break;

    case MULTI_LANG_TYPE_FR_FULL:     // French (FullKey)
    case MULTI_LANG_TYPE_DE_FULL:     // Dutch (FullKey)
    case MULTI_LANG_TYPE_ES_FULL:     // Spanish (FullKey)
    case MULTI_LANG_TYPE_PT_PT_FULL:  // Portuguese - Portugal - (FullKey)
    case MULTI_LANG_TYPE_PT_BR_FULL:  // Portuguese - Brazil - (FullKey)
    case MULTI_LANG_TYPE_IT_FULL:     // Italian (FullKey)
    case MULTI_LANG_TYPE_NL_FULL:     // Dutch (FullKey)
    case MULTI_LANG_TYPE_CA_FULL:     // Quebecois (FullKey)
        /* Set number of characters for fuzzy search */
        languageEngine->charSet.charSetCnt = MULTI_FLEXBLE_CHARSET_LATIN_FULL;
        ptrCharSet = (MULTI_CHARSET*)MULTI_CHARSET_TABLE_LATIN_FULL;
        break;
    default:
        languageEngine->charSet.charSetCnt = 0;
        ptrCharSet = NULL;
        break;
    }
    
    if (ptrCharSet != NULL) {
        for (i = 0; i < languageEngine->charSet.charSetCnt; i++) {
            // Set character before replacement 
            languageEngine->charSet.from[i] = (wchar_t*)ptrCharSet->from;
            // Set character after replacement 
            languageEngine->charSet.to[i]   = (wchar_t*)ptrCharSet->to;
            ptrCharSet++;
        }
    }
    return;
}


/*
 

 
  
  
  

 
 

 
 
 */
s16 internalGetPrediction(MULTI_CLASS *languageEngine, u16 resultCnt)
{
    s16    ret = 0;
    s16    cnt = 0;
    wchar_t    *ptrStroke  = NULL;
    IWNN_CHARSET *ptrCharSet = NULL;
    u16   i;
    MULTI_CANDIDATE   *ptrCandidate = &(languageEngine->candidateInfo);

    // Set fuzzy search 
    if ((languageEngine->predictInfo.flexibleSearch != MULTI_FLEXIBLE_SEARCH_OFF) &&
        (languageEngine->charSet.charSetCnt != 0)){
        ptrCharSet = &(languageEngine->charSet);
    }

    // Set reading string to search for 
    if (ptrCandidate->candidateCnt == 0) {
        ptrStroke = languageEngine->inputStroke;
    }

    // Set filtered search conditions 
    if (languageEngine->relation != 0) {
        languageEngine->predictInfo.analyzeInfo.mode |= (NJ_RELATION_ON | NJ_NO_READING_ON);
    } else {
        languageEngine->predictInfo.analyzeInfo.mode &= ~NJ_RELATION_ON;
        languageEngine->predictInfo.analyzeInfo.mode &= ~NJ_NO_READING_ON;
    }

    // Start search 
    for (i = 0; i < resultCnt; i++) {
        

        ret = mw::iwnn::IWNN::NjxAnalyze( &(languageEngine->iwnn),
                           &(ptrCandidate->candidate[ptrCandidate->candidateCnt]),            
                           ptrCharSet,
                           ptrStroke,
                           &(languageEngine->predictInfo.analyzeInfo));

        if (ret < 0) {
            // Error 
            return ret;
        } else if (ret == 0) {
            // Finish getting candidates
            break;
        } else {
        }
        cnt++;
        ptrCandidate->candidateCnt++;
        ptrStroke = NULL;
    }
    return cnt;
}


/*
 

 
  
  

 
 
 

 
 
 */
s16 internalGetAllCandidate(MULTI_CLASS *languageEngine,  u16 *tempCnt,u16 resultCnt)
{
    IWNN_RESULT *ptrRelationResult = NULL;
    MULTI_CANDIDATE   *ptrCandidate = &(languageEngine->candidateInfo);
    MULTI_CONVERSION  *ptrConversion = &(languageEngine->conversionInfo);
    u16  cnt;
    u16  i;
    s16   ret;

    // Maintain number of cases before getting candidates 
    cnt = ptrCandidate->candidateCnt;

    if (ptrCandidate->candidateCnt == 0) {
        // For the first search 
        ptrRelationResult = &(ptrCandidate->segment[ptrConversion->fixSegmentCnt]);
    }
    // Get one case 
    ret = mw::iwnn::IWNN::NjxAllCandidates(&(languageEngine->iwnn),
                         &(ptrCandidate->candidate[ptrCandidate->candidateCnt]),     
                         ptrRelationResult, 
                         ptrCandidate->candidateCnt);

    if (ret <= 0) {
        // Error
        return ret;
    }
    // Get candidates 
    ptrCandidate->candidateCnt++;
    *tempCnt = ret;
    ptrRelationResult = NULL;

    // Get the total number of candidates 
    for (i = 0; i < (resultCnt-1); i++) {
        ret = mw::iwnn::IWNN::NjxAllCandidates(&(languageEngine->iwnn), 
                             &(ptrCandidate->candidate[ptrCandidate->candidateCnt]),    
                             ptrRelationResult, 
                             ptrCandidate->candidateCnt);
        if (ret <= 0) {
            return ret;
        }
        ptrCandidate->candidateCnt++;
    }
    // Number of candidates obtained 
    return (ptrCandidate->candidateCnt - cnt);
}


/*
 

 
  

 
 

 
 
 */
s16 internalCreateDic(MULTI_CLASS *languageEngine, u16 type)
{
    s16      ret = 0;
    MULTI_LANG_INFO *languageInfo;
    u16    i;

    for (i = 0; i < MULTI_LANG_TYPE_USE_MAX; i++) {
        if (languageEngine->languageInfo[i].language == languageEngine->language) {
            break;
        }
    }
    if (i == MULTI_LANG_TYPE_USE_MAX) {
        // If an attempt was made to use a language not loaded
        return -1;
    }
    languageInfo = &(languageEngine->languageInfo[i]);

    // Initialize dictionary 
    if (type == MULTI_DIC_TYPE_USER){
        // User dictionary 
        if (languageInfo->userDicIndex >= 0) {            

            ret = mw::iwnn::IWNN::NjxCreateDic(languageInfo->dicSet.dic[languageInfo->userDicIndex].handle, 
                                   &(languageEngine->iwnn), 
                                   NJ_CREATE_DIC_TYPE_USER, languageInfo->userDicSize);
        }
    } else if (type == MULTI_DIC_TYPE_LEARN) {
        // Learning dictionary 
        if (languageInfo->learnDicIndex >= 0) {
            ret = mw::iwnn::IWNN::NjxCreateDic(languageInfo->dicSet.dic[languageInfo->learnDicIndex].handle, 
                                   &(languageEngine->iwnn), 
                                   NJ_CREATE_DIC_TYPE_LEARN, languageInfo->learnDicSize);
        }
    } else {
        // Morphological analysis learning dictionary
    }
    return ret;
}


/*
 

 
  

 
 

 
 
 */
s16 internalCheckDic(MULTI_CLASS *languageEngine, u16 type)
{
    s16    ret = 0;
    MULTI_LANG_INFO *languageInfo;
    s32   i;

    for (i = 0; i < MULTI_LANG_TYPE_USE_MAX; i++) {
        if (languageEngine->languageInfo[i].language == languageEngine->language) {
            break;
        }
    }
    if (i == MULTI_LANG_TYPE_USE_MAX) {
        // If an attempt was made to use a language not loaded
        return -1;
    }
    languageInfo = &(languageEngine->languageInfo[i]);

    // Dictionary check process 
    if (type == MULTI_DIC_TYPE_USER){
        // User dictionary 
        if (languageInfo->userDicIndex >= 0) {

            ret = mw::iwnn::IWNN::NjxCheckDic(languageInfo->dicSet.dic[languageInfo->userDicIndex].handle, &(languageEngine->iwnn), NJ_DIC_HANDLE_TYPE_NRM, 
                                  MULTI_DIC_RESTORE_OFF, languageInfo->userDicSize);

        }
    } else if (type == MULTI_DIC_TYPE_LEARN) {
        // Learning dictionary 
        if (languageInfo->learnDicIndex >= 0) {

            ret = mw::iwnn::IWNN::NjxCheckDic(languageInfo->dicSet.dic[languageInfo->learnDicIndex].handle, &(languageEngine->iwnn), NJ_DIC_HANDLE_TYPE_NRM, 
                                  MULTI_DIC_RESTORE_OFF, languageInfo->learnDicSize);
        }
    } else {
        // Morphological analysis learning dictionary
    }
    return ret;
}


/*
 

 
  
  

 
 
 
 

 
 
 */
s16 internalSelect(MULTI_CLASS *languageEngine, s16 gidx, s16 ridx, u16 type)
{
    IWNN_RESULT   *learnResult;
    IWNN_RESULT   *relationResult;
    MULTI_CANDIDATE  *ptrCandidate = &(languageEngine->candidateInfo);

    if(type == 0){
        if(gidx < 0){
            learnResult = NULL;
        } else {
            learnResult = &ptrCandidate->candidate[gidx];
        }
        if(ridx < 0){
            relationResult = NULL;
        } else {
            relationResult = &ptrCandidate->candidate[ridx];
        }
    } else {
        if(gidx < 0){
            learnResult = NULL;
        } else {
            learnResult = &ptrCandidate->segment[gidx];
        }
        if(ridx < 0){
            relationResult = NULL;
        } else {
            relationResult = &ptrCandidate->segment[ridx];
        }
    }
    return mw::iwnn::IWNN::NjxSelect(&(languageEngine->iwnn), learnResult, relationResult, languageEngine->connect);

}


/*
 

 
 
 

 
 
 
 
 */
s32 internalConvertInternal(MULTI_CLASS *languageEngine, wchar_t *out, wchar_t *in) {
    u8  *outPtr = (u8*)out;
    u8  *inPtr  = (u8*)in;
    u8  checkFlag = 0;
    u32 i;
    s32 lop;
    s32 checkCnt = 19;
    u8  checkTable[] = { 0x04, 0x06, 0x0C, 0x0E, 0x18, 
                           0x1A, 0x41, 0x43, 0x47, 0x52, 
                           0x58, 0x5A, 0x60, 0x64, 0x6E, 
                           0x78, 0x79, 0x7B, 0x7D };

    internalStrcpy((wchar_t*)outPtr,(wchar_t*)inPtr);


    // Uppercase/lowercase control 
    switch (languageEngine->language) {
    case MULTI_LANG_TYPE_EN_GB_FULL:
    case MULTI_LANG_TYPE_EN_US_FULL:
    case MULTI_LANG_TYPE_FR_FULL:
    case MULTI_LANG_TYPE_CA_FULL:
    case MULTI_LANG_TYPE_DE_FULL:
    case MULTI_LANG_TYPE_ES_FULL:
    case MULTI_LANG_TYPE_PT_PT_FULL:
    case MULTI_LANG_TYPE_PT_BR_FULL:
    case MULTI_LANG_TYPE_IT_FULL:
    case MULTI_LANG_TYPE_RU_FULL:    // Russian (FullKey)
    case MULTI_LANG_TYPE_NL_FULL:    // Dutch (FullKey) 
        languageEngine->caps = MULTI_STRING_UPPER_NONE;
        outPtr = (u8*)out;
        for (i = 0; (((*(outPtr) != 0x00) || (*(outPtr+1) != 0x00)) && (i < 2)); i++) {
            if (*(outPtr) != 0x00) {
                if (*(outPtr) == 0x01) {
                    checkFlag = 0;
                    outPtr++;
                    for (lop = 0; lop < checkCnt; lop++) {
                        if (*(outPtr) == checkTable[lop]) {
                            checkFlag = 1;
                            break;
                        }
                    }
                    if (checkFlag) {
                        outPtr++;
                        checkFlag = 0;
                        continue;
                    }
                } else if (*(outPtr) == 0x04) {
                    outPtr++;
                    if (((*(outPtr) >= 0x10) && (*(outPtr) <= 0x2F)) ||
                         (*(outPtr) == 0x01)) {
                        outPtr++;
                        continue;
                    }
                }
                break;
            }
            outPtr++;
            if ((*(outPtr) >= 0x41) && (*(outPtr) <= 0x5A)) {
                outPtr++;
                continue;
            }
            
            if ((*(outPtr) >= 0xC1) && (*(outPtr) <= 0xDD)) {
                outPtr++;
                continue;
            }
            break;
        }
        if (i == 1) {
            languageEngine->caps = MULTI_STRING_UPPER_TOP;
        } else if (i == 2) {
            languageEngine->caps = MULTI_STRING_UPPER_ALL;
        }
        break;
    default:
        languageEngine->caps = MULTI_STRING_UPPER_NONE;
        break;
    }
    return 0;
}


/*
 

 
 
 

 
 
 
 
 */
s32 internalConvertExternal(MULTI_CLASS *languageEngine, wchar_t *out,wchar_t *in) {
    u8  *outPtr = (u8*)out;
    u8  *inPtr  = (u8*)in;
    u8  checkFlag = 0;    
    u32 i;
    s32 lop;
    s32 checkCnt = 18;
    u8  checkTable[] = { 0x05, 0x07, 0x0D, 0x0F, 0x19, 
                           0x1B, 0x42, 0x44, 0x48, 0x53, 
                           0x59, 0x5B, 0x61, 0x65, 0x6F, 
                           0x7A, 0x7C, 0x7E };

    // Uppercase/lowercase control 
    if (languageEngine->caps != MULTI_STRING_UPPER_NONE) {
        inPtr = (u8*)in;
        for (i = 0; (*(inPtr) != 0x00) || (*(inPtr+1) != 0x00); i++) {
            if ((languageEngine->caps == MULTI_STRING_UPPER_TOP) &&
                (i >= 1)) {
                    break;
            }

            if (*(inPtr) != 0x00) {
                if (*(inPtr) == 0x01) {
                    checkFlag = 0;
                    inPtr++;
                    for (lop = 0; lop < checkCnt; lop++) {
                        if (*(inPtr) == checkTable[lop]) {
                            checkFlag = 1;
                            break;
                        }
                    }
                    if (checkFlag) {
                        *(inPtr) -= 0x01;
                        checkFlag = 0;
                    }
                } else if (*(inPtr) == 0x04) {
                    inPtr++;
                    if ((*(inPtr) >= 0x30) && (*(inPtr) <= 0x4F)) {
                        *(inPtr) -= 0x20;
                    } else if (*(inPtr) == 0x51) {
                        *(inPtr) -= 0x50;
                    }
                } else {
                    inPtr++;
                }
            } else {
                inPtr++;
                if (((*(inPtr) >= 0x61) && (*(inPtr) <= 0x7A)) ||
                    ((*(inPtr) >= 0xE1) && (*(inPtr) <= 0xFD))) {
                    *(inPtr) -= 0x20;
                } else if (*(inPtr) == 0xFF) {
                    *(inPtr-1) = 0x01;
                    *(inPtr)   = 0x78;
                }
            }
            inPtr++;
        }
    }

    inPtr = (u8*)in;

    internalStrcpy((wchar_t*)outPtr,(wchar_t*)inPtr);
    return 0;
}


/*
 

 
  

 
 
 
 
 
 
 
 
 
 
 
 
 
 
 
 

 
 
 */
s16 internalLoadDictionary (MULTI_CLASS *languageEngine, MULTI_LANG_INFO *languageInfo,
                               u32 dicType, u8 *dicName, u8 *extensionAreaName,
                               u8  *addInfo1, u8 *addInfo2,
                               u16 conversionBase,     u16 conversionHigh,
                               u16 predictionBase,     u16 predictionHigh,
                               u16 morphoBase,     u16 morphoHigh,
                               u8  cache,     u8  limit,
                               u8  dicCnt)
{
    u8      *ptrDic;
    s16     ret;
    s32     size, addInfoSize;
    
    if (dicName == NULL || extensionAreaName != NULL) {
        return -1;
    }
   
    switch (dicType) {
    case MULTI_DIC_TYPE_USER:          // User dictionary         
        size = internalReadBinary(&(languageInfo->dicSet.dic[dicCnt].handle),languageEngine, dicName);
        if (languageInfo->dicSet.dic[dicCnt].handle == NULL) {
            // Create new region in memory for the user dictionary if loading the file failed 
            languageInfo->dicSet.dic[dicCnt].handle = (IWNN_DIC_HANDLE)internalMallocation((u8*)(languageEngine->memHandle), NJ_USER_DIC_SIZE);
            if (languageInfo->dicSet.dic[dicCnt].handle == NULL) {
                // Insufficient memory. 
                return -1;
            }
            // Initialize the user dictionary region 

            ret = mw::iwnn::IWNN::NjxCreateDic(languageInfo->dicSet.dic[dicCnt].handle, &(languageEngine->iwnn), NJ_CREATE_DIC_TYPE_USER, NJ_USER_DIC_SIZE);

            if (ret < 0) {
                // Initialization failed.
                return ret;
            }
            size = NJ_USER_DIC_SIZE;
        }
        languageInfo->userDicIndex = dicCnt;
        languageInfo->userDicSize= size;
        languageInfo->dicSet.dic[dicCnt].type = NJ_DIC_HANDLE_TYPE_NRM;
        languageInfo->dicSet.dic[dicCnt].limit = limit;
        languageInfo->dicSet.dic[dicCnt].extensionArea = NULL;
        languageInfo->dicSet.dic[dicCnt].addInfo[0] = NULL;
        languageInfo->dicSet.dic[dicCnt].addInfo[1] = NULL;
        languageInfo->dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_CONVERSION].base = conversionBase;
        languageInfo->dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_CONVERSION].high = conversionHigh;
        languageInfo->dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_PREDICTION].base = predictionBase;
        languageInfo->dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_PREDICTION].high = predictionHigh;
        languageInfo->dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_MORPHOLIZE].base = morphoBase;
        languageInfo->dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_MORPHOLIZE].high = morphoHigh;
        if (cache == 1) {
            languageInfo->dicSet.dic[dicCnt].searchCache =
                                       (NJ_SEARCH_CACHE *)internalMallocation((u8*)(languageEngine->memHandle), 
                                                                              sizeof(NJ_SEARCH_CACHE));
        } else {
            languageInfo->dicSet.dic[dicCnt].searchCache = NULL;
        }
        break;
        
    case MULTI_DIC_TYPE_LEARN:         // Learning dictionary
    case MULTI_DIC_TYPE_LEARN_ADD:     // Learning dictionary: Auxiliary information  
    //case MULTI_DIC_TYPE_MORPHO:*/    /**< Morphological analysis learning dictionary 
        size = internalReadBinary( &(languageInfo->dicSet.dic[dicCnt].handle),languageEngine, dicName);
        if (languageInfo->dicSet.dic[dicCnt].handle == NULL) {
            // Create new region in memory for the learning dictionary if loading the file failed 
            languageInfo->dicSet.dic[dicCnt].handle = (IWNN_DIC_HANDLE)internalMallocation((u8*)(languageEngine->memHandle), 
                                                                                           MULTI_LEARN_DIC_SIZE);
            if (languageInfo->dicSet.dic[dicCnt].handle == NULL) {
                // Insufficient memory.
                return -1;
            }
            // Initialize learning dictionary region 
            if (dicType == MULTI_DIC_TYPE_LEARN_ADD) {

                ret = mw::iwnn::IWNN::NjxCreateDic(languageInfo->dicSet.dic[dicCnt].handle, &(languageEngine->iwnn), NJ_CREATE_DIC_TYPE_LEARN_ADDITIONAL, 
                                           MULTI_LEARN_DIC_SIZE);

            } else {

                ret = mw::iwnn::IWNN::NjxCreateDic(languageInfo->dicSet.dic[dicCnt].handle, &(languageEngine->iwnn), NJ_CREATE_DIC_TYPE_LEARN,
                                           MULTI_LEARN_DIC_SIZE);
            }
            if (ret < 0) {
                // Initialization failed.
                return ret;
            }
            size = MULTI_LEARN_DIC_SIZE;
        }
        languageInfo->learnDicIndex = dicCnt;
        languageInfo->learnDicSize= size;
        languageInfo->dicSet.dic[dicCnt].type = NJ_DIC_HANDLE_TYPE_NRM;
        languageInfo->dicSet.dic[dicCnt].limit = limit;
        languageInfo->dicSet.dic[dicCnt].extensionArea = NULL;
        languageInfo->dicSet.dic[dicCnt].addInfo[0] = NULL;
        languageInfo->dicSet.dic[dicCnt].addInfo[1] = NULL;
        languageInfo->dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_CONVERSION].base = conversionBase;
        languageInfo->dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_CONVERSION].high = conversionHigh;
        languageInfo->dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_PREDICTION].base = predictionBase;
        languageInfo->dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_PREDICTION].high = predictionHigh;
        languageInfo->dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_MORPHOLIZE].base = morphoBase;
        languageInfo->dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_MORPHOLIZE].high = morphoHigh;
        if (cache == 1) {
            languageInfo->dicSet.dic[dicCnt].searchCache =
                                         (NJ_SEARCH_CACHE *)internalMallocation((u8*)(languageEngine->memHandle), 
                                                                                sizeof(NJ_SEARCH_CACHE));
        } else {
            languageInfo->dicSet.dic[dicCnt].searchCache = NULL;
        }
        break;
        
    case MULTI_DIC_TYPE_SINGLE_KANJI:        // Single kanji dictionary         
        size = internalReadBinary(&(languageInfo->dicSet.dic[dicCnt].handle),languageEngine, dicName);
        languageInfo->dicSet.dic[dicCnt].type = NJ_DIC_HANDLE_TYPE_NRM;
        languageInfo->dicSet.dic[dicCnt].extensionArea = NULL;
        languageInfo->dicSet.dic[dicCnt].limit = limit;
        if (addInfo1 != NULL) {
            if (internalReadBinary((u8**)&(languageInfo->dicSet.dic[dicCnt].addInfo[0]),languageEngine, addInfo1) < 0){
                languageInfo->dicSet.dic[dicCnt].addInfo[0] = NULL;
            }
        } else {
            languageInfo->dicSet.dic[dicCnt].addInfo[0] = NULL;
        }
        if (addInfo2 != NULL) {
            if (internalReadBinary((u8**)&(languageInfo->dicSet.dic[dicCnt].addInfo[1]),languageEngine, addInfo2) < 0){
                languageInfo->dicSet.dic[dicCnt].addInfo[1] = NULL;
            }
        } else {
            languageInfo->dicSet.dic[dicCnt].addInfo[1] = NULL;
        }
        languageInfo->dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_CONVERSION].base = conversionBase;
        languageInfo->dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_CONVERSION].high = conversionHigh;
        languageInfo->dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_PREDICTION].base = predictionBase;
        languageInfo->dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_PREDICTION].high = predictionHigh;
        languageInfo->dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_MORPHOLIZE].base = morphoBase;
        languageInfo->dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_MORPHOLIZE].high = morphoHigh;

        if (cache == 1) {
            languageInfo->dicSet.dic[dicCnt].searchCache = 
                                          (NJ_SEARCH_CACHE *)internalMallocation((u8*)(languageEngine->memHandle), 
                                                                                 sizeof(NJ_SEARCH_CACHE));
        } else {
            languageInfo->dicSet.dic[dicCnt].searchCache = NULL;
        }
        break;
        
    case MULTI_DIC_TYPE_FLASH:        // FLASH dictionary          
        // Load and set FLASH dictionary 
        languageInfo->fileStream[dicCnt] = new(nn::fs::FileStream);       
        if (internalFopen(languageInfo->fileStream[dicCnt], (s8 *)dicName, (s8 *)"rb") == NULL) {
            // Failed to open file 
            delete languageInfo->fileStream[dicCnt];
            languageInfo->dicSet.dic[dicCnt].handle = NULL;
        } else {
            size = mw::iwnn::IWNN::NjxGetFlashDicCacheSize(&(languageEngine->iwnn), (IWNN_FILE*)(languageInfo->fileStream[dicCnt]));

            ptrDic = NULL;
            ptrDic = (u8 *)internalMallocation((u8*)(languageEngine->memHandle), size);

            if (mw::iwnn::IWNN::NjxSetFlashDicInfo(&(languageInfo->flashDicInfo[dicCnt]), 
                                             &(languageEngine->iwnn),  (IWNN_FILE*)(languageInfo->fileStream[dicCnt]), ptrDic, size) < 0) {

                languageInfo->dicSet.dic[dicCnt].handle = NULL;
            } else {
                languageInfo->dicSet.dic[dicCnt].type   = NJ_DIC_HANDLE_TYPE_ON_FLASH;
                languageInfo->dicSet.dic[dicCnt].handle = (IWNN_DIC_HANDLE)&(languageInfo->flashDicInfo[dicCnt]);
                size = 256 * 1024;
            }
        }

        /* In this iWnn library, all of the dictionary data is expanded to memory.
           The iWnn functions that issue file access are:
            - NjxGetFlashDicCacheSize
            - NjxSetFlashDicInfo
            - NjxCheckDic
           and those functions only.
           With this program, fileStream after this will become unnecessary so, the file is closed here
        */
        internalFclose(languageInfo->fileStream[dicCnt]);
        delete languageInfo->fileStream[dicCnt];

        languageInfo->dicSet.dic[dicCnt].extensionArea = NULL;
        languageInfo->dicSet.dic[dicCnt].limit = limit;
        if (addInfo1 != NULL) {
            if ((addInfoSize = internalReadBinary( (u8**)&(languageInfo->dicSet.dic[dicCnt].addInfo[0]),
                                                   languageEngine, addInfo1)) < 0){
                languageInfo->dicSet.dic[dicCnt].addInfo[0] = NULL;
            } else {
                ret = mw::iwnn::IWNN::NjxCheckAdditionalInfo(&(languageEngine->iwnn), languageInfo->dicSet.dic[dicCnt].type, 
                                                    languageInfo->dicSet.dic[dicCnt].handle, 
                                                    languageInfo->dicSet.dic[dicCnt].addInfo[0], addInfoSize);
                if (ret != 0x01) {
                    internalFree((u8*)(languageEngine->memHandle),languageInfo->dicSet.dic[dicCnt].addInfo[0]);
                    languageInfo->dicSet.dic[dicCnt].addInfo[0] = NULL;
                }    
            }
        } else {
            languageInfo->dicSet.dic[dicCnt].addInfo[0] = NULL;
        }
        if (addInfo2 != NULL) {
            if ((addInfoSize = internalReadBinary((u8**)&(languageInfo->dicSet.dic[dicCnt].addInfo[1]),
                                                  languageEngine, addInfo2)) < 0){
                languageInfo->dicSet.dic[dicCnt].addInfo[1] = NULL;
            } else {
                ret = mw::iwnn::IWNN::NjxCheckAdditionalInfo(&(languageEngine->iwnn), languageInfo->dicSet.dic[dicCnt].type, 
                                                      languageInfo->dicSet.dic[dicCnt].handle, 
                                                      languageInfo->dicSet.dic[dicCnt].addInfo[1], addInfoSize);
                if (ret != 0x01) {
                    internalFree((u8*)(languageEngine->memHandle),languageInfo->dicSet.dic[dicCnt].addInfo[1]);
                    languageInfo->dicSet.dic[dicCnt].addInfo[1] = NULL;
                }    
            }
        } else {
            languageInfo->dicSet.dic[dicCnt].addInfo[1] = NULL;
        }
        languageInfo->dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_CONVERSION].base = conversionBase;
        languageInfo->dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_CONVERSION].high = conversionHigh;
        languageInfo->dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_PREDICTION].base = predictionBase;
        languageInfo->dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_PREDICTION].high = predictionHigh;
        languageInfo->dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_MORPHOLIZE].base = morphoBase;
        languageInfo->dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_MORPHOLIZE].high = morphoHigh;
        if (cache == 1) {
            languageInfo->dicSet.dic[dicCnt].searchCache = 
                                     (NJ_SEARCH_CACHE *)internalMallocation((u8*)(languageEngine->memHandle), 
                                                                            sizeof(NJ_SEARCH_CACHE));
        } else {
            languageInfo->dicSet.dic[dicCnt].searchCache = NULL;
        }

        break;
        
    case MULTI_DIC_TYPE_NORMAL:        // Normal dictionary
    default:
        size = internalReadBinary( &(languageInfo->dicSet.dic[dicCnt].handle),languageEngine, dicName);
        languageInfo->dicSet.dic[dicCnt].type = NJ_DIC_HANDLE_TYPE_NRM;
        languageInfo->dicSet.dic[dicCnt].extensionArea = NULL;
        languageInfo->dicSet.dic[dicCnt].limit = limit;
        if (addInfo1 != NULL) {
            if (internalReadBinary( (u8**)&(languageInfo->dicSet.dic[dicCnt].addInfo[0]),languageEngine, addInfo1) < 0){
                languageInfo->dicSet.dic[dicCnt].addInfo[0] = NULL;
            }
        } else {
            languageInfo->dicSet.dic[dicCnt].addInfo[0] = NULL;
        }
        if (addInfo2 != NULL) {
            if (internalReadBinary((u8**)&(languageInfo->dicSet.dic[dicCnt].addInfo[1]),languageEngine, addInfo2 ) < 0){
                languageInfo->dicSet.dic[dicCnt].addInfo[1] = NULL;
            }
        } else {
            languageInfo->dicSet.dic[dicCnt].addInfo[1] = NULL;
        }
        languageInfo->dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_CONVERSION].base = conversionBase;
        languageInfo->dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_CONVERSION].high = conversionHigh;
        languageInfo->dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_PREDICTION].base = predictionBase;
        languageInfo->dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_PREDICTION].high = predictionHigh;
        languageInfo->dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_MORPHOLIZE].base = morphoBase;
        languageInfo->dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_MORPHOLIZE].high = morphoHigh;
        if (cache == 1) {
            languageInfo->dicSet.dic[dicCnt].searchCache = 
                                                           (NJ_SEARCH_CACHE *)internalMallocation((u8*)(languageEngine->memHandle), 
                                                                                                          sizeof(NJ_SEARCH_CACHE));
        } else {
            languageInfo->dicSet.dic[dicCnt].searchCache = NULL;
        }
        break;
    }
    return 0;
}


/*
 

 
  
  
  

 

 
 
 */
s16 internalGetPredictionCN(MULTI_CLASS *languageEngine)
{
    s16    ret = 0;
    s16    cnt = 0, len;
    wchar_t    *pStroke  = NULL;
    IWNN_CHARSET *pCharset = NULL;
    u16   i, j;
    MULTI_CANDIDATE   *pCandidate = &(languageEngine->candidateInfo);
    wchar_t  checkChar = 0;
    wchar_t  rom1Char = 0;
    wchar_t  rom4Char = 0;
    u8 tone4Flg = 0;

    /* Set fuzzy search */
    if ((languageEngine->predictInfo.flexibleSearch != MULTI_FLEXIBLE_SEARCH_OFF) &&
        (languageEngine->charSet.charSetCnt != 0)){
        pCharset = &(languageEngine->charSet);
    }

    /* Initial settings for filter */
    languageEngine->iwnn.option_data.phase1Filter = NULL;
    languageEngine->iwnn.option_data.phase1Option = NULL;
    internalMemset(&languageEngine->simplifiedOption, 0x0, sizeof(languageEngine->simplifiedOption));

    /* Set the delimiter character (#) and four-tone information characters (I thru IV) */
    MULTI_CONV_TO_WCHAR(&checkChar, MULTI_CHAR_NUMSIGN);
    MULTI_CONV_TO_WCHAR(&rom1Char, MULTI_CHAR_ROM1);
    MULTI_CONV_TO_WCHAR(&rom4Char, MULTI_CHAR_ROM4);

    /* String copy between input strings */
    j = 0;
    for (i = 0; languageEngine->inputStroke[i] != 0x0000; i++) {
        /* If the character is the delimiter */
        if (languageEngine->inputStroke[i] == (wchar_t)checkChar) {
            /* Set the delimiter character flag */
            languageEngine->simplifiedOption.separatorPos[languageEngine->simplifiedOption.separatorFlg] = i;
            languageEngine->simplifiedOption.separatorFlg++;
        /* If the character is a four-tone information character */
        } else if ((languageEngine->inputStroke[i] >= rom1Char) && (languageEngine->inputStroke[i] <= rom4Char)) {
            /* Set the four-tone information check flag */
            tone4Flg = 1;
        /* In all other cases */
        } else {
            /* Copy the string until a delimiter character or four-tone information character is found */
            if ((languageEngine->simplifiedOption.separatorFlg == 0) || (tone4Flg == 0)) {
                languageEngine->inputStroke2[j] = languageEngine->inputStroke[i];
                j++;
            }
        }
    }
    languageEngine->simplifiedOption.strokeLen = j;
    languageEngine->simplifiedOption.separatorPos[languageEngine->simplifiedOption.separatorFlg] = internalStrlen(languageEngine->inputStroke);

    NJ_SET_ENVIRONMENT(languageEngine->iwnn, languageEngine->dicSet);
    nj_strcpy(languageEngine->inputStroke2, languageEngine->inputStroke);
    pStroke = languageEngine->inputStroke2;

    /* Set the simplified Chinese filtering function */
    languageEngine->iwnn.option_data.phase1Filter = (void*)multidicSimplified10sFilter;
    languageEngine->iwnn.option_data.phase1Option = (void*)languageEngine;
    nj_strcpy(languageEngine->simplifiedOption.stroke, languageEngine->inputStroke);
    languageEngine->simplifiedOption.strokeLen = internalStrlen(languageEngine->simplifiedOption.stroke);

    /* Set filtered search conditions */
    if (languageEngine->relation != 0) {
        languageEngine->predictInfo.analyzeInfo.mode |= (NJ_RELATION_ON | NJ_NO_READING_ON | NJ_NO_MULTI_CONVERSION | NJ_NO_SINGLE_CONVERSION | NJ_NO_ALL_CANDIDATE);
    } else {
        languageEngine->predictInfo.analyzeInfo.mode |= (NJ_NO_MULTI_CONVERSION | NJ_NO_SINGLE_CONVERSION | NJ_NO_ALL_CANDIDATE);
        languageEngine->predictInfo.analyzeInfo.mode &= ~NJ_RELATION_ON;
        languageEngine->predictInfo.analyzeInfo.mode &= ~NJ_NO_READING_ON;
    }

    cnt = 0;
    /* Start search */
    for (i = 0; i < NJ_MAX_CANDIDATE; i++) {
        ret = mw::iwnn::IWNN::NjxAnalyze( &(languageEngine->iwnn),
                           &(pCandidate->candidate[pCandidate->candidateCnt]),    
                           pCharset,
                           pStroke,
                           &(languageEngine->predictInfo.analyzeInfo));


        if (ret < 0) {
            /* Error */
            return ret;
        } else if (ret == 0) {
            /* Finish getting candidates */
            if (cnt == 0) {
                len = internalStrlen(languageEngine->inputStroke2);
                languageEngine->inputStroke2[len-1] = NJ_CHAR_NUL;
                pStroke = languageEngine->inputStroke2;
                languageEngine->iwnn.dicSet.keyword[0] = NJ_CHAR_NUL;
                continue;
            }
            break;
        } else {
        }
        cnt++;
        pCandidate->candidateCnt++;
        pStroke = NULL;
    }
    return cnt;
}

/*
 

 
  
  
  

 

 
 
 */
s16 internalGetPredictionCNFull(MULTI_CLASS *languageEngine)
{
    s16    ret = 0;
    s16    cnt = 0;
    wchar_t    *pStroke  = NULL;
    IWNN_CHARSET *pCharset = NULL;
    u16   i, j;
    MULTI_CANDIDATE   *pCandidate = &(languageEngine->candidateInfo);
    wchar_t charLargeA = 0x0041;
    wchar_t charLargeZ = 0x005A;
    wchar_t charSmall = 0x0020;
    wchar_t searchChar[NJ_MAX_LEN + NJ_TERM_SIZE];
    u8 largeCharFlg = 0;
    u16 tmpCnt = 0;
    wchar_t largeAChar = 0;
    wchar_t largeZChar = 0;
    wchar_t smallChar = 0;

    /* Variable initialization */
    internalMemset(searchChar, 0x0, sizeof(searchChar));
    internalMemset(&languageEngine->simplifiedOption, 0x0, sizeof(languageEngine->simplifiedOption));

    /* Set fuzzy search */
    if ((languageEngine->predictInfo.flexibleSearch != MULTI_FLEXIBLE_SEARCH_OFF) &&
        (languageEngine->charSet.charSetCnt != 0)){
        pCharset = &(languageEngine->charSet);
    }

    /* Set reading string to search for */
    if (pCandidate->candidateCnt == 0) {
        pStroke = languageEngine->inputStroke;
    }
    j = 0;
    MULTI_CONV_TO_WCHAR(&largeAChar, charLargeA);
    MULTI_CONV_TO_WCHAR(&largeZChar, charLargeZ);
    MULTI_CONV_TO_WCHAR(&smallChar, charSmall);
    /* Set the reading string for searching */
    for (i = 0; i < internalStrlen(languageEngine->inputStroke); i++) {
        if ((*pStroke >= largeAChar) &&
            (*pStroke <= largeZChar)) {
            searchChar[i] = *pStroke + smallChar;
            largeCharFlg = 1;
        } else {
            searchChar[i] = *pStroke;
        }
        pStroke++;
    }

    /* Set the reading string for option */
    nj_strcpy(languageEngine->simplifiedOption.stroke, searchChar);
    languageEngine->simplifiedOption.strokeLen = internalStrlen(searchChar);

    /* Set filtered search conditions */
    if (languageEngine->relation != 0) {
        languageEngine->predictInfo.analyzeInfo.mode |= (NJ_RELATION_ON | NJ_NO_READING_ON);
    } else {
        languageEngine->predictInfo.analyzeInfo.mode &= ~NJ_RELATION_ON;
        languageEngine->predictInfo.analyzeInfo.mode &= ~NJ_NO_READING_ON;
    }

    if (largeCharFlg) {
        /* Set the filtering function */
        languageEngine->iwnn.option_data.phase1Filter = (void*)multidicSimplifiedFullkeyFilter;
        languageEngine->iwnn.option_data.phase1Option = (void*)languageEngine;

        /* Crunch the reading string and search until a candidate is found */
        for (i = internalStrlen(languageEngine->inputStroke); i > 0; i--) {
            searchChar[i] = 0x0000;
            pStroke = &searchChar[0];
            tmpCnt = NJ_MAX_CANDIDATE - cnt;

            /* Set filtered search conditions */
            if (languageEngine->relation != 0) {
                languageEngine->predictInfo.analyzeInfo.mode |= (NJ_RELATION_ON | NJ_NO_READING_ON | NJ_NO_MULTI_CONVERSION | NJ_NO_SINGLE_CONVERSION | NJ_NO_ALL_CANDIDATE);
            } else {
                languageEngine->predictInfo.analyzeInfo.mode &= ~NJ_RELATION_ON;
                languageEngine->predictInfo.analyzeInfo.mode &= ~NJ_NO_READING_ON;
                languageEngine->predictInfo.analyzeInfo.mode |= (NJ_NO_MULTI_CONVERSION | NJ_NO_SINGLE_CONVERSION | NJ_NO_ALL_CANDIDATE);
            }

            for (j = 0; j < tmpCnt; j++) {
                ret = mw::iwnn::IWNN::NjxAnalyze( &(languageEngine->iwnn),
                                   &(pCandidate->candidate[pCandidate->candidateCnt]),    
                                   pCharset,
                                   pStroke,
                                   &(languageEngine->predictInfo.analyzeInfo));
                if (ret < 0) {
                    /* Error */
                    return ret;
                } else if (ret == 0) {
                    break;
                } else {
                }
                cnt++;
                pCandidate->candidateCnt++;
                pStroke = NULL;
            }
            /* Exit search if a candidate is found. */
            if (cnt > 0) {
                break;
            }
        }
    } else {
        /* Set reading string to search for */
        if (pCandidate->candidateCnt == 0) {
            pStroke = languageEngine->inputStroke;
        }

        /* Start search */
        for (i = 0; i < NJ_MAX_CANDIDATE; i++) {
            ret = mw::iwnn::IWNN::NjxAnalyze( &(languageEngine->iwnn),
                           &(pCandidate->candidate[pCandidate->candidateCnt]),    
                           pCharset,
                           pStroke,
                           &(languageEngine->predictInfo.analyzeInfo));
            if (ret < 0) {
                /* Error */
                return ret;
            } else if (ret == 0) {
                /* Finish getting candidates */
                break;
            } else {
            }
            cnt++;
            pCandidate->candidateCnt++;
            pStroke = NULL;
        }
    }
    return cnt;
}

