/*-----------------------------------------------------------------------*
 Project: iWnn API Search/Add Word Demo: Main Function
 File:    main.cpp

 Copyright (C)2009-2012 Nintendo Co., Ltd.  All rights reserved.

 These coded instructions, statements, and computer programs contain
 proprietary information of Nintendo of America Inc. and/or Nintendo
 Company Ltd., and are protected by Federal copyright law.  They may
 not be disclosed to third parties or copied or duplicated in any form,
 in whole or in part, without the prior written consent of Nintendo.


 $Revision: $
 *-----------------------------------------------------------------------*/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <nn/os.h>
#include <nn/dbg.h>
#include <nn/fs.h>
#include <nn/fnd.h>
#include <nn/applet.h>
#include <mw/iwnn/iwnnCTR.h>
#include "fprintf.h"

#define DEMO_PARAM_ERR -1    //Argument error
#define DEMO_SIZE_OVER -2    //Size error


s32 readDic( u8 **dicData,char *dicName);
s32 loadString(wchar_t* ptrString, char *fileName,s32 stringSize);
s16 demoSearchAddWord( IWNN_CLASS *environment, wchar_t *reading,wchar_t *candidate,wchar_t * additional);
void releaseDic(void);

wchar_t *wcharStringCopy(wchar_t *dst, wchar_t *src);
u16 wcharStringLen(wchar_t *c);

wchar_t        inputCandidate[NJ_MAX_RESULT_LEN+NJ_TERM_SIZE];     //Candidate string storage region
wchar_t        inputReading[NJ_MAX_LEN+NJ_TERM_SIZE];              //Reading string storage region
wchar_t        inputAdditional[NJ_MAX_ADDITIONAL_LEN+NJ_TERM_SIZE]; //Additional information storage region
IWNN_CLASS      iwnnClass;    // iWnn class structure definition
IWNN_DIC_SET  dicSet;                                   //Dictionary set structure
s16    dicCnt;                                   //Mounted dictionary count
IWNN_FLASH_DIC_INFO    flashDicInfo[NJ_MAX_DIC];       //FLASH type dictionary information storage region

nn::fnd::ExpHeap            dicHeap;         //Instance for dynamic memory allocation
nn::fs::FileOutputStream*   logFile=NULL;    //Log file output instance
nn::fs::FileStream*    inFileUbase1=NULL;    //Instance for working with Integrated Dictionary File 1.
nn::fs::FileStream*    inFileUbase2=NULL;    //Instance for working with Integrated Dictionary File 2.
u8* ptrUbase1=NULL;                          //Integrated Dictionary 1 handle pointer
u8* ptrUbase2=NULL;                          //Integrated Dictionary 2 handle pointer





/*
 
               
               
               
               
               

 

 
 */
extern "C" int nnMain()
{
    s16    ret = 0;
    s32 fileSize;
    u8     bom[2]={0xff,0xfe};


    //Enabling the applet library
    nn::applet::Enable();

    //fs initialization
    nn::fs::Initialize();

    //SD Card initialization
    nn::Result result = nn::fs::MountSdmc();
    if( result.IsFailure() ){
        return -1;
    }    

    //Result output file (Result2.txt) initialization
    logFile = new (nn::fs::FileOutputStream);
    if(!logFile){
        nn::fs::Unmount("sdmc:");
        return -1;
    }

    logFile->Initialize(FPRINTF_FILE_PATH,true);
    //Write BOM symbol
    logFile->Write(bom,sizeof(bom),true);
    Fprintf(logFile,L"demo start!\n");

    //Initialize the heap region
    dicHeap.Initialize(nn::os::GetDeviceMemoryAddress(), nn::os::GetDeviceMemorySize(), nn::os::ALLOCATE_OPTION_LINEAR);

    dicCnt = 0;
    memset(&iwnnClass, 0x00, sizeof(IWNN_CLASS));
    memset(&dicSet, 0x00, sizeof(IWNN_DIC_SET));
    memset(&flashDicInfo, 0x00, (NJ_MAX_DIC) * sizeof(IWNN_FLASH_DIC_INFO));        


    /********************************/
    /* 1. Dictionary set structure settings       */
    /********************************/
    NN_LOG("  iWnn Demo: Mounting dictionaries.\n");
    // User dictionary        
    dicSet.dic[dicCnt].handle = (IWNN_DIC_HANDLE)dicHeap.Allocate(NJ_USER2_DIC_SIZE);
    if (dicSet.dic[dicCnt].handle == NULL) {
        Fprintf(logFile, L"error:user-dic malloc\n");
        logFile->Finalize();
        nn::fs::Unmount("sdmc:");
        delete logFile;
        dicHeap.Finalize();
        return -1;
    }
    //Create user dictionary (with additional information)
    ret = mw::iwnn::IWNN::NjxCreateDic(dicSet.dic[dicCnt].handle, &iwnnClass, NJ_CREATE_DIC_TYPE_USER_ADDITIONAL, NJ_USER2_DIC_SIZE);
    
    if (ret < 0) {
        releaseDic();
        Fprintf(logFile, L"error:user-dic create\n");
        logFile->Finalize();
        nn::fs::Unmount("sdmc:");
        delete logFile;
        dicHeap.Finalize();
        return -1;
    }
    dicSet.dic[dicCnt].type            = NJ_DIC_HANDLE_TYPE_NRM;
    dicSet.dic[dicCnt].limit        = 0;
    dicSet.dic[dicCnt].extensionArea        = NULL;
    dicSet.dic[dicCnt].searchCache     = NULL;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_CONVERSION].base = 501;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_CONVERSION].high = 1000;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_PREDICTION].base = 501;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_PREDICTION].high = 1000;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_MORPHOLIZE].base = 501;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_MORPHOLIZE].high = 1000;

    dicCnt++;


    //Integrated dictionary 1 (njubase1.a)
    inFileUbase1 = new(nn::fs::FileStream);
    inFileUbase1->Initialize(L"sdmc:/iWnn/dic/JA/32/njubase1.a",nn::fs::OPEN_MODE_READ);
    
    fileSize = mw::iwnn::IWNN::NjxGetFlashDicCacheSize(&iwnnClass, (IWNN_FILE*)inFileUbase1);    
    
    if(fileSize <= 0){   
        releaseDic();
        Fprintf(logFile, L"error:[%d]dic open\n", dicCnt);
        logFile->Finalize();
        nn::fs::Unmount("sdmc:");
        delete logFile;
        dicHeap.Finalize();
        return -1;
    }    
    
    ptrUbase1 = NULL;
    ptrUbase1  = (u8*)(dicHeap.Allocate(fileSize));
    if (mw::iwnn::IWNN::NjxSetFlashDicInfo(&(flashDicInfo[dicCnt]), &iwnnClass, (IWNN_FILE*)inFileUbase1, ptrUbase1, fileSize) < 0) {       
        releaseDic();
        Fprintf(logFile, L"error:[%d]dic open\n", dicCnt);
        logFile->Finalize();
        nn::fs::Unmount("sdmc:");
        delete logFile;
        dicHeap.Finalize();
        return -1;
        
    } else {
        dicSet.dic[dicCnt].handle = (IWNN_DIC_HANDLE)&(flashDicInfo[dicCnt]);
        dicSet.dic[dicCnt].addInfo[0] = NULL;
        dicSet.dic[dicCnt].addInfo[1] = NULL;
    }

    /* In this iWnn library, all of the dictionary data is expanded to memory.
       The iWnn functions that issue file access are:
        - NjxGetFlashDicCacheSize
        - NjxSetFlashDicInfo
        - NjxCheckDic
       and those functions only.
       With this program, inFileUbase1 after this will become unnecessary so, the file is closed here
    */
    inFileUbase1->Finalize();
    delete inFileUbase1;
    inFileUbase1=NULL;

    dicSet.dic[dicCnt].type                               = NJ_DIC_HANDLE_TYPE_ON_FLASH;
    dicSet.dic[dicCnt].limit                              = 0;
    dicSet.dic[dicCnt].extensionArea                           = NULL;
    dicSet.dic[dicCnt].searchCache                           = NULL;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_CONVERSION].base = 400;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_CONVERSION].high = 550;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_PREDICTION].base = 100;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_PREDICTION].high = 560;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_MORPHOLIZE].base = 0;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_MORPHOLIZE].high = 10;
    dicCnt++;
    
    //Integrated dictionary 2 (njubase2.a)
    inFileUbase2 = new(nn::fs::FileStream);
    inFileUbase2->Initialize(L"sdmc:/iWnn/dic/JA/32/njubase2.a",nn::fs::OPEN_MODE_READ);
        
    fileSize = mw::iwnn::IWNN::NjxGetFlashDicCacheSize(&iwnnClass, (IWNN_FILE*)inFileUbase2);     

    if(fileSize <= 0){ 
        releaseDic();
        Fprintf(logFile, L"error:[%d]dic open\n", dicCnt);
        logFile->Finalize();
        nn::fs::Unmount("sdmc:");
        delete logFile;
        dicHeap.Finalize();
        return -1;
    }
    
    ptrUbase2 = NULL;
    ptrUbase2 = (u8 *)dicHeap.Allocate(fileSize);
    if (mw::iwnn::IWNN::NjxSetFlashDicInfo(&(flashDicInfo[dicCnt]), &iwnnClass, (IWNN_FILE*)inFileUbase2, ptrUbase2, fileSize) < 0) {  
        releaseDic();
        Fprintf(logFile, L"error:[%d]dic open\n", dicCnt);
        logFile->Finalize();
        nn::fs::Unmount("sdmc:");
        delete logFile;
        dicHeap.Finalize();
        return -1;        
    } else {
        dicSet.dic[dicCnt].type   = NJ_DIC_HANDLE_TYPE_ON_FLASH;
        dicSet.dic[dicCnt].handle = (IWNN_DIC_HANDLE)&(flashDicInfo[dicCnt]);
        dicSet.dic[dicCnt].addInfo[0] = NULL;
        dicSet.dic[dicCnt].addInfo[1] = NULL;
    }

    /* In this iWnn library, all of the dictionary data is expanded to memory.
       The iWnn functions that issue file access are:
        - NjxGetFlashDicCacheSize
        - NjxSetFlashDicInfo
        - NjxCheckDic
       and those functions only.
       With this program, inFileUbase2 after this will become unnecessary so, the file is closed here
    */
    inFileUbase2->Finalize();
    delete inFileUbase2;
    inFileUbase2=NULL;

    dicSet.dic[dicCnt].type                               = NJ_DIC_HANDLE_TYPE_ON_FLASH;
    dicSet.dic[dicCnt].limit                              = 2;
    dicSet.dic[dicCnt].extensionArea                           = NULL;
    dicSet.dic[dicCnt].searchCache                           = NULL;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_CONVERSION].base = 400;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_CONVERSION].high = 550;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_PREDICTION].base = 100;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_PREDICTION].high = 560;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_MORPHOLIZE].base = 0;
    dicSet.dic[dicCnt].dicFrequency[NJ_MODE_TYPE_MORPHOLIZE].high = 10;

    dicCnt++;



    //Rule dictionary (njcon.a)
    if( readDic( &dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION],"sdmc:/iWnn/dic/JA/njcon.a") <= 0 ){
        releaseDic();
        Fprintf(logFile, L"error:rule dic open\n");
        logFile->Finalize();
        nn::fs::Unmount("sdmc:");
        delete logFile;
        dicHeap.Finalize();
        return -1;
    }
    dicSet.ruleHandle[NJ_MODE_TYPE_PREDICTION] = dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION];
    dicSet.ruleHandle[NJ_MODE_TYPE_MORPHOLIZE] = dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION];
    dicSet.mode  =  NJ_CACHE_MODE_NONE;

    /************************************************/
    /*2. Loading a reading string, candidate, and additional information string for word registration*/
    /************************************************/
    memset(inputReading,0,sizeof(u16)*(NJ_MAX_LEN+NJ_TERM_SIZE));
    if( loadString( inputReading,"sdmc:/iWnn/reading.txt",sizeof(wchar_t)*(NJ_MAX_LEN+NJ_TERM_SIZE)) <= 0 ){
        releaseDic();       
        Fprintf(logFile, L"error:demo reading file open\n");
        logFile->Finalize();
        nn::fs::Unmount("sdmc:");
        delete logFile;
        dicHeap.Finalize();
        return -1;
    }
    memset(inputCandidate,0,sizeof(u16)*(NJ_MAX_RESULT_LEN+NJ_TERM_SIZE));
    if( loadString( inputCandidate,"sdmc:/iWnn/candidate.txt", sizeof(wchar_t)*(NJ_MAX_RESULT_LEN+NJ_TERM_SIZE)) <= 0 ){
        releaseDic();       
        Fprintf(logFile, L"error:demo candidate file open\n");
        logFile->Finalize();
        nn::fs::Unmount("sdmc:");
        delete logFile;
        dicHeap.Finalize();
        return -1;
    }
    memset(inputAdditional,0,sizeof(u16)*(NJ_MAX_ADDITIONAL_LEN+NJ_TERM_SIZE));
    if( loadString( inputAdditional,"sdmc:/iWnn/additional.txt", sizeof(wchar_t)*(NJ_MAX_ADDITIONAL_LEN+NJ_TERM_SIZE)) <= 0 ){
        releaseDic();
        Fprintf(logFile, L"error:demo additional file open\n");
        logFile->Finalize();
        nn::fs::Unmount("sdmc:");
        delete logFile;
        dicHeap.Finalize();
        return -1;
    }
    Fprintf(logFile, L"**********************InputString in File*************************\n");
    Fprintf(logFile, L"input reading = ");
    FprintUTF16BE(logFile,inputReading);
    Fprintf(logFile, L"\n");
    Fprintf(logFile, L"input candidate = ");
    FprintUTF16BE(logFile,inputCandidate);
    Fprintf(logFile, L"\n");
    Fprintf(logFile, L"input additional = ");
    FprintUTF16BE(logFile,inputAdditional);
    Fprintf(logFile, L"\n");
    Fprintf(logFile, L"******************************************************************\n");


    /********************************/
    /* 3. iWnn engine initialization       */
    /********************************/
    NN_LOG("  iWnn Demo: Initializing iWnn engine.\n");
    mw::iwnn::IWNN::NjxInit(&iwnnClass, NULL);   
    iwnnClass.dicSet = dicSet;


    
    /********************************/
    /* 4. Word registration and search demo         */
    /********************************/    
    NN_LOG("  iWnn Demo: Search word and add word demo.\n");
    ret= demoSearchAddWord(&iwnnClass,inputReading,inputCandidate,inputAdditional);
    if( ret < 0 ){
        releaseDic();
        if(ret == DEMO_PARAM_ERR){
            Fprintf(logFile, L"error:search add demo (param error)\n");
        }else if(ret == DEMO_SIZE_OVER){
            Fprintf(logFile, L"error:search add demo (size error)\n");
        }else{
            Fprintf(logFile, L"error:search add demo (error code=0x%04x)\n", NJ_GET_ERR_CODE(ret));
        }
        
        logFile->Finalize();
        nn::fs::Unmount("sdmc:");
        delete logFile;
        dicHeap.Finalize();
        return -1;
    }
    Fprintf(logFile, L"**********************OutputString by iWnnAPI*********************\n");
    Fprintf(logFile, L"output reading = ");
    FprintUTF16BE(logFile,inputReading);
    Fprintf(logFile, L"\n");
    Fprintf(logFile, L"output candidate = ");
    FprintUTF16BE(logFile,inputCandidate);
    Fprintf(logFile, L"\n");
    Fprintf(logFile, L"output additional = ");
    FprintUTF16BE(logFile,inputAdditional);
    Fprintf(logFile, L"\n");
    Fprintf(logFile, L"******************************************************************\n");

    
    /********************************/
    /* 5. Dictionary unmount           */
    /********************************/
    NN_LOG("  iWnn Demo: Unmounting dictionaries.\n");
    releaseDic();


    Fprintf(logFile,L"demo end!\n");
    logFile->Finalize();    
    nn::fs::Unmount("sdmc:");
    delete logFile;
    dicHeap.Finalize();
    NN_LOG("  iWnn Demo: Demo end.\n");
    while(true){}
}



/*
 
               
               
               
               

 
 

 
 */
s32 readDic(u8 **dicData,char *dicName )
{         
    size_t  fileSize;

    nn::fs::FileInputStream inFile(dicName);
    fileSize = inFile.GetSize();
 
    if(fileSize <= 0 ){
        Fprintf(logFile,L" Invalid file size(%d)\n", fileSize);
        inFile.Finalize();
        return -1;
    }
    
    *dicData = (IWNN_DIC_HANDLE)dicHeap.Allocate(fileSize);
    if(*dicData == NULL ){
        Fprintf(logFile,L" Failed to allocation \n");
        inFile.Finalize();
        return -1;
    }

    fileSize = (s32)inFile.Read(*dicData, fileSize);    
    inFile.Finalize();
    return(fileSize);
}
/*
 
               
               
               

 
 
 

 
 */
s32 loadString(wchar_t *ptrString,char *fileName, s32 stringSize){
    // Check arguments
    if ( ptrString == NULL ){
        Fprintf(logFile, L" parameter error (ptrString = NULL)\n");
        return -1;
    }
    nn::fs::FileInputStream inFile(fileName);
    size_t fileSize = inFile.GetSize();
    if (fileSize >= stringSize) {
        Fprintf(logFile,L" Failed to read file status (%d)\n", fileSize);
        inFile.Finalize();
        return -1;
    }
    if (fileSize <= 0) {
        Fprintf(logFile,L" Failed to read file status (%d)\n", fileSize);
        inFile.Finalize();
        return -1;
    }

    fileSize = inFile.Read(ptrString, stringSize);
    inFile.Finalize();
    return(fileSize);
}
/*
 
               
               
               

 

 
 */
void releaseDic(void){
    u16 i;
    if(ptrUbase1 != NULL){
        dicHeap.Free(ptrUbase1);        
        dicSet.dic[1].handle=NULL;
        ptrUbase1 = NULL;
    }
    if(ptrUbase2 != NULL){
        dicHeap.Free(ptrUbase2);
        dicSet.dic[2].handle=NULL;
        ptrUbase2 = NULL;
    }

    for(i = 0; i < dicCnt;i++){
        if(dicSet.dic[i].handle != NULL &&
           dicSet.dic[i].type== NJ_DIC_HANDLE_TYPE_NRM){
            dicHeap.Free(dicSet.dic[i].handle);
        }
    }
    if( dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION] != NULL){
        dicHeap.Free(dicSet.ruleHandle[NJ_MODE_TYPE_CONVERSION]);
    }
    if(inFileUbase1!=NULL){
        inFileUbase1->Finalize();
        delete inFileUbase1;
    }    

    if(inFileUbase2!=NULL){
        inFileUbase2->Finalize();
        delete inFileUbase2;
    }
}

/*
 
               
               
               
               
               

 
 
 
 

 
 */
s16 demoSearchAddWord( IWNN_CLASS *environment, wchar_t *reading,wchar_t *candidate,wchar_t * additional){
    u16 stringLen;
    IWNN_WORD_INFO wordInfo;
    s16 ret;
    IWNN_CURSOR searchCursor;        // Dictionary search cursor 
    IWNN_RESULT result;

    memset(&wordInfo,0,sizeof(wordInfo));
    // Check arguments 
    if ((environment == NULL)||(reading == NULL) || (candidate==NULL)) {
        return DEMO_PARAM_ERR;
    }


    stringLen = wcharStringLen(reading);
    if(stringLen > NJ_MAX_LEN){
        return DEMO_SIZE_OVER;
    }
    //Set reading in word information structure
    wcharStringCopy(wordInfo.reading,reading);
    wordInfo.stem.readingLen=stringLen;
    wordInfo.stem.partsOfSpeech=NJ_PARTS_OF_SPEECH_NOUN;//Register as noun

    stringLen = wcharStringLen(candidate);
    if(stringLen > NJ_MAX_RESULT_LEN){
        return DEMO_SIZE_OVER;
    }
    //Set candidate in word information structure
    wcharStringCopy(wordInfo.candidate,candidate);
    wordInfo.stem.candidateLen=stringLen;

    stringLen = wcharStringLen(additional);
    if(stringLen > NJ_MAX_ADDITIONAL_LEN){
        return DEMO_SIZE_OVER;
    }
    //Set additional information in word information structure
    wcharStringCopy(wordInfo.additional,additional);

    //Register in user dictionary
    ret = mw::iwnn::IWNN::NjxAddWord(environment,&wordInfo,0,0);
    if(ret < 0){
        return ret;
    }
    Fprintf(logFile,L"njx_add_word() success\n");

    //Search dictionary
    memset(&searchCursor,0,sizeof(searchCursor));
    searchCursor.condition.dicSet = &(environment->dicSet);    //Set dictionary set 
    searchCursor.condition.reading = reading;            //Set search reading   
    searchCursor.condition.kanji = candidate;            //Set search candidate 
    searchCursor.condition.operation = NJ_CURSOR_OPERATION_FORE;// Specify forward matching search 
    searchCursor.condition.mode = NJ_CURSOR_MODE_READING;//Get reading order

    ret = mw::iwnn::IWNN::NjxSearchWord(&searchCursor,environment);
    if(ret < 0){
        return ret;
    }
    Fprintf(logFile,L"njx_search_word() success\n");

    //Get reading string, candidate, and additional information string from search results
    while(1){
        ret = mw::iwnn::IWNN::NjxGetWord(&searchCursor,&result,environment);
        if(ret > 0){
            Fprintf(logFile,L"njx_get_word() success\n");

            //Get reading
            ret = mw::iwnn::IWNN::NjxGetStroke(reading,environment, &result,  sizeof(wchar_t)*(NJ_MAX_LEN+NJ_TERM_SIZE));
            if(ret < 0){
                return ret;
            }
            Fprintf(logFile,L"njx_get_stroke() success\n");

            //Get candidate
            ret = mw::iwnn::IWNN::NjxGetCandidate(candidate,environment, &result, sizeof(wchar_t)*(NJ_MAX_RESULT_LEN+NJ_TERM_SIZE) );
            if(ret < 0){
                return ret;
            }
            Fprintf(logFile,L"njx_get_candidate() success\n");

            //Get additional information string
            ret = mw::iwnn::IWNN::NjxGetAdditionalInfo(additional,environment,&result,0,sizeof(wchar_t)*(NJ_MAX_ADDITIONAL_LEN+NJ_TERM_SIZE));
            if(ret < 0){
                return ret;
            }
            Fprintf(logFile,L"njx_get_additional_info() success\n");

            //Delete registered word from user dictionary
            ret = mw::iwnn::IWNN::NjxDeleteWord(environment,&result);
            if(ret < 0){
                return ret;
            }
            Fprintf(logFile,L"njx_delete_word() success\n");
        }else if(ret == 0){
            break;
        }else{
            return ret;
        }
    }
    return 0;

}

/*
 

 

 
 
 

 
 */
u16 wcharStringLen(wchar_t *c) {

    u16 count = 0;

    while (*c++ != NULL) {
        count++;
    }
    return count;
}

/*
 

 

 
 
 
 
 

 
 */
wchar_t *wcharStringCopy(wchar_t *dst, wchar_t *src) {

    wchar_t *ret = dst;

    while (*src != NULL) {
        *dst++ = *src++;
    }
    *dst = *src;
    return ret;
}
